! ESA (C) 2000-2018 
!
! This file is part of ESA's XMM-Newton Scientific Analysis System (SAS).
!
!    SAS is free software: you can redistribute it and/or modify
!    it under the terms of the GNU General Public License as published by
!    the Free Software Foundation, either version 3 of the License, or
!    (at your option) any later version.
!
!    SAS is distributed in the hope that it will be useful,
!    but WITHOUT ANY WARRANTY; without even the implied warranty of
!    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
!    GNU General Public License for more details.
!
!    You should have received a copy of the GNU General Public License
!    along with SAS.  If not, see <http://www.gnu.org/licenses/>.
module addflags

use dal
use sla
use sasdal
use caloaldefs
use cal
use omconstants_mod
use omsourceparam_mod
use omsourceflags_mod







integer(int32), parameter, private               :: int32NullValue = -1      
integer(int8), parameter, private                :: int8NullValue = 127      
integer(int16), parameter, private               :: int16NullValue = 32767      



public

contains


!*********************************************************************
!
!
!*********************************************************************
subroutine createoutputlist



     

    call copyDataSet( trim( stringParameter("set") ), trim( stringParameter("outset") ))



end subroutine createoutputlist
!*********************************************************************
!
!
!*********************************************************************


subroutine addSourceFlags(nSources, source1)
    integer, intent(in) :: nsources
    type(sourceParameters) :: source1(1 : nSources)    

    type(dataSetT) :: dataSetH
    type(tableT) :: tableH
    
    type(sourceParameters), pointer :: source(:) => null()
    integer :: position, i, j
    character(100), pointer :: flagText(:) =>null()
    allocate(source(1 : nsources))

    source(1 : nsources) = source1(1 : nsources)    



!   write(*, *) "INITIAL FLAGS"
!    do i = 1, nsources
!        write(*, *) i, ") x=", source(i)%xpos, " y=", source(i)%ypos, " qflag=", source(i)%qualityflag
!    end do





!***********************************************************************
! ADDED ON 15th OCTOBER
!***********************************************************************




 
    source%extensionflag = 0

    call ReadOutStreaks(nSources,               &
                  source%rawCountRate,          &
                  source%xpos,                  &
                  source%extensionFlag,         &
                  source%qualityFlag)

  






    !***********************************************************************
    ! Sources near to a bright source
    !**********************************************************************


    call BrightSourceProximityFlags(nSources, source%rawCountRate, &
                source%xpos, source%ypos, source%qualityFlag)
 

    !***********************************************************************
    ! Sources within a smoke ring
    !**********************************************************************




    call SmokeRings(nSources, source%rawCountRate, source%xpos, source%ypos, &
               source%qualityFlag)



    !***********************************************************************
    ! Sources on a star diffraction spike
    !*********************************************************************
!    source%theta = source%theta - pascx - 90.0
    where(source%theta .lt. 0.0) source%theta = source%theta + 180.0


    call StarSpikes( nSources, source, source%xpos, source%ypos)
 


    !***********************************************************************
    ! Sources on an edge
    !**********************************************************************
    call EdgeSources( nSources, source)



    !***********************************************************************
    ! Sources on a corner
    !**********************************************************************
    call setCornerFlags(source%xpos, source%ypos, source, int(nSources, int16))



!    write(*, *) "Line 135"


    !**************************************************************************************************
    ! Quality flag from combined-source list
    !**************************************************************************************************





    dataSetH = dataSet( trim( stringParameter("outset") ), modify)

    tableH = table(dataSetH, "SRCLIST")



 !  write(*, *) "Line 152"



    if(hasColumn(tableH, "QFLAG_COMB")) then
        position = columnNumber(tableH, "QFLAG_COMB")
        call deleteColumn(tableH, "QFLAG_COMB")    

        call AddInt16Column(tableH, source%qualityFlag, nsources, &
               "QFLAG_COMB", "", "Quality flag using combined source-list", &
                int(int16nullValue, int32), position=position)

   else

        position = columnNumber(tableH, "QFLAG")
        call deleteColumn(tableH, "QFLAG")    

       call AddInt16Column(tableH, source%qualityFlag, nsources, &
               "QFLAG", "", "Quality flag using combined source-list", &
                int(int16nullValue, int32), position=position)

    end if
   

 !  write(*, *) "Line 176"


    call addcomment(tableH, "QFLAG- bits set (value refers to integer value, colour refers to region colour)")
 



 

  
    call addcomment(tableH, "if QFLAG=0 then region colour will be either")
    call addcomment(tableH, "green (point-source) or cyan (extended source)")

    do i = 1, getNoQflags()

        call addcomment(tableH, trim(getQFlagText(i)) )

    end do

   

    !************************************************************************
    ! Convert into a a text array
    !************************************************************************
    allocate(flagText(1 : nsources))
    flagText = ""
    do i = 1, nsources
        do j = 1, getNoQflags()
            if ( btest(source(i)%qualityFlag, j - 1)) then
                flagText(i)(j : j) = '1'
            else
                flagText(i)(j : j) = '0'
            end if
        end do
    end do
     
    

    !***************************************************
    ! 
    !***************************************************
    position = position + 1

!   write(*, *) "Line 220"


    call AddCharColumn(tableH, "QFLAG_BITS", "", "Quality flag using combined source-list", &
         nsources, flagtext, position=position + 1)

    call addcomment(tableH, "0=flag not set, 1=flag set")











   if(hasColumn(tableH, "CFLAG_MAX")) then
        position = columnNumber(tableH, "CFLAG_MAX")
        call deleteColumn(tableH, "CFLAG_MAX")    
  

        call AddInt8Column(tableH, source%confusionFlag, nsources, &
               "CFLAG_MAX", "", "Confusion flag using combined source-list", &
                int(int8nullValue, int32), position=position)


    else

        position = columnNumber(tableH, "CFLAG")
        call deleteColumn(tableH, "CFLAG")    
  

        call AddInt8Column(tableH, source%confusionFlag, nsources, &
               "CFLAG", "", "Confusion flag using combined source-list", &
                int(int8nullValue, int32), position=position)


    end if


!   write(*, *) "Line 262"



    call addcomment(tableH, "CFLAG- bits set (colour refers to region colour")


!   write(*, *) "Line 269"

    do i = 1, getNoCflags()

        call addcomment(tableH, trim(getCFlagText(i)) )

    end do


!   write(*, *) "Line 278"



    !************************************************************************
    ! Convert into a a text array
    !************************************************************************
    flagText = ""

    do i = 1, nsources
        do j = 1, getNoCflags()
            if ( btest(source(i)%confusionFlag, j - 1)) then
                flagText(i)(j : j) = '1'
            else
                flagText(i)(j : j) = '0'
            end if
        end do
    end do
     
    

    !***************************************************
    ! 
    !***************************************************
  
 
!   write(*, *) "Line 304"



    call AddCharColumn(tableH, "CFLAG_BITS", "", "Confusion flag using combined source-list", &
         nsources, flagtext, position=position+1)

    call addcomment(tableH, "0=flag not set, 1=flag set")





!   write(*, *) "Line 317"






    call release(datasetH)   

     deallocate(source, flagText)

!  write(*, *) "Line 328"


end subroutine addSourceFlags


end module addflags
