! ESA (C) 2000-2018 
!
! This file is part of ESA's XMM-Newton Scientific Analysis System (SAS).
!
!    SAS is free software: you can redistribute it and/or modify
!    it under the terms of the GNU General Public License as published by
!    the Free Software Foundation, either version 3 of the License, or
!    (at your option) any later version.
!
!    SAS is distributed in the hope that it will be useful,
!    but WITHOUT ANY WARRANTY; without even the implied warranty of
!    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
!    GNU General Public License for more details.
!
!    You should have received a copy of the GNU General Public License
!    along with SAS.  If not, see <http://www.gnu.org/licenses/>.
!**************************************************************
! January 2002         
! Compares OM photometry with WFU calibration data%expI
!***************************************************************
module checkoffsets_mod

    
   use param
   use cal
   use dal
   use sla

   use errorHandling  
   use omallocation_mod
  
   use omgraph_mod  
   use omnumerics_mod
   use statistics_mod
   use omutility_mod
 
   implicit none



  
type AstrometryData
   
    real(single) :: raOffset = 0.0, raOffsetErr = 0.0
    real(single) :: decOffset = 0.0, decOffsetErr = 0.0
    real(single) :: rotation = 0.0, rotationError = 0.0
    real(single) :: rmsResidual = 0.0
    integer :: noSources = 0
    integer :: noMatches = 0
    real(single) :: JD = 0.0
  
    real(single) :: raPoint = 0.0, decPoint = 0.0
    real(single) :: glongPoint = 0.0, glatPoint = 0.0
    logical :: okay

end type AstrometryData
 




contains
!*********************************************************
!
!********************************************************
subroutine checkoffsets
    type(datasetT) :: dataSetH
    type(TableT) :: tableH
    type(AstrometryData) :: data1, data2
    real(single) :: deltaRa, deltaDec

    datasetH = dataSet( trim( stringParameter("set1") ), READ)
    tableH = Table( dataSetH, "SRCLIST", READ)

   
    call loadoffsets( tableH, data1)

    
    call release( datasetH )
  
    datasetH = dataSet( trim( stringParameter("set2") ), READ)
    tableH = Table( dataSetH, "SRCLIST", READ)

   
    call loadoffsets( tableH, data2)

    
    call release( datasetH )
  
    !************************************
    ! Compare the offsets
    !************************************
    deltaRa = abs( data1%raOffset -  data2%raOffset)
    deltaDec = abs( data1%decOffset -  data2%decOffset)

    if ( data1%okay .and. data2%okay ) then



        write(*, *) "sourcelist file 1=", trim( stringParameter("set1") )
        write(*, *) "Computed offsets-"
        write(*, '(a, f6.2, a, f6.2)') " raoffset=", data1%raOffset, "+/-", data1%raoffsetErr
        write(*, '(a, f6.2, a, f6.2)') " decoffset=", data1%decOffset, "+/-", data1%decoffsetErr
        write(*, *) "sourcelist file 2=", trim( stringParameter("set2") )
        write(*, *) "Computed offsets-"
        write(*, '(a, f6.2, a, f6.2)') "raoffset=", data2%raOffset, "+/-", data2%raoffsetErr
        write(*, '(a, f6.2, a, f6.2)') " decoffset=", data2%decOffset, "+/-", data2%decoffsetErr
        write(*,'(a, f6.2, a, f6.2)') "Abs Delta ra difference=", deltaRa, " delta dec=", deltaDec

        if ( deltaRa .gt. 1.0 .or. deltaDec .gt. 1.0) then
            call fatal("bug")
        end if
    end if



end subroutine checkoffsets

!**************************************************************************
!
!*************************************************************************
subroutine loadoffsets( tableH, data)
    type(TableT) :: tableH
    type(AstrometryData), intent(out) :: data


    if ( hasAttribute( tableH, "RAOFFSET" ) )then

        data%raOffset = real32attribute( tableH, " RAOFFSET")
        data%raoffseterr = real32attribute( tableH, "RAOFFERR")
        data%decoffset = real32attribute( tableH, "DEOFFSET")
        data%decoffseterr =  real32attribute( tableH, "DEOFFERR")
        data%rmsresidual = real32attribute( tableH, "RMSRESID")   
        data%nosources =  int32attribute( tableH, "NMATCHES")
        data%okay =  booleanattribute( tableH, "POSCOROK")


    else
        call fatal("bug at line 194")

    end if   


end subroutine loadoffsets

end module checkoffsets_mod















