// ESA (C) 2000-2018 
//
// This file is part of ESA's XMM-Newton Scientific Analysis System (SAS).
//
//    SAS is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    SAS is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with SAS.  If not, see <http://www.gnu.org/licenses/>.
// -*-C++-*-
//
// File:      omsource.cc
// Author:    Jorgo Bakker (jbakker@astro.estec.esa.nl)
// Generated: Fri Aug  9 17:05:36 MEST 2002
#include "imagefilechecker.h"
#include <iostream>
#include "Dal.h"
#include <sstream>
#include<fstream>
#include <iomanip>
#include "sourcelist.h"
#include "OmImage.h"
#include "allocate.h"
#include "mode.h"

void checkImage(); 





void checkImageStatistics( float **image, int nx, int ny, string filter, float exposure);


bool  fileStored(string &fileName, string &filter);   

void imagefilechecker()
{
  


    string fileName = stringParameter( "imageset");
   




    // Open the file
    DataSet *dataSet = dataSetServer->open(fileName, Dal::Read);




    string filter = dataSet->attribute( "FILTER" )->asString();   





    if ( (filter.find("GRI", 0) != string::npos) || (filter.find("Gri", 0) != string::npos) ||  (filter.find("gri", 0) != string::npos))
    {

    

       dataSetServer->close(dataSet); // Finished with the file


       exit(0);;

    }






    string dataMode = dataSet->attribute( "DATAMODE" )->asString();   



    dataSetServer->close(dataSet); // Finished with the file





    if ( (dataMode.find("FAST", 0) == string::npos) &&  (dataMode.find("IMAGING", 0) == string::npos) &&
	 (dataMode.find("fast", 0) == string::npos) &&  (dataMode.find("imaging", 0) == string::npos) )

    {

      exit(0);


    }


    cout << "Line 89\n";


    /************************************************************************
    * Check to see if file stores
    ************************************************************************/
    if (!fileStored(fileName, filter))   
        checkImage();
    
 
};

/*************************************************************
 *
 *************************************************************/
void checkImage() 
{     



    int nx = 0, ny = 0;


    string imageFile = stringParameter("imageset");

    cout << "Checking image " << imageFile << endl;

    DataSet *dataSet = dataSetServer->open(imageFile, Dal::Read);


    string filter = dataSet->attribute( "FILTER" )->asString();   



    float exposure = dataSet->attribute("EXPOSURE")->asReal();


    int xBin = dataSet->attribute("BINAX1")->asInt();
    int yBin = dataSet->attribute("BINAX2")->asInt();


    bool binBPE = dataSet->attribute( "BINBPE" )->asBool();


    xBin = (int)pow(2.0, xBin);
    yBin = (int)pow(2.0, yBin);


    if (binBPE) 
    {
        xBin *= 2;
        yBin *= 2;
    }



    string windowString = dataSet->attribute("DETSEC")->asString();







    dataSetServer->close(dataSet); // Finished with the file





 
    istringstream text(windowString);

    int windowX0, windowX1, windowY0, windowY1;

    char c;
    text >> c >> windowX0 >> c >> windowX1 >> c >> windowY0 >> c >> windowY1 >> c;
 

    windowX0 *= xBin;
    windowY0 *= yBin;









    float** image = LoadImage( (char *)imageFile.c_str(), nx, ny);


    
    checkImageStatistics( image, nx, ny, filter, exposure);


    if (image) de_allocate(&image, nx);

}

/************************************************************
*
*
************************************************************/
void checkImageStatistics( float **image, int nx, int ny, string filter, float exposure)
{




    float mean = 0.0, median = 0.0;






 
    vector<float> data;


    for (int i = 0; i < nx; i++)
      for (int j = 0; j < ny; j++)
	data.push_back( image[i][j] );

    float mode = halfRangeMode(data, mean, median);



    string imageFile = stringParameter("imageset");




    string outputfile = "imagefiles_" + filter + ".dat";
    ofstream output(outputfile.c_str(), ios::app);



     char buffer[200];
     sprintf( buffer, "%40s   %6d %6d %8.1f %6.1f %6.1f %6.1f", 
	      imageFile.c_str(), nx, ny, exposure, mean, median, mode); 
                       

     output << buffer << endl;
         
     cout << buffer << endl;
  



    output.close();
}



/************************************************************************
* Check to see if file stores
************************************************************************/
bool fileStored(string &fileName, string &filter)   
{




    string inputFile = "imagefiles_" + filter + ".dat";
    ifstream input(inputFile.c_str(), ios::app);
    if(!input) return false;


    cout << "Checking file " << inputFile << " for " << fileName << endl;

    char file[100];
    while (!input.eof())
    { 
	 input.getline(file, 100);
    
        
        cout << "file=" << file << endl;


        if ( fileName.find(file, 0) != string::npos)
        {
	  cout << "File " << fileName << "  found\n";
          input.close();
          return true;
        }

    }   




    input.close();
    return false;

}
