// ESA (C) 2000-2018 
//
// This file is part of ESA's XMM-Newton Scientific Analysis System (SAS).
//
//    SAS is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    SAS is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with SAS.  If not, see <http://www.gnu.org/licenses/>.
#include <fstream>
#include <iostream>
#include "modifyobsfile.h"
#include "Dal.h"
#include "param.h"
#include<cstring>
#include <cstdio>
#include "utility.h"
#include "sourcelist.h"
#include "qfile.h"
#include "qdir.h"


#include "OmSourceParameters.h"


/******************************************************************
* THIS PROGRAM IS FOR MODYFYING AN OBSERVATION SOURCE-LIST FILE
* FOR THE OMCATALOGUE
*
*******************************************************************/






/**************************************************************************
*
**************************************************************************/
class ImageData
{
public:
    int16 _exposure;

    double _cdelt1;  // RA plate-scale (degrees and negative)
    double _cdelt2;  // Dec plate-scale (degrees)
    double _crpix1;
    double _crpix2;
    double _crval1;
    double _crval2;
    int _nx;
    int _ny;
  vector< vector<float> > _image;

    string _name;
    ImageData() : 
        _exposure(0),
        _cdelt1(0),
        _crpix1(0),
        _crpix2(0),
        _crval1(0),
        _crval2(0),
        _nx(0),
        _ny(0),
        _name("")  
       {};


  ~ImageData() {};

};



void computeVegaMag( string filter, float rate, float rateErr, float &mag, float &magErr);



F90MODPROCPROTO(void, coordinates, computestandardcoordinates)
       (double *ra, double *dec, double *raPnt, double *decPnt, double *xi, double *eta);



F90MODPROCPROTO(void, coordinates, list)
       (int *n, double *data);

F90MODPROCPROTO(void, coordinates, list1)
       (int *n, OmSourceParameter *data);



F90MODPROCPROTO(void, addflags, addsourceflags)
       (int *n, OmSourceParameter *data);


F90MODPROCPROTO(double, coordinates, getplatescale)();


F90MODPROCPROTO(void, addflags, createoutputlist)();

F90MODPROCPROTO(void, coordinates, initialise)();


F90MODPROCPROTO(void, coordinates, addistortion)(double *, double *, double *, double *);

F90MODPROCPROTO(void, coordinates, undistortion)(double *, double *, double *, double *);

//F90MODPROCPROTO(void, coordinates, windowparameters)(char[256], int *, int *, int *, int *);
F90MODPROCPROTO(void, coordinates, windowparameters1)(unsigned char *, size_t *, int *, int *, int *, int *);

F90MODPROCPROTO( void, coordinates, convertcoords)(float *x, float*y, int *x0, int *y0, int *binx, int *biny);


F90MODPROCPROTO( void, convert_mod, computetangentplanecoords)(double *, double *, double *, double*, double *, double *);




/*************************************************************************************
*
*************************************************************************************/




void getSourceListNames( vector<string> &fileName );

void getSourceListNames( string directory, vector<string> &fileName );




windowParameters getWindowParameters( string fileName );



void loadImageData(const DataSet *dataSet, ImageData &data);



vector<QString> loadDirectoryFiles(const string directory, 
				   const int type, const string filter = "*" );




void addVegaMags( );


void getPointing(vector<string> &name, double &raPnt, double &decPnt, double &pa);

void convertCoords( windowParameters &parameters, float &x, float &y);

bool loadInputData (vector<string> &fileName, map< string,  vector< vector<OmSourceParameter> > >  &sourceList);


void convertCombinedCoords( map<string, vector< vector<OmSourceParameter> > > &sourceList,   
                            vector < OmSourceParameter > &source);


void resetflags(  vector < OmSourceParameter > &source);

void addExposures(float minExposure, float maxExpsure, int nSources);


void addExposureNumbers( int size, 
			 map<string,  vector< vector<OmSourceParameter> > > &sourceListMap);






void loadImageFiles( const string directory, 
                     const vector<string>& srcList, 
                     vector<string> &imageList );



void modifySignificances( const vector<string> &imageList);



vector<vector<float> >  loadImage(const DataSet *set, int &nx, int &ny);


bool imageHasSource(double ra, double dec, ImageData &data);

string revolutionNo = "";

vector<string> omFilter;



string leicesterDirectory;





/**********************************************************************
*  This is the main program
* 
*
**********************************************************************/
void modifyobsfile()
{

 
  
    omFilter.push_back( "UVW2" );
    omFilter.push_back( "UVM2" );
    omFilter.push_back( "UVW1" );
    omFilter.push_back( "U" );
    omFilter.push_back( "B" );
    omFilter.push_back( "V" );
    omFilter.push_back( "WHITE" );




    /********************************************************
    * Create the output source-list
    ********************************************************/


    F90MODPROC( addflags, createoutputlist )();


    F90MODPROC( coordinates, initialise )();

    string obsFile = stringParameter( "set" );

    /******************************************************************
    * Get the directory where obsFile is
    *******************************************************************/
    string directory = ".";
    if (obsFile.length() > 31)
    {
        directory = obsFile.substr(0, obsFile.length() - 32);

    }
    
    //   cout << "Line 235\n";


    /************************************************************
    * Get the list of input source-list files to omsrclistcomb
    *************************************************************/
    vector<string> srcList;
    getSourceListNames( directory, srcList );



    //   cout << "Line 246\n";


    /*******************************************************************
    * Get the image file names
    *
    *******************************************************************/
    vector<string> imageList;

    string name;
    if(obsFile.length() > 31)
    {
   
        name = obsFile.substr(obsFile.length() - 38, 6);
   

    }
    else
    {    
        name =  (const char*)QDir::currentDirPath();
        name = name.substr(name.length() - 6, 6);
    } 
    leicesterDirectory = "/disk/xmmssc2/ssco2/data/OM/leicester/2XMMdata/MAY2007DATA/" + name;
    
 
    
    loadImageFiles( leicesterDirectory,  srcList, imageList );

 


   

    /************************************************************************
    * Load the data from each source-list file and store it in a map, with
    * the key being the filter
    * Each filter will have a vector of source-lists 
    *
    ************************************************************************/

    map<string,  vector< vector<OmSourceParameter> >  > sourceListMap;
 

    if ( !loadInputData( srcList, sourceListMap) )
    {
        cout << "bug somewhere";
        Sas::fatal("Could not load file", srcList[0].c_str());
        if(sourceListMap.size() == 0) exit(0);

    }






    float minExposure = 1e6;
    float maxExposure = -minExposure;
    int noExposures = 0;




    
    map<string,  vector< vector<OmSourceParameter> >  >::iterator iter;
 

    for(iter = sourceListMap.begin(); iter != sourceListMap.end(); iter++)
    {

        vector< vector<OmSourceParameter> >  source = iter->second;
        for(unsigned i = 0; i < source.size(); i++) 
	{
	    if( source[i].size() > 0)
	    {
                noExposures++;

                minExposure = min(minExposure, source[i][0].exposure);
                maxExposure = max(maxExposure, source[i][0].exposure);
                if(source[i][0].exposure <= 0.0)
	        {
             
	            cout << "Invalid exposure -" << source[i][0].exposure << " in file " << srcList[i] << endl; 
                   cout << "Source-list size=" << source[i].size() << endl;
                }
            }
        }  

    }




    if(maxExposure < 0 && noExposures > 0) 
    {
      Sas::fatal("Error loading exposures", srcList[0].c_str());
    }



    /***************************************************************
    * Load the data from the observation source-list file
    ***************************************************************/


    int size = 0;
    vector < OmSourceParameter > source;
    string filter;
    float exposure;



     
    if ( !loadSourceParameters( obsFile, "SRCLIST", size, source, filter, exposure))                     
    {

      cout << "bug somewhere- could not load file" << obsFile << endl;
      Sas::fatal("Could not load file-", obsFile);


    }

   
   



    addExposures( minExposure, maxExposure, size );



    /************************************************************************
    * For each source in the combined source-list, for each filter we
    * need the number of exposures (sources merged)
    * size is the number of sources in the combined source-list
    *************************************************************************/
    addExposureNumbers( size, sourceListMap);


    addVegaMags();



   
    /*****************************************************************************************
    * Need to modify the significances 
    *
    ******************************************************************************************/
    modifySignificances( imageList);

  


}

/********************************************************
*
*
* Add the min and max exposures to the header
* 
* 
*
*********************************************************/
void addExposures(float minExposure, float maxExposure, int nSources)
{





 



    string file = stringParameter("outset");


    //cout << "Line 346 adding exposures to file " << file << endl;  



    DataSet *dataSet = dataSetServer->open( file, Dal::Modify);
 


    //    cout << "Line 279\n";  


    if (!dataSet)
    {
   
        Sas::fatal("Could not load file-",  file );

    }



 


   *dataSet->addAttribute( "EXP_MIN", "The minimum exposure value" ) = minExposure;
   *dataSet->addAttribute( "EXP_MAX", "The maximum exposure value" ) = maxExposure;


   *dataSet->addAttribute( "REVOLUT", "Orbital revolution number of spacecraft" ) = revolutionNo;

   *dataSet->addAttribute( "N_SOURCES", "Number of sources detected during observation" ) = nSources;



    dataSetServer->close(dataSet); // Finished with the file
  
  
    //    cout << "Line 302\n";  


}


/*************************************************************************
*
**************************************************************************/
void getSourceListNames(string directory, vector<string> &fileNames)
{


    vector<QString> file = loadDirectoryFiles( directory, QDir::Files, "SWS"  );  
     
    if( file.size() == 0)
    {

     


        cout << "BUG- NO SWS FILE\n";
        exit(0);

    }





  
    for(unsigned i = 0; i < file.size(); i++)
    {
      
        string name;    
        if (directory.length() < 4)
            name = (const char *)file[i];
        else  
            name = directory + "/" + (const char *)file[i];
     
 
        if ( !QFile::exists( name ) )
        {
            cout << i << ") File " << name << " does not exist\n";
 
            Sas::fatal("Could not load file", name.c_str());
        }



        DataSet *dataSet = dataSetServer->open( name, Dal::Read);
        Table *table = dataSet->table( "SRCLIST", Dal::Read ); 
        if (table->hasColumn( "SRC_ID" ))
        { 
	    fileNames.push_back( name );
       
        } 

        dataSetServer->close(dataSet); // Finished with the file
 




    }

   



}


/****************************************************************
* Obtain a list of all the SRCLIST files in the directory and
* see which one was processed by omsrclistcomb
*
****************************************************************/
void getSourceListNames( vector<string> &fileName )
{





    string file = stringParameter("set");
    //        cout << "Line 204 File =" << file << endl;
    //    ifstream input(file.c_str(), ios::in | ios::binary);

    char buffer[82];
   
   

    // cout << "Line 344\n";



   FILE *f;
   

   if ((f=fopen(file.c_str(),"rb"))==NULL) {
      printf("File \"%s\" not found\n",file.c_str());
      Sas::fatal("Could not load file-", file);

   }
   


   //cout << "Line 359\n";




   string command;
   while ( !feof(f) ) 
   {
       fgets(buffer, 81, f);
   




      //   while(!input.eof())
      //   {

 
 
      //      input.getline(buffer, 81, '\0');

        
 
 

        string text(buffer);      
        
        if ( text.find("XPROC0") != string::npos ) 
	{
	    command = text;
            break;
        }
    }

    while ( !feof(f) ) 
    {
        fgets(buffer, 81, f);
   
     

        string text(buffer);      
        if ( text.find("CONTINUE") == string::npos ) 
	{
	   
            break;
        }
        else
	{
	    command += text;


	}
    }

    unsigned int k;



    while( ( k = command.find(' ', 0)) != string::npos)
    {
    
        command.erase(k, 1);

    }





    // cout << "Line 428\n";


    
    while( ( k = command.find('&', 0)) != string::npos)
    {

        command.erase(k, 1);
    }


    while( ( k = command.find("'CONTINUE'", 0)) != string::npos)
    {

        command.erase(k, 10);
    }



    // cout << "Line 447\n";




  
   
    while( ( k = command.find("P0", 0)) != string::npos)
    {

   
        /**************************************************************
	* NB the files may be compressed (FTZ) but
        * the command-line may be for the uncompressed ones (*FIT*)
	***************************************************************/


        string file = command.substr(k, 31);
        if ( !QFile::exists( file ) )
	{

            file.replace( file.length() - 3, 3, "FTZ");



	}

        if ( !QFile::exists( file ) )
	{

	    cout << "File " << file << " does not exist\n";
	    
   

        }
        else
	{
            fileName.push_back( file );
        }    
        command.erase(0, k + 31);

    }


    //   cout << "Line 465\n";


    //       for (unsigned i = 0; i < fileName.size(); i++)
    //            cout << i << ") " << fileName[i] << endl;


    unsigned n = fileName.size();
    fileName.resize(n - 1);  


    //  cout << "Line 483\n";



}
/**********************************************
*
***********************************************/
void getPointing(vector<string>  &fileName, double &raPnt, double &decPnt, double &pa)
{

 

  
    double sum1 = 0l, sum2 = 0l, sum3 = 0l;
  
    for(unsigned i = 0; i < fileName.size(); i++)
    {

        // Open the file 
        DataSet *dataSet = dataSetServer->open(fileName[i], Dal::Read);
        if (!dataSet)
        {
            cout << "could not open file " << fileName[i] << endl;
            Sas::fatal("Could not load file-", fileName[i]);

        }

  
 

        float raPnt = dataSet->attribute("RA_PNT")->asReal();

        sum1 += (double)raPnt;
        float decPnt = dataSet->attribute("DEC_PNT")->asReal();
        sum2 += (double)decPnt;
  
        float pa = dataSet->attribute("PA")->asReal();
        sum3 += (double)pa;
  

	//        cout << "rapnt=" << raPnt << " decpnt=" << decPnt << " pa=" << pa << endl;



        dataSetServer->close(dataSet); // Finished with the file
  
   
    }

    double n = (double)fileName.size();
    raPnt = sum1 / n;

    decPnt = sum2 / n;


    pa = sum3 / n;

 }

/***************************************************************
*
*
****************************************************************/
/*
windowParameters getWindowParameters( string fileName )

{

 
    windowParameters parameters;    

    
   
    char name[256];
    strcpy( name, fileName.c_str());

    F90MODPROC(coordinates, windowparameters)( name, &parameters.x0, &parameters.y0, &parameters.xBin, &parameters.yBin);

    return parameters;

}
*/
/***************************************************************
*
*
****************************************************************/
windowParameters getWindowParameters( string fileName )

{

 
    windowParameters parameters;    
   
    char name[256];
    int stringLength1=fileName.length();
    //strcpy( name, fileName.c_str());

    sprintf(name, "%s", fileName.c_str());
    size_t stringLength2=strlen(name);

 
    unsigned char name3u[256];

    // passing strings to fortran see:
    // http://www.xgear.eu/callfortranfromc.html

    //cout << "L723 name="<< name << " stringLength1=" << stringLength1 << endl;
    //cout << "L729 stringLength2="<< stringLength2 << endl;
    

    for (int ii=0; ii<256; ii++){
      name3u[ii]='\0';
    }
    for (int ii=0;ii<stringLength1; ii++){
      name3u[ii]=(unsigned char)name[ii];
    }

    F90MODPROC(coordinates, windowparameters1)(&name3u[0], &stringLength2, &parameters.x0, 
			       &parameters.y0, &parameters.xBin, &parameters.yBin);
    return parameters;

}

/********************************************************************************
*
********************************************************************************/
void convertCoords( windowParameters &parameter, float &x, float &y)
{
    F90MODPROC( coordinates, convertcoords)( &x, &y, &parameter.x0, &parameter.y0, &parameter.xBin, &parameter.yBin);
}

/********************************************************************************
*
* Load various data from each source-list
*
********************************************************************************/

bool loadInputData (vector<string> &fileName, 
                    map<string, vector < vector<OmSourceParameter> > > &sourceListMap)
{
 
   

    for(unsigned i = 0; i < fileName.size(); i++)
    {

        int size = 0;
        string filter;
        vector<OmSourceParameter> source;
	float exposure;
        if ( !loadSourceParameters( fileName[i], "SRCLIST", size, source, filter, exposure)) 
        {

	   
 
            cout << "could not load file name " << fileName[i] << endl;  
	    return false;
        }
        else
	{
	  //            cout << "Successfully loaded data from " << fileName[i] << endl;  


        }




        /******************************************************************************
	* Convert x and y coordinates to full-frame coordinates
        *******************************************************************************/
        windowParameters parameter = getWindowParameters( fileName[i] );

        for(unsigned j = 0; j < source.size(); j++)
        {


            convertCoords( parameter, source[j].xcoord, source[j].ycoord );

        }

	sourceListMap[filter].push_back(source);     

    }

    /******************************************************************************
    * Get the satellite revolution
    ******************************************************************************/


    DataSet *dataSet = dataSetServer->open( fileName[0], Dal::Read);
   
    revolutionNo = dataSet->attribute("REVOLUT")->asString();



    dataSetServer->close(dataSet); // Finished with the file
 




  
    return true;

}

/*************************************************************************
*
*
*
*
*
*
**************************************************************************/
void convertCombinedCoords( map< string, vector< vector<OmSourceParameter> > >  &sourceListMap,   
                            vector < OmSourceParameter > &source)

{ 

    vector< vector<OmSourceParameter> > sourceList;  // To store all the source-lists from each SWS file

    map<string, vector< vector<OmSourceParameter> > >::iterator iter;
 
 

    map<int, vector<OmSourceParameter> > sourceNoMap;



    for(iter = sourceListMap.begin(); iter!= sourceListMap.end(); iter++)
    {
        vector< vector<OmSourceParameter> > source1 = iter->second;
	for( unsigned i = 0; i < source1.size(); i++)
	{    
	    sourceList.push_back( source1[i] ); 
     
        
            for(unsigned j = 0; j < source1[i].size(); j++)
	    {
	        sourceNoMap[ source1[i][j].number - 1 ].push_back( source1[i][j] );

            }
        }

    }

    //   cout << "No of sources=" << source.size() << endl;

    map<int, vector<OmSourceParameter> >::iterator iter1;
    for(iter1 = sourceNoMap.begin(); iter1 != sourceNoMap.end(); iter1++)
    {

      
        if(iter1->second.size() == 0)
        {
            cout << "bug at line 851\n";
	    exit(0);
        }

    }

   
    //   cout << "Line 844 No of input source-lists =" << sourceList.size() << endl;


    if(sourceList.size() == 1) // Only one input source-list file
    {

        for(unsigned i = 0; i < source.size(); i++)
        {

            source[i].xcoord = sourceList[0][i].xcoord;
            source[i].ycoord = sourceList[0][i].xcoord;           

       }
       return;
    }


    



    /***********************************************************************************
    *
    *
    * More than one input source-list to omsrclistcomb
    * NB source%number is the unique source number in the combined source-list file
    ************************************************************************************/


    //    for(unsigned i = 0; i < source.size(); i++)  // Looping through each source in the combined source list



    for(iter1 = sourceNoMap.begin(); iter1 != sourceNoMap.end(); iter1++)
    {

        int i = iter1->first;
        
     
        Integer2 maxQFlag = 0;        
        Integer1 maxCFlag = 0;        

        float maxRate = -100.0;
     
        
	//	vector<OmSourceParameter> source1 = sourceNoMap[i];
     	vector<OmSourceParameter> source1 = iter1->second;
   
     
        double sum1 = 0.0, sum2 = 0.0, sum3 = 0.0;
	//      cout << "\n\n\n******************************************************************\n";
	//	cout << "input data for combined source " << i << endl;
        for ( unsigned k = 0; k < source1.size(); k++)
	{

	  


             maxQFlag = max(maxQFlag, source1[k].qualityflag);
             maxCFlag = max(maxCFlag, source1[k].confusionflag);

             maxRate = max(maxRate,   source1[k].rawcountrate);
             double error = max(0.1, (double)source1[k].positionerror);
                 
	     double weight = 1.0 / ( error * error);
             sum1 += (double)source1[k].xcoord * weight;
             sum2 += (double)source1[k].ycoord * weight;

             sum3 += weight;

	     //	     cout << k << ") x=" << source1[k].xcoord << " y=" << source1[k].ycoord << " number=" << source1[k].number << endl;



        }




 
	source[i].xcoord = sum1 / sum3;
        source[i].ycoord = sum2 / sum3;
   	   

	source[i].qualityflag = maxQFlag; 
	source[i].confusionflag = maxCFlag; 

      	source[i].rawcountrate = maxRate; 
       
       
	//       cout << i << ") Merged source x=" << source[i].xcoord << " y=" << source[i].ycoord << endl;
          
    		    
        for ( unsigned k = 0; k < source1.size(); k++)
	{
	  if ( abs( source1[k].xcoord - source[i].xcoord ) > 50.0 ||
               abs( source1[k].ycoord - source[i].ycoord ) > 50.0 )

	  {
         
            cout << k << ") x=" << source1[k].xcoord << " y=" << source1[k].ycoord << endl;

             cout << i << ") Merged source x=" << source[i].xcoord << " y=" << source[i].ycoord << endl;

	    cout << "Bug at line 966\n";
            exit(0); 



	  }

        }
        
    }

};

/***************************************************************************
*
*
*
*
*
*
**************************************************************************/
void resetflags( vector<OmSourceParameter> &source)
{

    for(unsigned i = 0; i < source.size(); i++)
    {
        int qflag = 0, cflag=0;

        if(source[i].qualityflag & (1 << 0))  // Bad pixel
	    qflag |= (1 << 0);

        if(source[i].qualityflag & (1 << 1))  // Read-out streak
	    qflag |= (1 << 1);



	/*********************************************************************
	 * NB SKIP SMOKE RINGS AND DIFFRACTION SPIKES - KNOWN TO BE UNRELIABLE
	 ************************************************************************/
	//        if(source[i].qualityflag & (1 << 2))  // Smoke ring
	//	  qflag |= (1 << 2);

	//        if(source[i].qualityflag & (1 << 3))  // Diffraction spike
	//	  qflag |= (1 << 3);

        if(source[i].qualityflag & (1 << 4))  // Mod-8 pattern
	    qflag |= (1 << 4);




	if(source[i].qualityflag & (1 << 6))  // Central enhancement
	    qflag |= (1 << 5);




	if(source[i].qualityflag & (1 << 5))  // Near a bright source
	    qflag |= (1 << 6);



        if( source[i].confusionflag & (1 << 2))  // near an edge
	{
            qflag |= (1 << 7);

        } 



        if( source[i].extensionflag == 0 && source[i].confusionflag & (1 << 3))  // 
	{
	    qflag |= (1 << 8);     // Point source within an extended-source region

        } 



        if( source[i].confusionflag & (1 << 0))  // 
	{
	    cflag |= (1 << 0);     

        } 


        if( source[i].confusionflag & (1 << 1))  // 
	{
	    cflag |= (1 << 1);     

        } 

        source[i].qualityflag = qflag;

        source[i].confusionflag = cflag;

    }


}

/**************************************************
*
*
**************************************************/
vector<QString> loadDirectoryFiles(const string directory, 
         const int type, const string filter )
{
  
    QDir d( directory  );
    
    d.setFilter( type ); 
    const QFileInfoList *list = d.entryInfoList();

    QFileInfoListIterator it( *list );
    QFileInfo *fi;

    vector<QString> fileList;

    if (filter == "*")
    { 
        while ((fi = it.current()) != 0)
        {
        
            QString s = fi->fileName().latin1();	
            if(s.length() > 4)
	        fileList.push_back( s );
                     
    
            ++it;
        }
    }
    else 
    { 
        while ((fi = it.current()) != 0)
        {
        
            QString s = fi->fileName().latin1();
            	
            if(s.length() > 4 && s.contains(filter) && s.contains("FTZ")  )
	    {
	     
	        fileList.push_back( s );
               
            }
            ++it;
        }
        if(fileList.size() == 0)
	{
	 

            QFileInfoListIterator it( *list );
            QFileInfo *fi;



            while ((fi = it.current()) != 0)
            {
        
     
                QString s = fi->fileName().latin1();
	        
      
                if(s.length() > 4 && s.contains(filter) && s.contains("FIT")  )
	        {
	     
	            fileList.push_back( s );
               
                }
                ++it;
            }
        }
   



    } 

    return fileList;
}




/************************************************************************
* For each source in the combined source-list, for each filter we
* need the number of exposures (sources merged)
* size is the number of sources in the combined source-list
*************************************************************************/
void addExposureNumbers( int size, 
                map<string,  vector< vector<OmSourceParameter> > > &sourceListMap)
{
  
    string file = stringParameter("outset");

     
    map<string, vector< vector<OmSourceParameter> > >::iterator iter;

    //   cout << "Line 1228\n";

    //    for(iter = sourceListMap.begin(); iter != sourceListMap.end(); iter++ )



    for(unsigned k = 0; k < omFilter.size(); k++)

    {

      //        string filter = iter->first;  // The filter
        string filter = omFilter[k];

        iter = sourceListMap.find( filter );
        if(iter != sourceListMap.end() ) // Filter exists
	{         

            vector< vector<OmSourceParameter> >  sourceList = iter->second;
            vector<int16> number(size);
       
        
            for(unsigned i = 0; i < sourceList.size(); i++)
	    {
	  
                for(unsigned j = 0; j < sourceList[i].size(); j++)
	        {
	            //	      cout << j << ") number=" <<  sourceList[i][j].number << endl;
	            number[ sourceList[i][j].number - 1]++;         

                }
 

            }
    
            for(unsigned i = 0; i < number.size(); i++)
	        if(number[i] == 0) number[i] = 0;

            string columnName = "N_" + filter + "_EXP";
            string comment = "Number of " + filter + " exposures during observation";
	
       
            /***************************************************************
	    *
            ****************************************************************/  
            if ( !addColumn( file,
                         "SRCLIST", 
                         columnName, 
                         "",
                         comment,   
	                 size,
	                 number) )
            {


            }
	}
        else // No sources in this filter
	{
            vector<int16> number(size, 0);
            string columnName = "N_" + filter + "_EXP";
            string comment = "Number of " + filter + " exposures during observation";
	
       
            /***************************************************************
	    *
            ****************************************************************/  
            if ( !addColumn( file,
                         "SRCLIST", 
                         columnName, 
                         "",
                         comment,   
	                 size,
	                 number) )
            {


            }


        }


    }

  
}





/************************************************************************
* 
*
*
*
*************************************************************************/
void addVegaMags( )
{
  

    string inFile = stringParameter("set");
    DataSet *inDataSet = dataSetServer->open( inFile, Dal::Read);
    Table *inTable = inDataSet->table( "SRCLIST", Dal::Read);
 


    string outFile = stringParameter("outset");
    DataSet *outDataSet = dataSetServer->open( outFile, Dal::Modify);
    Table *outTable = outDataSet->table( "SRCLIST", Dal::Modify);
 
     
    int size = (int)outTable->rows();

    for(unsigned k = 0; k < omFilter.size(); k++)
    {

        string filter = omFilter[k];
        vector<float> vegaMag, vegaMagErr;
     
   



        string columnName = filter + "_CORR_RATE";

        /************************************************************************
	* Check to see if this columnn exists
	***********************************************************************/
        if (inTable->hasColumn( columnName) ) 
	{          

            const ColumnData *columnData = inTable->column(columnName)->data();
      
            float *xData = columnData->real32Data();

            columnName = filter + "_CORR_RATE_ERR";

            const ColumnData *columnData1 = inTable->column(columnName)->data();
      

            float *xDataErr = columnData1->real32Data();

	    //         cout << "infile=" << inFile << " column=" << columnName << endl;
            for(int i = 0; i < size; i++)
	    {
	 
	      float mag, magErr;
	      computeVegaMag( filter, xData[i], xDataErr[i], mag, magErr);
	        vegaMag.push_back( mag );
	        vegaMagErr.push_back( magErr );
	  
		//		cout << i << ") vega mag=" << vegaMag[i] << "+/-" << vegaMagErr[i] << endl;
	
            }
            delete columnData;
            delete columnData1;

          

        }
        else  // Column doesn't exist
	{


            for(int i = 0; i < size; i++)
	    {
	        vegaMag.push_back( -9999.0);
	        vegaMagErr.push_back( -9999.0);;
	             	
            }
      

        }

        columnName = filter + "_VEGA_MAG";
        string comment = "Source Vega magnitude through the " + filter + " filter"; 
        /***************************************************************
	*
        ****************************************************************/  

        if ( !addColumn( outTable, 
                         columnName.c_str(), 
                         "mag",
                         comment.c_str(),   
	                 size,
	                 vegaMag) )

        {


           
	}
        /***************************************************************
	*
        ****************************************************************/

        columnName = filter + "_VEGA_MAG_ERR";
        comment = "1-sigma Vega magnitude through the " + filter + "filter"; 
  
        if ( !addColumn( outTable, 
                         columnName.c_str(), 
                         "mag",
                         comment.c_str(),   
	                 size,
	                 vegaMagErr) )
        {


           
	}




   
    }








    dataSetServer->close( inDataSet ); // Finished with the file
 
    dataSetServer->close( outDataSet ); // Finished with the file
 
}
/********************************************************************
*
* For each SWS file find its unrotated image counterpart
*
*
*********************************************************************/
void loadImageFiles( const string directory, 
                     const vector<string>& srcList, 
                     vector<string> &imageList )
{

  

    unsigned size1 = srcList.size();
   

    imageList.resize( 0 );

    //    cout << "matchfiles number of SWS files=" << size1 << endl;

  
    /*********************************************************************
    * Get a list of the IMAGE_ files
    *
    *********************************************************************/

 

    vector<QString> tempList = loadDirectoryFiles( directory, QDir::Files, "SIMAG");
    

    

    unsigned size2 = (int)tempList.size();
    for (unsigned i = 0; i < size1; i++)  // Match source-list with image
    {
        QString file = srcList[i];
        int k = file.length() - 31;
        QString string1 = file.mid(k, 17);
        QString string2 = file.mid(k + 23, 8);
	//      cout << i << ") SWS file=" << srcList[i] << endl;




        for (unsigned j = 0; j < size2; j++)
	{
	    QString file = tempList[j];
          
            QString string3 = file.mid(0, 17);
            QString string4 = file.mid(23, 8);
            if ( string1 == string3 && string2 == string4 )
	    {
	        imageList.push_back( tempList[j] );

              

	    
                break;

            }

        }
    }




    if ( imageList.size() != srcList.size() )
    {




        cout << "bug at line 5100\n";
 



          for(unsigned i = 0; i < srcList.size(); i++)
          {
	    cout << i << ") " << srcList[i] << endl;
          }


          for(unsigned i = 0; i < tempList.size(); i++)
          {
              cout << i << ") " << tempList[i] << endl;
          }






       exit(0); 

    }




    //    for(unsigned i = 0; i < srcList.size(); i++)
    //    {
    //      cout << i << ") " << srcList[i] << " " << imageList[i] << endl;
    //    }

}
/************************************************************************
*
*
*
*************************************************************************/
void modifySignificances( const vector<string> &imageList)
{

  //  cout << "modify signi\n";

  //    map<string, vector<ImageData> > imageMap;

    map<string, vector<string> > imageMap;

    for(unsigned i = 0; i < imageList.size(); i++)
    {


        string file = leicesterDirectory + "/" + imageList[i];

        DataSet *dataSet = dataSetServer->open( file, Dal::Read);
        if(!dataSet)
	{
	    cout << "Could not open file " << file << endl;
	    exit(0);

        }

        string filter = dataSet->attribute("FILTER")->asString();
	//        ImageData data;     
	//       loadImageData( dataSet, data);



        dataSetServer->close( dataSet ); // Finished with the file

	//        data._name = file;         
  
	//       imageMap[filter].push_back( data ); 
        imageMap[filter].push_back( file ); 


    }


    string file = stringParameter("outset");

    DataSet *dataSet = dataSetServer->open( file, Dal::Modify);
 
    Table *table = dataSet->table( "SRCLIST", Dal::Modify ); 

    /**********************************************************************************
    * Need the RAs and Decs from the combined source-list file
    ***********************************************************************************/


    vector<double> obsRa, obsDec;
    string name = "RA";
    if ( !LoadColumnData(table, name, obsRa) )
    {
        cout << "Could not load column " << name << " from file " << file << endl;
    
        dataSetServer->close( dataSet ); // Finished with the file
        exit(0);
    }



    name = "DEC";
    if ( !LoadColumnData(table, name, obsDec) )
    {
        cout << "Could not load column " << name << " from file " << file << endl;
        dataSetServer->close( dataSet ); // Finished with the file

    }


    /*********************************************************************
    * Check the significances for each filter
    *
    **********************************************************************/

    map<string, vector<string> >::iterator iter;

   
    for(iter = imageMap.begin(); iter != imageMap.end(); iter++)
    {
        string filter = iter->first;
        vector<ImageData> imageData;

        vector<string> imageFiles = iter->second;
        for(unsigned i = 0; i < imageFiles.size(); i++)
	{
            DataSet *dataSet = dataSetServer->open( imageFiles[i], Dal::Read);
   
	    ImageData data;     
	    loadImageData( dataSet, data);
            dataSetServer->close( dataSet ); // Finished with the file
            imageData.push_back( data ); 


        }

        string name = filter + "_SIGNIF";
	//        cout << "Checking column " << name << endl;
        if( !table->hasColumn( name ) )
	{
	  cout << "Bug\n";
          dataSetServer->close(dataSet); // Finished with the file
 
	}
    
        vector<float> signif;
        if ( !LoadColumnData(table, name, signif) )
        {
            cout << "Could not load column " << name << " from file " << file << endl;
            dataSetServer->close( dataSet ); // Finished with the file

        }

      
        /*******************************************************************************
	* Now need to check each source for a NULL significance and check the images
        * to see if it could have been detected
        *
        *******************************************************************************/
       
        for(unsigned  j = 0; j < obsRa.size(); j++)
	{

	 
	    if( isNull(signif[j] ) || signif[j] < 0.0  )
	    { 
	        int count = 0;  
	        for(unsigned k = 0; k < imageData.size(); k++)
	        {
	         
		    
		    if ( imageHasSource( obsRa[j], obsDec[j], imageData[k]) )
		    {
		        ++count;

                    }

                  
                }

                if(count > 0) signif[j] = -999.0;
            } 
             

        } 
    
       
        ColumnData *columnData = table->column(name)->data();
        setColumnData( (int)signif.size(), columnData, signif);

    } 



    dataSetServer->close(dataSet); // Finished with the file
 




}


/********************************************************************
*
*
*
*
********************************************************************/
void loadImageData(const DataSet *dataSet, ImageData &data)
{


    /**************************************************************
    * Get tne WCS keywords for converting pixels to RA and DEC
    **************************************************************/

  

    const Block *block = dataSet->block(0);
 
    if (block->hasAttribute("CDELT1") ) // Sky image
    {
       
        data._exposure = (int16) ( dataSet->attribute("EXPOSURE")->asReal() );  // RA plate-scale (degrees and negative)
    
        data._cdelt1 = block->attribute("CDELT1")->asReal();  // RA plate-scale (degrees and negative)
        data._cdelt2 = block->attribute("CDELT2")->asReal();  // Dec plate-scale (degrees)
        data._crpix1 = block->attribute("CRPIX1")->asReal();
        data._crpix2 = block->attribute("CRPIX2")->asReal();
        data._crval1 = block->attribute("CRVAL1")->asReal();
        data._crval2 = block->attribute("CRVAL2")->asReal();


        /***********************************************************************************
	*
        *  Load the image
        ************************************************************************************/

        data._image = loadImage(dataSet, data._nx, data._ny);

    }
    else
    {
        cout << "Error loading data from file\n";
        exit(0);

    }



}



/*********************************************************************
*
*
*
*
*********************************************************************/
vector<vector<float> >  loadImage(const DataSet *set, int &nx, int &ny)
{


    /*****************************************************************
    * Get the size of the image
    *****************************************************************/
    const Array* array = set->array(0);
 

    const ArrayData * arraydata = array->data();
    const real32 * theData = arraydata->real32Data();
 

    /*****************************************************************
    * Get the size of the image
    *****************************************************************/
 
    longVec size = array->size();
   



    nx = (int)size[1];
    ny = (int)size[0];


    vector< vector<float> > image(nx); 


    for (int i = 0; i < nx; i++)
    {
        image[i].resize(ny); 
        for (int j = 0; j < ny; j++)
        {
	    image[i][j] = theData[i + (j * nx)];  
	 
        }

  
    }


    delete arraydata;
   

    return image;
}
/**************************************************************
*
**************************************************************/
bool imageHasSource(double ra, double dec, ImageData &data)
{
 
 


    double dx, dy; 
    F90MODPROC( convert_mod, computetangentplanecoords)(&data._crval1, &data._crval2, &ra, &dec, &dx, &dy);

    /*********************************************************************
    * NB crpix1 is the reference x pixel, y the reference y pixel
    * crval1 is the ra value at crpix1 and crval2 the dec value at crpix2
    * cdelt1 and cdelt2 are the plate scales in the x and y directions
    **********************************************************************/

    double x = dx / data._cdelt1 + data._crpix1;  // The x and y coordinates on the image
    double y = dy / data._cdelt2 + data._crpix2;

  

    int i0 = int(x - 0.5);

    if(i0 < 1) return false;
    if(i0 > data._nx) return false;


    i0 = max(0, i0);
    i0 = min(data._nx - 1, i0);

    int j0 = int(y - 0.5);
    if(j0 < 1) return false;  
    if(j0 > data._ny) return false;



    j0 = max(0, j0);
    j0 = min(data._ny - 1, j0);
   


    if( isNull( data._image[i0][j0] ) )
      return false;
    else
      return true;

     

        
}
/***********************************************************************
*
*
*
*
***********************************************************************/

void computeVegaMag( string filter, float rate, float rateErr, float &mag, float &magErr)
{

    if(isNull(rate) || rate < 0.0 ) 
    {
      mag = rate;
      magErr = rate;
      return;

    }

    double f0 = 0.0;
    double a0v = 0.0;
    double mag0 = 0.0;
    if (filter == "v" || filter == "V")
    {
        f0 = 3.70e-9;   // Vega flux for mag0 
        mag0 = 0.03;
        a0v = 2.50e-16;

    }
    else if (filter == "b" || filter == "B")   
    {
        f0 = 6.40e-9;
        mag0 = 0.03;
        a0v = 1.34e-16;

    }

    else if (filter == "u" || filter == "U")   
    {
        f0 = 3.20e-9;
        mag0 = 0.025;
        a0v = 1.70e-16;

    }

    else if (filter == "uvw1" || filter == "UVW1")   
    {
        f0 = 3.68e-9;
        mag0 = 0.025;
        a0v = 4.86e-16;

    }
    else if (filter == "uvm2" || filter == "UVM2")   
    {
        f0 = 4.33e-9;
        mag0 = 0.025;
        a0v = 2.19e-15;

    }

    else if (filter == "uvw2" || filter == "UVW2")   
    {
        f0 = 5.03e-9;
        mag0 = 0.025;
        a0v = 5.88e-15;

    }
    else
    {
        mag = 0.0;
        magErr = 0.0;
        return;
    }

    double flux = a0v * rate;
    double fluxErr = a0v * rateErr;


    mag = mag0 - 2.5 * log10( flux / f0);
    magErr = max(0.01, 0.4342945 * 2.5 * fluxErr / flux);






}

