// ESA (C) 2000-2018 
//
// This file is part of ESA's XMM-Newton Scientific Analysis System (SAS).
//
//    SAS is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    SAS is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with SAS.  If not, see <http://www.gnu.org/licenses/>.
#include <fstream>
#include <iostream>
#include "omflags.h"
#include "Dal.h"
#include "param.h"
#include<cstring>
#include <cstdio>
#include "utility.h"
#include "sourcelist.h"
#include "qfile.h"
#include "qstring.h"
#include "qdatetime.h"
#include "qfileinfo.h"
#include "qdir.h"


#include "OmSourceParameters.h"


/******************************************************************
*
*******************************************************************/
F90MODPROCPROTO(void, coordinates, computestandardcoordinates)
       (double *ra, double *dec, double *raPnt, double *decPnt, double *xi, double *eta);



F90MODPROCPROTO(void, coordinates, list)
       (int *n, double *data);

F90MODPROCPROTO(void, coordinates, list1)
       (int *n, OmSourceParameter *data);



F90MODPROCPROTO(void, addflags, addsourceflags)
       (int *n, OmSourceParameter *data);


F90MODPROCPROTO(double, coordinates, getplatescale)();


F90MODPROCPROTO(void, addflags, createoutputlist)();

F90MODPROCPROTO(void, coordinates, initialise)();


F90MODPROCPROTO(void, coordinates, addistortion)(double *, double *, double *, double *);

F90MODPROCPROTO(void, coordinates, undistortion)(double *, double *, double *, double *);

//F90MODPROCPROTO(void, coordinates, windowparameters)(char[256], int *, int *, int *, int *);
F90MODPROCPROTO(void, coordinates, windowparameters1)(unsigned char *, size_t *, int *, int *, int *, int *);
//F90MODPROCPROTO(void, coordinates, windowparameters1)(int , int *, int *, int *, int *);

F90MODPROCPROTO( void, coordinates, convertcoords)(float *x, float*y, int *x0, int *y0, int *binx, int *biny);



/*************************************************************************************
*
*************************************************************************************/

bool  fileOkay( string file );


void getSourceListNames( vector<string> &fileName );

void getSourceListNames( string directory, vector<string> &fileName );

//void getRevolutionFromSummary( string directory, vector<string> &fileName );
void getRevolutionFromSummary( string directory, char *revolution );


windowParameters getWindowParameters( string fileName );






vector<QString> loadDirectoryFiles(const string directory, 
				   const int type, const string filter = "*" );

//vector<QString> loadSummaryFiles(const string directory, 
//				   const int type, const string filter = "*" );
vector<QString> loadSummaryFile(const string directory, 
				char *filter, char *revolution);

void addVegaMags( );


void getPointing(vector<string> &name, double &raPnt, double &decPnt, double &pa);

void convertCoords( windowParameters &parameters, float &x, float &y);

//bool loadInputData (vector<string> &fileName, 
//map< string,  vector< vector<OmSourceParameter> > >  &sourceList);
bool loadInputData (vector<string> &fileName, char * revolution, float filterExposure[],
		    float &minExposure, float &maxExposure, 
		    map< string,  vector< vector<OmSourceParameter> > >  &sourceList);

void convertCombinedCoords( map<string, vector< vector<OmSourceParameter> > > &sourceList,   
                            vector < OmSourceParameter > &source);


void resetflags(  vector < OmSourceParameter > &source);

void addExposures(float minExposure, float maxExpsure, float filterExposure[], int nSources);


void addExposureNumbers( int size, 
			 map<string,  vector< vector<OmSourceParameter> > > &sourceListMap);



void modifyPositionErrors(Table *&inTable, Table *&outTable);

string revolutionNo = "";

vector<string> omFilter;


/**********************************************************************
* 
*
*
*
**********************************************************************/
void omflags()
{

 
  
    omFilter.push_back( "UVW2" );
    omFilter.push_back( "UVM2" );
    omFilter.push_back( "UVW1" );
    omFilter.push_back( "U" );
    omFilter.push_back( "B" );
    omFilter.push_back( "V" );
    omFilter.push_back( "WHITE" );


    /********************************************************
    *
    *
    * Create the output source-list
    *
    ********************************************************/

    F90MODPROC( addflags, createoutputlist )();

    F90MODPROC( coordinates, initialise )();

    string obsFile = stringParameter( "set" );
    cout << "L172 obsFile=" << obsFile << endl;
    /******************************************************************
    * Get the directory where obsFile is
    *******************************************************************/
    string directory = ".";
    if (obsFile.length() > 31)
    {
        directory = obsFile.substr(0, obsFile.length() - 32);

    }
    

    cout << "L184 Obsfile=" << obsFile << " Directory=" << directory << endl;

    /************************************************************
    * Get the list of summary files  files to omsrclistcomb
    *************************************************************/
    cout<< "L189 getting the revolution number from the summary file "<<endl;
    //vector<string> summaryFileName;
    char revolution[5]={' ',' ',' ',' ','\0'};
    int revLength=0;
    
    revLength=strlen(revolution);
    //printf("L173 revolution=%s revLength=%d \n", revolution, revLength);
    getRevolutionFromSummary( directory, revolution );
    //cout<<"L175 finished getRevolutionFromSummary"<<endl;
    revLength=strlen(revolution);
    //printf("L176 revolution=%s revLength=%d \n", revolution, revLength);

    /************************************************************
    * Get the list of input source-list files to omsrclistcomb
    *************************************************************/
    vector<string> fileName;
    cout<<"L205 calling getSourceListNames "<<endl;
    getSourceListNames( directory, fileName );

    //cout << "L170  after getSourceListNames " << endl;

    /************************************************************************
    *
    * Load the data from each source-list file and store it in a map, with
    * the key being the filter
    * Each filter will have a vector of source-lists 
    *
    ************************************************************************/

    map<string,  vector< vector<OmSourceParameter> >  > sourceListMap;

    float filterExposure[6]={};

    float minExposure = 1e6;
    float maxExposure = -minExposure;

    cout << "L209 loading input source lists (calling loadInputData) \n";
    cout << "--- --- ---\n";
    //if ( !loadInputData( fileName, sourceListMap) )
    if ( !loadInputData( fileName, revolution, filterExposure, minExposure, maxExposure, sourceListMap) )
    {
      //        cout << "bug somewhere";
        Sas::fatal("Could not load file", fileName[0].c_str());
        if(sourceListMap.size() == 0) exit(0);

    }
    cout << "L235 sourceListMap.size90=" << sourceListMap.size() << endl;
    cout << "L236 minExposure=" << minExposure << " maxExposure=" << maxExposure << endl;

    //cout << "L195\n";

    /*
    cout << "------------\n";
    for (int ii=0; ii<6; ii++){
      cout << "L221 filterIndex=" << ii << " filterExposure=" << filterExposure[ii] << endl;
    }
    cout << "------------\n";
    */

    int noExposures = 0;

    //cout << "L223\n";
    //cout << "L233 minExposure="<< minExposure << " maxExposure="<< maxExposure << endl;
    map<string,  vector< vector<OmSourceParameter> >  >::iterator iter;
    
    
    int ii=0;
    for(iter = sourceListMap.begin(); iter != sourceListMap.end(); iter++)
    {
      printf("L258 iter=%d \n", iter);
      ii++;
      
      vector< vector<OmSourceParameter> >  source = iter->second;
      cout << "L262 ii=" << ii << " source.size="<< source.size() << endl;   
      for(unsigned i = 0; i < source.size(); i++) 
	{
	  cout << "L265 i=" << i << " minExposure="<< minExposure << " maxExposure="<< maxExposure << endl;
	  if( source[i].size() > 0)
	    {
	      noExposures++;
	      //printf("L242 i=%d minExposure=%f maxExposure=%f exposure=%f \n", 
	      //     minExposure, maxExposure, source[i][0].exposure);
	      //minExposure = min(minExposure, source[i][0].exposure);
	      //cout << "L245 i=" << i << " minExposure="<< minExposure << " source.exposure=" << source[i][0].exposure << endl;
	      //maxExposure = max(maxExposure, source[i][0].exposure);
	      //cout << "L247 i=" << i << " maxExposure="<< maxExposure << " source.exposure=" << source[i][0].exposure << endl;
	      //printf("L248 i=%d minExposure=%f maxExposure=%f exposure=%f \n", 
	      //     minExposure, maxExposure, source[i][0].exposure);
	      if(source[i][0].exposure <= 0.0)
	        {
		  
		  //          cout << "Invalid exposure -" << source[i][0].exposure << " in file " << fileName[i] << endl; 
		  //  cout << "Source-list size=" << source[i].size() << endl;
                }
            }
	  //cout << "L262 i=" << i << " minExposure="<< minExposure << " maxExposure="<< maxExposure << endl;
        }  
      //cout << "L264 ii=" << ii << " minExposure="<< minExposure << " maxExposure="<< maxExposure << endl;      
    } // end iter
    //cout << "L266 minExposure="<< minExposure << " maxExposure="<< maxExposure << endl;
    cout << "L272 noExposures="<< noExposures << endl;
    if(maxExposure < 0 && noExposures > 0) 
    {
      Sas::fatal("Error loading exposures", fileName[0].c_str());
    }



    //cout << "Line 237\n";  


    /***************************************************************
    * Load the data from the observation source-list file
    ***************************************************************/
    cout << "L303 loading data from the observation source-list file\n";
    cout << "L304 (calling loadSourceParameters for obsFile=" << obsFile << ")\n";
    cout << "... ... ... \n";
    int size = 0;
    vector < OmSourceParameter > source;
    string filter;
    float exposure;
    cout << "L310 obsFile=" << obsFile << endl;

    //cout << "L290 minExposure="<< minExposure << " maxExposure="<< maxExposure << endl;
    if ( !loadSourceParameters( obsFile, "SRCLIST", size, source, filter, exposure))                     
    {

      cout << "bug somewhere- could not load file" << obsFile << endl;
      Sas::fatal("Could not load file-", obsFile);

    } else {
      cout << "L320 source parameters are loaded filter="  << filter 
	   << " exposure=" << exposure << endl;
    }
   
    cout << "L324 minExposure="<< minExposure << " maxExposure="<< maxExposure << endl;
    //cout <<"L264 size="<< size << endl;
    /******************************************************************
    *
    *
    ******************************************************************/
    /*
    cout << "------------\n";
    for (int ii=0; ii<6; ii++){
      cout << "L310 filterIndex=" << ii << " filterExposure=" << filterExposure[ii] << endl;
    }
    cout << "------------\n";
    */
    addExposures( minExposure, maxExposure, filterExposure, size );
    //cout<<"L315 after addExposures size="<< size<<endl;

    /************************************************************************
    * For each source in the combined source-list, for each filter we
    * need the number of exposures (sources merged)
    * size is the number of sources in the combined source-list
    *************************************************************************/
    addExposureNumbers( size, sourceListMap);
   
    //cout << "L17 exposure numbers added, size="<< size<< endl ;  


    if(size == 0) return;
   

    /*****************************************************************************
    * Need to convert the x and y coordinates in the combined source-list
    * to full-frame coordinates
    *
    *****************************************************************************/
    cout<<"L340 converting combined coordinates "<<endl;
    convertCombinedCoords(sourceListMap, source);
    //cout<<"L292 "<<endl;

    int n = source.size();
    //cout<<"L295 n="<<n<<endl;

    OmSourceParameter *source1 = new OmSourceParameter[n];
 
    for(int i = 0; i < n; i++)
    {
        source1[i].xcoord = source[i].xcoord; 
        source1[i].ycoord = source[i].ycoord; 

        source1[i].extensionflag = source[i].extensionflag;
        source1[i].confusionflag = source[i].confusionflag; 
        source1[i].qualityflag = source[i].qualityflag; 
        source1[i].rawcountrate = source[i].rawcountrate; 

    
	//	source[i].qualityflag &= (~5);
	//		source1[i].qualityflag = 0;

	//		source1[i].confusionflag = 0;


    }


    //cout<<"L319 n="<<n<<endl;

    delete [] source1;

    /*
    cout << "------------\n";
    for (int ii=0; ii<6; ii++){
      cout << "L370 filterIndex=" << ii << " filterExposure=" << filterExposure[ii] << endl;
    }
    cout << "------------\n";
    */

    //cout << "L328\n";
    cout << "L369 calling addExposures \n"; 
    addExposures( minExposure, maxExposure, filterExposure, size );

    //cout << "L330\n";


    /************************************************************************
    * For each source in the combined source-list, for each filter we
    * need the number of exposures (sources merged)
    * size is the number of sources in the combined source-list
    *************************************************************************/
    addExposureNumbers( size, sourceListMap);

    //cout<<"L342 "<< endl;
    addVegaMags( );
 
    //cout << "L343\n";

    //cout <<"L398 Finished running program omflags\n";

}

/********************************************************
*
*
* Add the min and max exposures to the header
* 
* 
*
*********************************************************/
void addExposures(float minExposure, float maxExposure, float filterExposure[], int nSources)
{

  string file = stringParameter("outset");
  cout << "L414 adding exposure values to the file " << file << endl;
  //cout << "L404 opening file= " << file << "\n";  
  
  DataSet *dataSet = dataSetServer->open( file, Dal::Modify);
  
  //cout << "L417 opened the data set \n";  
  
   
  if (!dataSet)
    {
      
      Sas::fatal("Could not load file-",  file );
      
    }
  
  
  //cout << "L434 adding attributes EXP_MIN and EXP_MAX  \n";  
  
  *dataSet->addAttribute( "EXP_MIN", "The minimum exposure value" ) = minExposure;
  *dataSet->addAttribute( "EXP_MAX", "The maximum exposure value" ) = maxExposure;
  
  //cout << "L439 adding attributes EXP_filter   \n";  
  /*
  cout << "------------\n";
  for (int ii=0; ii<6; ii++){
    cout << "L439 filterIndex=" << ii << " filterExposure=" << filterExposure[ii] << endl;
  }
  
  cout << "------------\n";
  */
  *dataSet->addAttribute( "EXP_UVW2", "The summed UVW2 exposure" ) = filterExposure[0];
  *dataSet->addAttribute( "EXP_UVM2", "The summed UVM2 exposure" ) = filterExposure[1];
  *dataSet->addAttribute( "EXP_UVW1", "The summed UVW1 exposure" ) = filterExposure[2];
  *dataSet->addAttribute( "EXP_U", "The summed U exposure" ) = filterExposure[3];
  *dataSet->addAttribute( "EXP_B", "The summed B exposure" ) = filterExposure[4];
  *dataSet->addAttribute( "EXP_V", "The summed V exposure" ) = filterExposure[5];

  *dataSet->addAttribute( "REVOLUT", "Orbital revolution number of spacecraft" ) = revolutionNo;
  
  *dataSet->addAttribute( "NSOURCE", "Number of sources detected during observation" ) = nSources;
  
  
  dataSetServer->close(dataSet); // Finished with the file
  
  //    cout << "Line 302\n";  


}


/*************************************************************************
*
**************************************************************************/
void getSourceListNames(string directory, vector<string> &fileNames)
{

  printf("L435 loading directory files \n");
  
  vector<QString> file = loadDirectoryFiles( directory, QDir::Files, "SWS"  );  
     
  if( file.size() == 0){
      
    cout << "BUG- NO SWS FILE\n";
    exit(0);
  } else {
    printf("L444 file loaded file_size=%d \n",file.size()); 
  }

  
  for(unsigned i = 0; i < file.size(); i++)
    {
      
      string name;    
      if (directory.length() < 4){
	printf("L452 directory length=%d \n", directory.length());
	name = (const char *)file[i];
	cout << "L455 name="<< name << endl;
      } else{  
	//printf("L463 directory length=%d \n", directory.length());
	//printf("L464 directory c_str= %s \n", directory.c_str());
	name = directory + "/" + (const char *)file[i];
	//printf("L466 \n");
	printf("L461 name=%s \n", name.c_str());
      }
      if ( !QFile::exists( name ) ){
	cout <<"L464 " << i << ") File " << name << " does not exist\n";
	  
	Sas::fatal("L466 Could not load file", name.c_str());
      }else {
	//printf("L474 file loaded OK \n");
      }

      /************************************************************
       * 
       * Check that the file is okay
       *
       ************************************************************/
      
      if (fileOkay( name ) ) {
	//cout << "L482 Checking file:  " << name << " OK" << endl; 
	fileNames.push_back( name );
	  
      } else {
	printf("L482 Checking file: failed \n");
      } 
      //cout << "L486\n"; 
          
    }


}

/*************************************************************************
*
**************************************************************************/
//void getRevolutionFromSummary(string directory, vector<string> &fileNames)
void getRevolutionFromSummary(string directory, char *revolution)
{

  //printf("L473 loading directory files \n");
  
  //char revolution[5]={' ',' ',' ',' ','\0'};
  vector<QString> file = loadSummaryFile( directory, "SUM", revolution  );  
  
  if( file.size() == 0){
      
    cout << "L503 Could not get the revolution number from summary \n";
    
    //exit(0);
  } else {
    //printf("L488 summary file loaded file_size=%d \n",file.size()); 
    //printf("L492 revolution=%s \n", revolution);
  }


}

/****************************************************************
* Obtain a list of all the SRCLIST files in the directory and
* see which one was processed by omsrclistcomb
*
****************************************************************/
void getSourceListNames( vector<string> &fileName )
{


    string file = stringParameter("set");
    //        cout << "Line 204 File =" << file << endl;
    //    ifstream input(file.c_str(), ios::in | ios::binary);

    char buffer[82];
   
   

    // cout << "Line 344\n";



   FILE *f;
   

   if ((f=fopen(file.c_str(),"rb"))==NULL) {
      printf("File \"%s\" not found\n",file.c_str());
      Sas::fatal("Could not load file-", file);

   }
   


   //cout << "Line 359\n";

   string command;
   while ( !feof(f) ) 
   {
       fgets(buffer, 81, f);
   
      //   while(!input.eof())
      //   {

 
 
      //      input.getline(buffer, 81, '\0');

        string text(buffer);      
        
        if ( text.find("XPROC0") != string::npos ) 
	{
	    command = text;
            break;
        }
    }

    while ( !feof(f) ) 
    {
        fgets(buffer, 81, f);
   
     

        string text(buffer);      
        if ( text.find("CONTINUE") == string::npos ) 
	{
	   
            break;
        }
        else
	{
	    command += text;


	}
    }

    unsigned int k;



    while( ( k = command.find(' ', 0)) != string::npos)
    {
    
        command.erase(k, 1);

    }





    // cout << "Line 428\n";


    
    while( ( k = command.find('&', 0)) != string::npos)
    {

        command.erase(k, 1);
    }


    while( ( k = command.find("'CONTINUE'", 0)) != string::npos)
    {

        command.erase(k, 10);
    }



    // cout << "Line 447\n";




  
   
    while( ( k = command.find("P0", 0)) != string::npos)
    {

   
        /**************************************************************
	* NB the files may be compressed (FTZ) but
        * the command-line may be for the uncompressed ones (*FIT*)
	***************************************************************/


        string file = command.substr(k, 31);
        if ( !QFile::exists( file ) )
	{

            file.replace( file.length() - 3, 3, "FTZ");



	}

        if ( !QFile::exists( file ) )
	{

	    cout << "File " << file << " does not exist\n";
	    
   

        }
        else
	{
            fileName.push_back( file );
        }    
        command.erase(0, k + 31);

    }


    //   cout << "Line 465\n";


    //       for (unsigned i = 0; i < fileName.size(); i++)
    //            cout << i << ") " << fileName[i] << endl;


    unsigned n = fileName.size();
    fileName.resize(n - 1);  


    //  cout << "Line 483\n";



}
/**********************************************
*
***********************************************/
void getPointing(vector<string>  &fileName, double &raPnt, double &decPnt, double &pa)
{

 

  
    double sum1 = 0l, sum2 = 0l, sum3 = 0l;
  
    for(unsigned i = 0; i < fileName.size(); i++)
    {

        // Open the file 
        DataSet *dataSet = dataSetServer->open(fileName[i], Dal::Read);
        if (!dataSet)
        {
            cout << "could not open file " << fileName[i] << endl;
            Sas::fatal("Could not load file-", fileName[i]);

        }

  
 

        float raPnt = dataSet->attribute("RA_PNT")->asReal();

        sum1 += (double)raPnt;
        float decPnt = dataSet->attribute("DEC_PNT")->asReal();
        sum2 += (double)decPnt;
  
        float pa = dataSet->attribute("PA")->asReal();
        sum3 += (double)pa;
  

	//        cout << "rapnt=" << raPnt << " decpnt=" << decPnt << " pa=" << pa << endl;



        dataSetServer->close(dataSet); // Finished with the file
  
   
    }

    double n = (double)fileName.size();
    raPnt = sum1 / n;

    decPnt = sum2 / n;


    pa = sum3 / n;

 }

/***************************************************************
*
*
****************************************************************/
windowParameters getWindowParameters( string fileName )

{

 
    windowParameters parameters;    
   
    char name[256];
    int stringLength1=fileName.length();
    //strcpy( name, fileName.c_str());

    sprintf(name, "%s", fileName.c_str());
    size_t stringLength2=strlen(name);

 
    unsigned char name3u[256];

    // passing strings to fortran see:
    // http://www.xgear.eu/callfortranfromc.html

    //cout << "L723 name="<< name << " stringLength1=" << stringLength1 << endl;
    //cout << "L729 stringLength2="<< stringLength2 << endl;
    

    for (int ii=0; ii<256; ii++){
      name3u[ii]='\0';
    }
    for (int ii=0;ii<stringLength1; ii++){
      name3u[ii]=(unsigned char)name[ii];
    }

    F90MODPROC(coordinates, windowparameters1)(&name3u[0], &stringLength2, &parameters.x0, 
			       &parameters.y0, &parameters.xBin, &parameters.yBin);
    return parameters;

}
/********************************************************************************
*
********************************************************************************/
void convertCoords( windowParameters &parameter, float &x, float &y)
{
    F90MODPROC( coordinates, convertcoords)( &x, &y, &parameter.x0, &parameter.y0, &parameter.xBin, &parameter.yBin);
}

/********************************************************************************
*
* Load various data from each source-list
*
********************************************************************************/

//bool loadInputData (vector<string> &fileName, 
//                    map<string, vector < vector<OmSourceParameter> > > &sourceListMap)
bool loadInputData (vector<string> &fileName, char *revolution, float filterExposure[], 
		    float &minExposure, float &maxExposure, map<string, 
		    vector < vector<OmSourceParameter> > > &sourceListMap)
{
  printf("L808 revolution=%s \n", revolution);
  int revLength=strlen(revolution);
  //float filterExposure[6]={};
  int filterIndex=-1;

  //float minExposure = 1e6;
  //float maxExposure = -minExposure;

  cout << "L861 running the task loadInputData \n"; 
  printf("L862 revolution=%s revLength=%d fileName.size=%d\n", revolution, revLength, fileName.size());
  for(unsigned i = 0; i < fileName.size(); i++)
    {

      int size = 0;
      string filter;
      float exposure;
      vector<OmSourceParameter> source;
      cout << "L870 i=" << i << " fileName[i] : " << fileName[i] << endl;
      if ( !loadSourceParameters( fileName[i], "SRCLIST", size, source, filter, exposure)) 
        {
	  
	  cout << "L874 could not load file name " << fileName[i] << endl;  
	  return false;
        }
      else
	{
	  cout << "L879 Successfully loaded data from " << fileName[i] << endl;  
	  cout << "L880 source.size=" << source.size() << " filter=" << filter 
	       << " exposure=" << exposure << endl; 
	  
	  //for(unsigned ii = 0; ii < source.size(); ii++){
	  
	  if (filter.compare("UVW2") == 0){
	    filterIndex=0;
	    cout << "L887 the filter is UVW2 filterIndex=" << filterIndex << endl;
	  } else {
	    cout << "L889 the filter is NOT uvw2 " << endl;
	  }
	  if (filter.compare("UVM2") == 0){
	    filterIndex=1;
	    cout << "L893 the filter is UVM2 filterIndex=" << filterIndex << endl;
	  } else {
	    cout << "L895 the filter is NOT uvm2 " << endl;
	  }
	  if (filter.compare("UVW1") == 0){
	    filterIndex=2;
	    cout << "L899 the filter is UVW1 filterIndex=" << filterIndex << endl;
	  } else {
	    cout << "L901 the filter is NOT uvw1 " << endl;
	  }
	  if (filter.compare("U") == 0){
	    filterIndex=3;
	    cout << "L905 the filter is U filterIndex=" << filterIndex << endl;
	  } else {
	    cout << "L907 the filter is NOT u " << endl;
	  }
	  if (filter.compare("B") == 0){
	    filterIndex=4;
	    cout << "L911 the filter is B filterIndex=" << filterIndex << endl;
	  } else {
	    cout << "L913 the filter is NOT b " << endl;
	  }
	  if (filter.compare("V") == 0){
	    filterIndex=5;
	    cout << "L917 the filter is V filterIndex=" << filterIndex << endl;
	  } else {
	    cout << "L919 the filter is NOT v " << endl;
	  }
	  cout << "L921 using filterIndex=" << filterIndex << endl;
 
	  if (fileName[i].find("SWSRLI")!= string::npos){
	    cout << "L924 the file is of the SWSRLI type " << endl;
	    cout << "L925 fileName.find(SWSRLI=" << fileName[i].find("SWSRLI") << endl;
	    cout << "L926 string:npos=" << string::npos << endl;
	    cout << "L905 adding the exposure " << exposure << " to the filter  " << filterIndex << endl;
	    filterExposure[filterIndex]+=exposure;
	    if (exposure<minExposure){
	      minExposure=exposure;
	    }
	    if (exposure>maxExposure){
	      maxExposure=exposure;
	    }
	  } else {
	    cout << "L936 the file is NOT of the SWSRLI type " << endl; 
	  }
	  

	  //} 

	 	 
        } // else 

      cout << "L945 minExposure=" << minExposure << " maxExposure=" << maxExposure << endl;
      cout << "L946 i=" << i << endl;
      int length1=fileName[i].length();
      cout << "L948 i="<< i << " fileName="<< fileName[i]
         << " length1="<<length1 << endl;

      /******************************************************************************
       * Convert x and y coordinates to full-frame coordinates
       *******************************************************************************/
      windowParameters parameter = getWindowParameters( fileName[i] );
      
      for(unsigned j = 0; j < source.size(); j++)
        {
	  
	  convertCoords( parameter, source[j].xcoord, source[j].ycoord );
	  
        }
      
	cout << "L963 source.size=" << source.size() <<  "   i=" << i << endl;
        //cout << "L964 pushing source to the filter"  << end;
	sourceListMap[filter].push_back(source);     
	cout << "---\n";
    } // for i

    /******************************************************************************
    * Get the satellite revolution
    ******************************************************************************/
    cout << "L972 getting the satellite revolution" << endl;
    cout << "L973 fileName[0]=" << fileName[0] << endl;
    
    DataSet *dataSet = dataSetServer->open( fileName[0], Dal::Read);

    if (!dataSet->hasAttribute("REVOLUT" )){
      cout << "L956 No REVOLUT attribute found (using the summary file)\n";      
      revolutionNo=std::string(revolution);
      // test uncommenting ----
      cout<<"L868 revolutionNo="<<revolutionNo<<endl;
      int length1=revolutionNo.length();
      printf("L871 length1=%d \n", length1);
      // test end ^^^
      int length2=revolutionNo.find_last_not_of(" \n\r\t")+1;
      if (length2 <4){
	printf("L963 revolution number is empty \n");
	//revolutionNo="0000";
      }
      printf("L873 length2=%d \n", length2);      
      revolutionNo.erase(revolutionNo.find_last_not_of(" \n\r\t")+1);
    } else{
      revolutionNo = dataSet->attribute("REVOLUT")->asString();
    }

    dataSetServer->close(dataSet); // Finished with the file
    
    cout << "L809 revolutionNo=" << revolutionNo << endl;

    return true;

}

/*************************************************************************
*
*
*
*
*
*
**************************************************************************/
void convertCombinedCoords( map< string, vector< vector<OmSourceParameter> > >  &sourceListMap,   
                            vector < OmSourceParameter > &source)

{ 

    vector< vector<OmSourceParameter> > sourceList;  
    // To store all the source-lists from each SWS file

    map<string, vector< vector<OmSourceParameter> > >::iterator iter;
 
    map<int, vector<OmSourceParameter> > sourceNoMap;

    //cout<<"L853 "<<endl;

    for(iter = sourceListMap.begin(); iter!= sourceListMap.end(); iter++)
    {
        vector< vector<OmSourceParameter> > source1 = iter->second;
	for( unsigned i = 0; i < source1.size(); i++)
	{    
	    sourceList.push_back( source1[i] );         
            for(unsigned j = 0; j < source1[i].size(); j++)
	    {
	        sourceNoMap[ source1[i][j].number - 1 ].push_back( source1[i][j] );

            }
        }

    }

    //cout << "L870 No of sources=" << source.size() << endl;

    map<int, vector<OmSourceParameter> >::iterator iter1;
    for(iter1 = sourceNoMap.begin(); iter1 != sourceNoMap.end(); iter1++)
    {
        if(iter1->second.size() == 0)
        {
            cout << "bug at line 878\n";
	    //exit(0);
	    return;
        }

    }

   
    //cout << "L884 No of input source-lists =" << sourceList.size() << endl;

    if(sourceList.size() == 1) // Only one input source-list file
    {
      cout<<"L887 "<<endl;
        for(unsigned i = 0; i < source.size(); i++)
        {
            source[i].xcoord = sourceList[0][i].xcoord;
            source[i].ycoord = sourceList[0][i].xcoord;           

       }
       return;
    }

    //cout<<"L897 "<<endl;    
    
    /***********************************************************************************
    *
    *
    * More than one input source-list to omsrclistcomb
    * NB source%number is the unique source number in the combined source-list file
    ************************************************************************************/

    //    for(unsigned i = 0; i < source.size(); i++)  // Looping through each source in the combined source list

    for(iter1 = sourceNoMap.begin(); iter1 != sourceNoMap.end(); iter1++)
    {

        int i = iter1->first;
        
     
        Integer2 maxQFlag = 0;        
        Integer1 maxCFlag = 0;        

        float maxRate = -100.0;
     
        
	//	vector<OmSourceParameter> source1 = sourceNoMap[i];
     	vector<OmSourceParameter> source1 = iter1->second;
        
        double sum1 = 0.0, sum2 = 0.0, sum3 = 0.0;
	//      cout << "\n\n\n******************************************************************\n";
	//	cout << "input data for combined source " << i << endl;
        for ( unsigned k = 0; k < source1.size(); k++)
	{
             maxQFlag = max(maxQFlag, source1[k].qualityflag);
             maxCFlag = max(maxCFlag, source1[k].confusionflag);

             maxRate = max(maxRate,   source1[k].rawcountrate);
             double error = max(0.1, (double)source1[k].positionerror);
                 
	     double weight = 1.0 / ( error * error);
             sum1 += (double)source1[k].xcoord * weight;
             sum2 += (double)source1[k].ycoord * weight;

             sum3 += weight;

	     //	     cout << k << ") x=" << source1[k].xcoord << " y=" << source1[k].ycoord << " number=" << source1[k].number << endl;



        }

	source[i].xcoord = sum1 / sum3;
        source[i].ycoord = sum2 / sum3;
   	   

	source[i].qualityflag = maxQFlag; 
	source[i].confusionflag = maxCFlag; 

      	source[i].rawcountrate = maxRate; 
              
	//       cout << i << ") Merged source x=" << source[i].xcoord << " y=" << source[i].ycoord << endl;
              		    
        for ( unsigned k = 0; k < source1.size(); k++)
	{
	  if ( abs( source1[k].xcoord - source[i].xcoord ) > 50.0 ||
               abs( source1[k].ycoord - source[i].ycoord ) > 50.0 )

	  {         
            cout<< "L1116 k="<< k << " x=" << source1[k].xcoord << " y=" 
			  << source1[k].ycoord << endl;
	    cout<< "L1118 i=" << i << " Merged source x=" << source[i].xcoord 
			    << " y=" << source[i].ycoord << endl;

	    cout << "L1121  distance between sources too large \n";
            //exit(0); 
	    return;
	  }

        }
        
    }

};

/***************************************************************************
*
*
*
*
*
*
**************************************************************************/
void resetflags( vector<OmSourceParameter> &source)
{

    for(unsigned i = 0; i < source.size(); i++)
    {
        int qflag = 0, cflag=0;

        if(source[i].qualityflag & (1 << 0))  // Bad pixel
	    qflag |= (1 << 0);

        if(source[i].qualityflag & (1 << 1))  // Read-out streak
	    qflag |= (1 << 1);



	/*********************************************************************
	 * NB SKIP SMOKE RINGS AND DIFFRACTION SPIKES - KNOWN TO BE UNRELIABLE
	 ************************************************************************/
	//        if(source[i].qualityflag & (1 << 2))  // Smoke ring
	//	  qflag |= (1 << 2);

	//        if(source[i].qualityflag & (1 << 3))  // Diffraction spike
	//	  qflag |= (1 << 3);

        if(source[i].qualityflag & (1 << 4))  // Mod-8 pattern
	    qflag |= (1 << 4);




	if(source[i].qualityflag & (1 << 6))  // Central enhancement
	    qflag |= (1 << 5);




	if(source[i].qualityflag & (1 << 5))  // Near a bright source
	    qflag |= (1 << 6);



        if( source[i].confusionflag & (1 << 2))  // near an edge
	{
            qflag |= (1 << 7);

        } 



        if( source[i].extensionflag == 0 && source[i].confusionflag & (1 << 3))  // 
	{
	    qflag |= (1 << 8);     // Point source within an extended-source region

        } 



        if( source[i].confusionflag & (1 << 0))  // 
	{
	    cflag |= (1 << 0);     

        } 


        if( source[i].confusionflag & (1 << 1))  // 
	{
	    cflag |= (1 << 1);     

        } 

        source[i].qualityflag = qflag;

        source[i].confusionflag = cflag;

    }


}

/**************************************************
*
*
**************************************************/
vector<QString> loadDirectoryFiles(const string directory, 
         const int type, const string filter )
{
  
    QDir d( directory  );
    
    d.setFilter( type ); 
    const QFileInfoList *list = d.entryInfoList();

    QFileInfoListIterator it( *list );
    QFileInfo *fi;

    vector<QString> fileList;

    if (filter == "*")
    { 
        while ((fi = it.current()) != 0)
        {
        
            QString s = fi->fileName().latin1();	
            if(s.length() > 4)
	        fileList.push_back( s );
                     
    
            ++it;
        }
    }
    else 
    { 
        while ((fi = it.current()) != 0)
        {
        
            QString s = fi->fileName().latin1();
            	
            if(s.length() > 4 && s.contains(filter) && s.contains("FTZ")  )
	    {
	     
	        fileList.push_back( s );
               
            }
            ++it;
        }
        if(fileList.size() == 0)
	{
	 

            QFileInfoListIterator it( *list );
            QFileInfo *fi;



            while ((fi = it.current()) != 0)
            {
        
     
                QString s = fi->fileName().latin1();
	        
      
                if(s.length() > 4 && s.contains(filter) && s.contains("FIT")  )
	        {
	     
	            fileList.push_back( s );
               
                }
                ++it;
            }
        }
   



    } 

    return fileList;
}



/**************************************************
*
*
**************************************************/
/*
vector<QString> loadSummaryFiles(const string directory, 
         const int type, const string filter )
{
  
  cout<<"L1220"<<endl;
  cout<<"L1221 type="<<type<<endl;
    QDir d( directory  );
    
    d.setFilter( type ); 
    const QFileInfoList *list = d.entryInfoList();

    QFileInfoListIterator it( *list );
    QFileInfo *fi;

    vector<QString> fileList;

    if (filter == "*")
    { 
        while ((fi = it.current()) != 0)
        {
        
            QString s = fi->fileName().latin1();	
            if(s.length() > 4)
	        fileList.push_back( s );
                     
    
            ++it;
        }
    }
    else 
    { 
        while ((fi = it.current()) != 0)
        {
        
            QString s = fi->fileName().latin1();
            	
            if(s.length() > 4 && s.contains(filter) && s.contains("FTZ")  )
	    {
	     
	        fileList.push_back( s );
               
            }
            ++it;
        }
        if(fileList.size() == 0)
	{
	 

            QFileInfoListIterator it( *list );
            QFileInfo *fi;



            while ((fi = it.current()) != 0)
            {
        
     
                QString s = fi->fileName().latin1();
	        
      
                if(s.length() > 4 && s.contains(filter) && s.contains("FIT")  )
	        {
	     
	            fileList.push_back( s );
               
                }
                ++it;
            }
        }
   



    } 

    return fileList;
}
*/

/**************************************************
 *
 *
 *************************************************/
 //const string directory,
 //vector<QString> loadSummaryFiles(char *filter, char c)
vector<QString> loadSummaryFile(const string directory, char *filter, char *revolution)
{
  
  //cout<<"L1306"<<endl;
  //cout<<"L1307 filter="<< filter << endl;

  //QDir d;
  QDir d( directory  );
  //char revolution[5]={' ',' ',' ',' ','\0'};  
  d.setFilter (QDir::Files | QDir::Hidden |
               QDir::NoSymLinks);
  //    d.setSorting(QDir::Size | QDir::Name); // Sort by name
  d.setSorting(QDir::Name); // Sort by name

  const QFileInfoList *list = d.entryInfoList();

  QFileInfoListIterator it(*list);
  QFileInfo *fi;

  vector<QString> fileList;
  
  while ((fi = it.current()) != 0)
    {
      
      QString s = fi->fileName().latin1();
      //cout<<"L1328  s="<<s<<endl;
      //if (s[0] == c && s.contains(filter) && s.contains("FIT") && !s.contains("new") )
      if (s.contains(filter) && s.contains("SAS"))
	{
	  //cout<<"L1328  s="<<s<<"  PUSHING ... "<< endl;
	  
	  fileList.push_back(s);             

	  strncpy(revolution, (char*) s.ascii(), 4);
	  //printf("L1341 revolution=%s \n", revolution);
        }
      ++it;
    } 
  return fileList;
  
}




/************************************************************************
* For each source in the combined source-list, for each filter we
* need the number of exposures (sources merged)
* size is the number of sources in the combined source-list
*************************************************************************/
void addExposureNumbers( int size, 
   map<string,  vector< vector<OmSourceParameter> > > &sourceListMap)
{
  //cout<< "L1189 size="<<size<<endl;
  string file = stringParameter("outset");
  //cout<<"L1191 file="<< file << endl;
     
  map<string, vector< vector<OmSourceParameter> > >::iterator iter;

  //cout << "L1195\n";

  //    for(iter = sourceListMap.begin(); iter != sourceListMap.end(); iter++ )

  for(unsigned k = 0; k < omFilter.size(); k++)
    {

      //        string filter = iter->first;  // The filter
      string filter = omFilter[k];
      //cout << "L1206 filter="<<filter << endl;
      iter = sourceListMap.find( filter );
      //cout<<"L1208 "<< endl;
      if(iter != sourceListMap.end() ) // Filter exists
	{         
	  //cout<<"L1211 filter exists "<<endl;
	  vector< vector<OmSourceParameter> >  sourceList = iter->second;
	  //vector<int16> number(size);
	  vector<int16> number(size,0);
      
	  //cout<<"L1215 sourceList size="<< sourceList.size()<<endl;
	  for(unsigned i = 0; i < sourceList.size(); i++)
	    {
	      //cout<<"L1218 i="<<i<<endl;
	      for(unsigned j = 0; j < sourceList[i].size(); j++)
	        {
		  
		  //number[ sourceList[i][j].number - 1]++;         
		  number[ sourceList[i][j].number - 1,0]++;  
                }
 

            }
	  //cout<<"L1228 number size="<< number.size() <<endl;
	  for(unsigned i = 0; i < number.size(); i++){
	    //cout<<"L1230 i="<<i<<" number[i]="<< number[i,0]<<endl;
	    if(number[i,0] == 0){ 
	      number[i,0] = 0;
	    }
	    //if(number[i] == 0){ 
	    //  number[i] = 0;
	    //}
	  }
	  //cout<<"L1235"<<endl;
	  string columnName = "N_" + filter + "_EXP";
	  string comment = "Number of " + filter + " exposures during observation";
	  //cout<<"L1238 columnName="<< columnName<<endl;
	  //cout<<"L1239 comment="<<comment<<endl;
	  /***************************************************************
	   *
	   ****************************************************************/ 
	  
	  //cout<<"L1243 size="<<size<<" number_size="<< number.size()<<" file="<< file << endl;
	  
	  if ( !addColumn( file,
			   "SRCLIST", 
			   columnName, 
			   "",
			   comment,   
			   size,
			   number) )
            {
	      
	      cout<<"L1252 addColumn error"<<endl;
            }
	  

	  //cout<<"L1254 "<<endl;
	}
      else // No sources in this filter
	{
	  //cout<<"L1257 no sources in this filter "<<endl;
	  vector<int16> number(size, 0);
	  string columnName = "N_" + filter + "_EXP";
	  string comment = "Number of " + filter + " exposures during observation";
	  //cout<<"L1261 columnName="<< columnName<<endl;
	  //cout<<"L1262 comment="<<comment<<endl;	  
	  //cout<< "L1269 size="<< size<<endl;
	  //vector< vector<OmSourceParameter> >  sourceList = iter->second;
	  
	  //cout<<"L1272 sourceList size="<< sourceList.size()<<endl;
	  //for(unsigned i = 0; i < sourceList.size(); i++)
	  //  {
	  //    cout<<"L1273 i="<<i<<endl;
	  //    for(unsigned j = 0; j < sourceList[i].size(); j++)
	  //      {
	  //	  cout << "L1276 j="<< j << " number=" <<  sourceList[i][j].number << endl;
	  //	  number[ sourceList[i][j].number - 1]++;         
	  //
          //      }
          //  }

	  //cout<<"L1282 number size="<< number.size()<< " size="<< size <<endl;
	  //for(unsigned i = 0; i < number.size(); i++){
	  //  cout<<"L1284 i="<<i<<" number[i]="<< number[i]<<endl;
	  //  if(number[i] == 0){ 
	  //    number[i] = 0;
	  //  }
	  //}
	  /***************************************************************
	   *
	   ****************************************************************/  
	  if ( !addColumn( file,
			   "SRCLIST", 
			   columnName, 
			   "",
			   comment,   
			   size,
			   number) )
            {
	      
	      cout<<"L1276 addColumn error"<<endl;   
            } else{
	    //cout<<"L1284 addColumn OK"<<endl;
	  }
	  
	  
	  
        }
      
      //cout<<"L1291 finished adding a filter "<<endl;
    }
  
  cout<<"L1570 finished adding filters "<<endl;
}



    //The old flags
    //!****************************************************************************
    //! QFLAG source quality flags convention
    //! BIT NO             FLAGGING                         Integer
    //!
    //!   0                BAD PIXEL                                 1
    //!   1                READ-OUT STREAK                           2
    //!   2                SMOKE-RING                                4
    //!   3                SOURCE ON STAR-SPIKE                      8
    //!   4                Mod-8 source                             16
    //!   5                Source lies near to a bright source      32
    //!   6                Source within central enhancement        64
    //!*****************************************************************************
    //!****************************************************************************
    //! CFLAG source confusion flags convention
    //! BIT NO             FLAGGING                                           Integer
    //!
    //!   0                one or more source within 0.5-1.0 aperture radius  1 
    //!   1                one or more source within 0.5 aperture radius      2
    //!   2                point source lies with 1.0 aperture radius of      4
    //!                edge or some point of extended source lies on edge      
    //!   3           Point-source lies partly or entirely within extended    8
    //!                source region 
    //!*****************************************************************************


    //THe new ones


    //   "0) Source on a bad pixel                       (red)                                                   ",   &
    //   "1) Source on a read-out streak                 (magenta)                                               ",   &
    //   "2) Source near or within a smoke-ring          (yellow)                                                ",   &
    //   "3) Source on or near a diffraction-spike       (white)                                                 ",   &
    //   "4) Source surrounded by a strong mod-8 pattern (black)                                                 ",   & 
    //   "5) Source within central enhanced region       (red)                                                   ",   &
    //   "6) Source lies near to a bright source         (black)                                                 ",   &
    //   "7) Point source lies with 1.0 aperture radius of edge/some point of extended source lies on edge (blue)",   &
    //   "8) Point-source lies partly or entirely within extended source region (red)                            ",   &
    //   "9) Compact source/exceptionally strange        (red)                                                   "/)    





/************************************************************************
* 
* 
*
*
*************************************************************************/
void addVegaMags( )
{
  

    string inFile = stringParameter("set");
    DataSet *inDataSet = dataSetServer->open( inFile, Dal::Read);
    Table *inTable = inDataSet->table( "SRCLIST", Dal::Read);
 


    string outFile = stringParameter("outset");
    DataSet *outDataSet = dataSetServer->open( outFile, Dal::Modify);
    Table *outTable = outDataSet->table( "SRCLIST", Dal::Modify);
 


    /**************************************************************************
    *
    * Modify each source position error by adding the astrometry RMS value
    *
    **************************************************************************/
    modifyPositionErrors(inTable, outTable);




     
    int size = (int)outTable->rows();

    for(unsigned k = 0; k < omFilter.size(); k++)
    {

        string filter = omFilter[k];
        vector<float> vegaMag, vegaMagErr;
     
   

        string columnName = filter + "_INS_MAG";

        /************************************************************************
	* Check to see if this columnn exists
	***********************************************************************/
        if (inTable->hasColumn( columnName) ) 
	{          

            const ColumnData *columnData = inTable->column(columnName)->data();
      
            float *xData = columnData->real32Data();

            columnName = filter + "_INS_MAG_ERR";

            const ColumnData *columnData1 = inTable->column(columnName)->data();
      

            float *xDataErr = columnData1->real32Data();

	    //         cout << "infile=" << inFile << " column=" << columnName << endl;
            for(int i = 0; i < size; i++)
	    {
	 
	        vegaMag.push_back( xData[i] );
	        vegaMagErr.push_back( xDataErr[i] );
	  
		//		cout << i << ") vega mag=" << vegaMag[i] << "+/-" << vegaMagErr[i] << endl;
	
            }
            delete columnData;
            delete columnData1;

          

        }
        else  // Column doesn't exist
	{


            for(int i = 0; i < size; i++)
	    {
	        vegaMag.push_back( -9999.0);
	        vegaMagErr.push_back( -9999.0);;
	             	
            }
      

        }

        columnName = filter + "_VEGA_MAG";
        string comment = "Source Vega magnitude through the " + filter + " filter"; 
        /***************************************************************
	*
        ****************************************************************/  

        if ( !addColumn( outTable, 
                         columnName.c_str(), 
                         "mag",
                         comment.c_str(),   
	                 size,
	                 vegaMag) )

        {


           
	}
        /***************************************************************
	*
        ****************************************************************/

        columnName = filter + "_VEGA_MAG_ERR";
        comment = "1-sigma Vega magnitude through the " + filter + "filter"; 
  
        if ( !addColumn( outTable, 
                         columnName.c_str(), 
                         "mag",
                         comment.c_str(),   
	                 size,
	                 vegaMagErr) )
        {


           
	}




   
    }








    dataSetServer->close( inDataSet ); // Finished with the file
 
    dataSetServer->close( outDataSet ); // Finished with the file
 
}
/*****************************************************************
* Get the size of the image
*****************************************************************/
bool  fileOkay( string file )
{
  
  /***************************************************************
   *
   * First check when file last modified
   *
   **************************************************************/
  //printf("L1496 checking the file \n");
  QFileInfo fi(file);
  QDateTime time = fi.lastModified();
  QString date = time.toString("ddd MMMM yyyy");

  // cout << "L1500 date=" << date << endl;
  /*
  if ( !date.contains( "2008") ){
    printf("L1504 date does not contain 2008: returning false \n");
    return false;
  } else {
    printf("L1507 date does contain 2008: returning true \n");
  }
  */
  /*
  if ( !date.contains( "May") && !(date.contains("June") || date.contains("July")) ){
    printf("L1510 date does not contain May, June July: returning false \n");
    return false;
  }else {
    printf("L1513 date contain either May, June or July: returning true \n");
  }
  */

  //printf("L1515 opening data server \n");
  DataSet* set = dataSetServer->open( file, DataSet::Read);
  //printf("L1517 opening table \n");
  Table *table = set->table("SRCLIST");
  if (!table->hasColumn( "SRC_ID")) {
    //printf("L1520 the table does not contain the column SRC_ID: returning false \n");
    dataSetServer->close(set);
    return false;

  } else {
    //printf("L1525 the table contains the column SRC_ID: returning true \n");
  }

  //printf("L1528 reading attribute BINBPE \n");
  bool binbpe = set->attribute("BINBPE")->asBool();
  //printf("L1530 reading attribute BINAX1 \n");
  int binax1 = set->attribute("BINAX1")->asInt();
  //printf("L1532 checking the attributes \n");

  bool okay = true;
  if (binax1 == 0 && !binbpe)
    {
      //printf("L1537 either binax1=0 or binbpe is false \n");
      int naxis1 =  set->attribute("WINDOWDX")->asInt();
      int naxis2 =  set->attribute("WINDOWDY")->asInt();
      //printf("L1540 naxis1=%d naxis2=%d \n", naxis1, naxis2);
      if(naxis1 < 256 && naxis1 == naxis2){
	//printf("L1542 either naxis1<256 or naxis1=naxis2: returning false \n");
	okay = false; 
      }
    }
  //printf("L1546 closing the data set \n");
  dataSetServer->close(set);
  //cout << "L1548 returning the code okay=" << okay << endl;
  return okay;

}

bool  summaryFileOkay( string file )
{
  

  printf("L1624 checking the file \n");
  QFileInfo fi(file);
  QDateTime time = fi.lastModified();
  QString date = time.toString("ddd MMMM yyyy");

  cout << "L1629 date=" << date << endl;

 
  bool okay = true;

  return okay;

}


/**************************************************************************
*
* Modify each source position error by adding the astrometry RMS value
*
**************************************************************************/
void modifyPositionErrors(Table *&inTable, Table *&outTable)
{

    
    if( !inTable->hasAttribute( "RMSRESID" ))
      return;

    float rms = inTable->attribute( "RMSRESID" )->asReal();
    float rmsSquared = rms * rms;
    const ColumnData *columnData1 = inTable->column("POSERR")->data();      
    const float *xData1 = columnData1->real32Data();



   
    ColumnData *columnData2 = outTable->column("POSERR")->data();      
    float *xData2 = columnData2->real32Data();

    for(unsigned i = 0; i < inTable->rows(); i++)
    {
        xData2[i] = sqrt( xData1[i] * xData1[i] + rmsSquared); 


    }

    delete columnData1;
    delete columnData2;
 


}



