! ESA (C) 2000-2018 
!
! This file is part of ESA's XMM-Newton Scientific Analysis System (SAS).
!
!    SAS is free software: you can redistribute it and/or modify
!    it under the terms of the GNU General Public License as published by
!    the Free Software Foundation, either version 3 of the License, or
!    (at your option) any later version.
!
!    SAS is distributed in the hope that it will be useful,
!    but WITHOUT ANY WARRANTY; without even the implied warranty of
!    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
!    GNU General Public License for more details.
!
!    You should have received a copy of the GNU General Public License
!    along with SAS.  If not, see <http://www.gnu.org/licenses/>.

!**************************************************************
! 
!***************************************************************
module smokerings_mod

    
   use param
   use cal
   use dal
   use sla

   use errorHandling  
   use omallocation_mod
  
   use omgraph_mod  
   use omnumerics_mod
   use statistics_mod
   use omutility_mod
   use leastsquares_mod
   use getfilterid_mod 
   use omconstants_mod
   implicit none


 


  
contains

!********************************************************
!
!*******************************************************
subroutine smokerings
    real(single), pointer, dimension(:) ::  xStar, yStar, xSmoke, ySmoke
    real(single), pointer, dimension(:) :: rStar, rRing, error, residual


    integer :: i, j, nData
    real(double) :: dx, dy
    real(single) ::  gradient, gradientError, constant, constantError
    real(single) :: x, y, fit, xcoeff(20), dxCoeff(20), std   
    character(100) :: text       
    nData = 0
  

    call CAL_setState(OM, filterId = filterIdToInt("V"))

 

    call loadData(nData, xStar, yStar, xSmoke, ySmoke)





    allocate(rstar(1 : 12))
    allocate(rRing(1 : 12))
    allocate(error(1 : 12))
    allocate(residual(1 : 12))

    error = 0.0
    do i = 1, 12
        dx = xcentre - xStar(i)
        dy = ycentre - ystar(i)
        rstar(i) = sqrt(dx * dx + dy * dy)


        dx = xcentre - xsmoke(i)
        dy = ycentre - ysmoke(i)
        rring(i) = sqrt(dx * dx + dy * dy)


    end do


    call pgbegin(0, "smokerings.ps/VCPS", 1, 1)
   

    


    !*******************************************************************
    ! RA OM Pointing direction
    !******************************************************************
   
    
    call DrawGraph(nData, rStar, rring, &

                  "SMOKE RINGS (DISTORTED)", "STAR RADIUS", "SMOKE RING RADIUS", &
                  0.1, 0.9, 0.1, 0.9 )




    do i = 2, nData - 1

        call leastsquaresPolyFit(nData, xstar, xsmoke, error, int(i, int8), 1_int8, xcoeff, dxcoeff, std)
        write(*, *) "Least-squares polynomial fit- order ", i, " std=", std
        do j = 1, i + 1
            write(*, *) j, ") coeff=", xcoeff(j), "+/-", dxcoeff(j)
        end do

    end do




    call LeastSquaresFit(rStar, rRing, error, gradient, gradientError, constant, constantError)
   
    
    x = minval(rStar)
    y = gradient * x + constant
    call pgmove(x, y)

    x = maxval(rStar)
    y = gradient * x + constant
    call pgdraw(x, y)

    write(text, '(a, f7.4, a, f7.4)') "Least squares line- Y = ", gradient, " X + ", constant
    call pgmtxt('T', -1.0, 0.0, 0.0, trim(text))
   




    call pgpage

    !***********************************************************************
    !
    !**********************************************************************#

    do i = 1, nData
        fit = rStar(i) * gradient + constant
        residual(i) = rring(i) - fit
       
        
    end do


   
    call DrawGraph(nData, rring, residual, &

                  "SMOKE RINGS (DISTORTED)", "TRUE SMOKE RING RADIUS", "RESIDUAL (TRUE - PREDICTED)", &
                  0.1, 0.9, 0.1, 0.9 )




    call pgpage







    do i = 1, 12
        
        call RemoveDistortion(xstar(i), ystar(i), xstar(i), ystar(i))



        dx = xcentre - xStar(i)
        dy = ycentre - ystar(i)
        rstar(i) = sqrt(dx * dx + dy * dy)

        call RemoveDistortion(xsmoke(i), ysmoke(i), xsmoke(i), ysmoke(i))
  

        dx = xcentre - xsmoke(i)
        dy = ycentre - ysmoke(i)
        rring(i) = sqrt(dx * dx + dy * dy)


   
    end do




    do i = 1, 12
        write(*, *) i, xstar(i), ystar(i), xsmoke(i), ysmoke(i)

    end do





    call DrawGraph(nData, rstar, rring, &

                  "SMOKE RINGS (UNDISTORTED)", "STAR RADIUS", "SMOKE RING RADIUS", &
                  0.1, 0.9, 0.1, 0.9 )

    call LeastSquaresFit(rStar, rRing, error, gradient, gradientError, constant, constantError)


    x = minval(rStar)
    y = gradient * x + constant
    call pgmove(x, y)

    x = maxval(rStar)
    y = gradient * x + constant
    call pgdraw(x, y)



    write(text, '(a, f7.4, a, f7.4)') "Least squares line- Y = ", gradient, " X + ", constant
    call pgmtxt('T', -1.0, 0.0, 0.0, trim(text))
 




    call pgpage


    !***********************************************************************
    !
    !**********************************************************************#

    do i = 1, nData
        fit = rStar(i) * gradient + constant
        residual(i) = rring(i) - fit
               
    end do

   
    call DrawGraph(nData, rring, residual, &

                  "SMOKE RINGS (UNDISTORTED)", "TRUE SMOKE RING RADIUS", "RESIDUAL (TRUE - PREDICTED)", &
                  0.1, 0.9, 0.1, 0.9 )


    call pgpage






    call pgclos
 
    if (associated(xStar)) then
        deallocate(xStar, yStar, xSmoke, ySmoke, rring, rstar, error, residual)
    end if
end subroutine smokerings
!************************************************************************
!
!
!***********************************************************************


!**************************************************************
! Corrects position x and y for distortion
!**************************************************************
subroutine RemoveDistortion(x, y, xOut, yOut)

    real(single), intent(in) :: x, y
    real(single), intent(out) :: xOut, yOut
    integer(int32) :: mode 
    real(single), pointer, dimension( : ) :: rawx, rawy, deltax, deltay, sigmax, sigmay 
 
    allocate(rawx(0 : 0))
    allocate(rawy(0 : 0))

    !****************************************************! 
    ! compute absolute position of object within OM frame
    !****************************************************
    rawx(0) = x   
    rawy(0) = y 
   
    !***************************************************
    ! correct positions for pincushion/barrel distortion
    !***************************************************
  
    mode = 1
    call CAL_omDistortion(rawx, rawy, deltax, deltay, sigmax, sigmay, mode)


 
    xOut = x - real(deltax(0))      
    yOut = y - real(deltay(0)) 


end subroutine RemoveDistortion
!*********************************************************************
!
!*********************************************************************
subroutine loadData(nData, xStar, yStar, xSmoke, ySmoke)
    integer, intent(out) :: nData

    real(single), pointer, dimension(:) ::  xStar, yStar, xSmoke, ySmoke
    real(single) x
    integer :: i
    

    open(unit = 1, status = "old", file = "/home/ssco2/smoke_ring_distorted_coords_fullres.txt")
    

    allocate(xstar(1 : 12))
    allocate(ystar(1 : 12))
    allocate(xsmoke(1 : 12))
    allocate(ysmoke(1 : 12))


    do i = 1, 12
        read(1, *, end = 1) xstar(i), ystar(i)
       

   
    end do




    do i = 1, 12
        read(1, *, end = 1) xSmoke(i), ySmoke(i)
       


      
    end do

    do i = 1, 12
        write(*, *) i, xstar(i), ystar(i), xsmoke(i), ysmoke(i)

    end do
    nData = 12
    close (1)

    return
1   call fatal("bug")

end subroutine loadData


end module smokerings_mod















