! ESA (C) 2000-2018 
!
! This file is part of ESA's XMM-Newton Scientific Analysis System (SAS).
!
!    SAS is free software: you can redistribute it and/or modify
!    it under the terms of the GNU General Public License as published by
!    the Free Software Foundation, either version 3 of the License, or
!    (at your option) any later version.
!
!    SAS is distributed in the hope that it will be useful,
!    but WITHOUT ANY WARRANTY; without even the implied warranty of
!    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
!    GNU General Public License for more details.
!
!    You should have received a copy of the GNU General Public License
!    along with SAS.  If not, see <http://www.gnu.org/licenses/>.
!**********************************************************
! Program for producing the OM catalogue
!
!
! 
! NB use integer8, integer16 and integer32 and
! not int8, int16 and int32
!
!
!*********************************************************
module  ommastercatalogue_mod

  use dal
  use sasdal
  use param
  use omsourceparam_mod  ! For definition of null value
  use ommasterdefs_mod
  use omconstants_mod
  use omutility_mod
  use omnumerics_mod
  use omastromcorrect_mod
  use omallocation_mod
  use omsourceflags_mod
  use omnumerics_mod
  use sla
  !use IEEE_ARITHMETIC  ! For checking for Nans, etc

  use, intrinsic :: IEEE_ARITHMETIC

interface Peirce
    module procedure Peirce1, Peirce2
end interface Peirce


interface isNanvalue
    module procedure isnanvalue1, isnanvalue2
end interface isnanvalue

  integer(int32) :: raCompare1Index = 0

  integer(int16), private :: nocolumns2 = 9
  integer(int16), private :: nocolumns3 = 3




  !**************************************************************
  ! This is for the input SRCLIST table   
  !**************************************************************
  integer(int16), private :: nocolumns1 = 30
  type(newColumn), private, save, dimension(200) :: columndata1


  type(newColumn), private, save, dimension(200) :: columndata2
  type(newColumn), private, save, dimension(200) :: columndata3


  !**************************************************************
  ! This is for the SRCLIST table in the catalogue  
  !**************************************************************
  integer(int16), private :: nocolumns4 = 30
  type(newColumn), private, save, dimension(200) :: columndata4





  !**************************************************************
  ! This is for the AVERAGED_MULTIPLE_DETECTIONS table in the catalogue  
  !**************************************************************
  integer(int16), private :: nocolumns5 = 30
  type(newColumn), private, save, dimension(200) :: columndata5








interface createNewColumns 
    module procedure createnewColumns1, createnewcolumns2
end interface


interface averagedata 
    module procedure averagerealdata, averagedoubledata
end interface averagedata


interface AverageTableData
    module procedure  averagerealTabledata, averagedoubleTabledata
end interface AverageTableData


!***************************************************************************
! The OM filters
!***************************************************************************



character(5), private, pointer :: omFilter(:) => null()
integer(int8) :: noFilters = 0

logical, private :: uvOpticalCat = .false.



!*****************************************************************
! This stores the new row number in the SUMMARY_NO for
! the new observation data 
!****************************************************************
integer(int32), private :: obs_summary_no = 1 



!********************************************************************
!
! The following is to limit the maximum number of sources in
! an observation source-list file
!
!*******************************************************************
integer(int32), private, parameter :: maxNumberOfRows = 200000

!********************************************************
! For debugging
!*******************************************************
logical, private :: debug=.false.
contains


!****************************************************************************
! Creates an OM catalogue from individual combined source-list files
!
!***************************************************************************
subroutine  ommastercatalogue

    integer :: i
    character(256) :: oldmasterCatalogue = "oldmastercatalogue.fits"
    character(256) :: newmasterCatalogue = "newmastercatalogue.fits"
    character(256) :: srclistfile = ""
    character(256) :: tmpsrclistfile = "TEMPSOURCELIST.FITS"
    character(256) :: outFolder ="out"
    character(256) :: masterSourceList0 ="MASTERSOURCELIST0.FITS"
    character(256) :: finalCatalogueName ="MASTERCATALOGUE.FITS"
    character(256) :: finalCatalogue = "mastercatalogue.fits"

    integer :: prefix ! if not negative: index to extend SRCNUM, otherwise no extension

!**************************************************
! The routine addIAUCoordinates was a last-minute addition to the catalogue:
! commenting it (vny 2012-12-08)
!*************************************************    
!    call addIAUCoordinates( "omcatalogue.fits" )
!    
!    stop
!*************************************************



!*****************************************************
!
!*****************************************************

!call modifycolumns
!stop


!call testPeirce

   
    newmastercatalogue = trim(stringParameter("newmastercatalogue")) ! The new master file
    write(*,*) "L169 newmastercatalogue=", trim(newmastercatalogue)
    
    if( setExists(newMasterCatalogue)) then
       !call fatal("new catalogue file already exists - can't overwrite it!")
       write(*,*) "L171 the file newMasterCatalogue already exists - will be overwritten" 
    end if

    outFolder = trim(stringParameter("outdirectory")) ! The output folder name 
    write(*,*) "L175 outFolder=", trim(outFolder)

    prefix= intParameter("prefix")
    write(*,*) "L90 prefix=", prefix

    !********************************************************
    !
    !
    !*******************************************************
    call setUpFilters()

    !********************************************************************
    !
    !*********************************************************************
    debug = .true.

    call setUpColumnNames
    !debug = .false.

    oldmastercatalogue = trim(stringParameter("oldmastercatalogue")) ! The old master file 
    write(*,*) "L194 oldmastercatalogue=", trim(oldmastercatalogue)
    write(*,*) "L195 boolean parameter finish=", booleanParameter("finish") 
    finalCatalogue = trim(stringParameter("finalcatalogue")) ! The final catalogue file name
 
    !******************************************************************
    ! Create the final catalogue
    !*****************************************************************  
    if (booleanParameter("finish")) then
       write(*,*) "L201 applying the boolean parameter finish"
       call finish( oldMasterCatalogue, newMasterCatalogue)
       write(*, *) "L203 end finalising newMasterCatalogue"

       !***********************************************************************
       !
       ! Carry out various checks
       !
       !***********************************************************************
       call checkForErrors
       !***********************************************************************
       !
       ! Change various column names
       !
       !***********************************************************************
       call modifycolumns( newMasterCatalogue )
       
       !*****************************************************************************
       !
       ! Create the final catalogue
       ! The routine just removes junk from the fits file
       ! and copies the columns of MASTERCATALOGUE.FITS to MASTERCATALOGUE.FITS 
       !*****************************************************************************
       !finalCatalogueName= trim(outFolder)  // "/MASTERCATALOGUE.FITS"
       finalCatalogueName= trim(finalCatalogue) 

       write(*,*) "L226 final catalogue name: ", trim(finalCatalogueName)
       !call createFinalCatalogue( newMasterCatalogue, "MASTERCATALOGUE.FITS")
       call createFinalCatalogue( newMasterCatalogue, finalCatalogueName)
       
       !call addIAUCoordinates( "MASTERCATALOGUE.FITS" )
       call addIAUCoordinates( finalCatalogueName )
       write(*,*) "L232 finished creating the final version of the catalogue"
       stop
    end if


    !*************************************************************************
    ! Check astrometry- ignore if POSCORROK = false
    !*************************************************************************
    srclistfile = trim(stringParameter("srclistfile"))  ! The file to be merged with the old master file
    tmpsrclistfile = trim(stringParameter("tmpsrclistfile"))
 
    write(*,*) "L235 srclistfile=", trim(srclistfile)
    
    if (.not.  goodAstrometry( trim(srcListFile) ) ) then
       call warning("Bad astrometry - ignoring file", trim(srcListFile))        
       stop
    end if

    !************************************************************************
    ! Copy the input source-list file to "TEMPSOURCELIST.FITS", since
    ! some extra data is added to create the catalogue
    !************************************************************************

    !call copydataSet( trim ( srclistfile ), trim(outFolder) // "/TEMPSOURCELIST.FITS" )
    call copydataSet( trim ( srclistfile ), trim(tmpsrclistfile) )

    srclistFile = trim(tmpsrclistfile)

    write(*,*) "L250 srcListFile=", trim(srcListFile)
    !****************************************************************
    ! If creating a UV-OPT catalogue then we only want
    ! to keep sources that have at least one UV-measurment
    !***************************************************************
    if (.not. setExists( trim(oldmasterCatalogue))) then
       write(*,*) "L255 the file oldMasterCatalogue does not exist (will create from scratch) "
    else
       write(*,*) "L257 the file oldMasterCatalogue already exists "
    endif
    if (booleanParameter("create")) then
       write(*,*) "L260 requested creation of a new catalogue from scratch, create=", &
            booleanParameter("create")
    else
       write(*,*) "L263 the creation of a new catalogue is not requested, create=", &
            booleanParameter("create")
    endif
    
    if (.not. setExists( trim(oldmasterCatalogue)) .or. booleanParameter("create") ) then
       write(*,*) "L268 Creating from scratch (either oldMasterCatalogue does not exist or a new creation requested) "
       !******************************************
       ! Create a new catalogue fits file
       !***************************************
       write(*,*) "L272 uvOpticalCat=", uvOpticalCat
       if (uvOpticalCat) then
          write(*,*) "L274 UV-OPT catalogue type"
          if(debug)   write(*, *) "L263 refining the source list"
          
          call refineSourceList2( trim ( srclistfile ) )

       end if

       if(debug)   write(*, *) "L281 sorting the file"

       call sortFile( srcListFile )

       if(debug)   write(*, *) "L285 finished sorting the file"

       
       call setStringParameter( "srclistfile", trim(srclistfile))
       write(*,*) "L309 ********* here the input source list does not yet contain nfilter-sources ***** "
       if(debug) write(*,*) "L310 srclistfile=", trim(srclistfile)
       
       call CreateCatalogue( newmastercatalogue, srclistfile )
       if(debug) write(*,*) "L313 finished creating a fresh copy of the catalogue"
       
       write(*,*) "L315 finished creating a fresh copy of the catalogue"

       call updateOBSSummaryTable( newmasterCatalogue, srcListFile )
       if(debug) write(*,*) "L319 finished updating the summary table"
       write(*,*) "L319 finished updating the summary table"

       !masterSourceList0= trim(outFolder) // "/MASTERSOURCELIST0.FITS"
       masterSourceList0 = trim(stringParameter("tmpmastercatalogue"))
       write(*,*) "L323 masterSourceList0=", trim(masterSourceList0)

       call copydataset( trim(newmastercatalogue), trim(masterSourceList0))
       if(debug) write(*,*) "L326 copied the new mastercatalogue to ", &
            trim(masterSourceList0) 
       write(*,*) "L326 copied the new mastercatalogue to ", &
            trim(masterSourceList0) 

       !****************************************************************
       ! Merge the observation file with the newly created catalogue
       !****************************************************************
       if(debug) write(*,*) "L334 merging the observation file with the catalogue" 
       write(*,*) "L334 merging the observation file with the catalogue" 
       write(*,*) "L336 masterSourceList0=", trim(masterSourceList0)
       write(*,*) "L337 newmasterCatalogue=", trim(newMasterCatalogue)
       !call MergeSourceLists( "MASTERSOURCELIST0.FITS", srclistfile, newmasterCatalogue, &
       call MergeSourceLists( masterSourceList0, srclistfile, newmasterCatalogue, &
            "SRCLIST",  nocolumns1, columndata1, compare1)
       if(debug) write(*,*) "L341 setting the source numbers to each source " 
       write(*,*) "L341 setting the source numbers to each source " 
       !************************************************************************************
       ! Set the source number of each source
       ! 
       !************************************************************************************ 
        !call setUniqueSrcNumber( "MASTERSOURCELIST0.FITS" , "SRCLIST" )
        !call setUniqueSrcNumber( masterSourceList0 , "SRCLIST" )
       write(*,*) "L349 newMasterCatalogue=", trim(newMasterCatalogue)
       call setUniqueSrcNumber( newMasterCatalogue , "SRCLIST" )
       if(debug) write(*,*) "L320 finished ingesting a source list"
    
    else ! The catalogue already exists

       if(debug) write(*,*) "L355 copying the data from old to new master catalogue"
       write(*,*) "L355 copying the data from old to new master catalogue"
       call copydataSet( trim ( oldMasterCatalogue), trim ( newmasterCatalogue ) )
       
       !write(*,*) "L342 testing the primary block header"
       !call testHeader( newmasterCatalogue )
       !write(*,*) "L344 finished testing the primary block header"
       
       !****************************************************************************************
       ! Update the SUMMARY table and check that the observation doesn't already
       ! exist in the catalogue
       !***************************************************************************************   
        if(debug) write(*,*) "L367 Updating the summary table "
        write(*,*) "L367 Updating the summary table "

        call updateOBSSummaryTable( newmasterCatalogue, srcListFile )
        !write(*,*) "L331 uvOpticalCat=", uvOpticalCat
        if (uvOpticalCat) then
           
           if(debug)   write(*, *) "L334 using the option UV-OPT catalogue type "
           write(*, *) "L375 using the option UV-OPT catalogue type "
           call refineSourceList2( trim ( srclistfile ) )
           
        end if

        if(debug)   write(*, *) "L380 sorting the file "
        call sortFile( srcListFile )        
        if(debug)   write(*, *) "L382 finished sorting the file"

        call setStringParameter( "srclistfile", trim(srclistfile))
        !if(debug)   write(*, *) "L347 srcliostfile=", trim(srclistfile)

        !************************************************************************************
        ! Merge the srclistfile with the oldmastercatalogue file
        ! The merged file is newmastercatalogue
        !************************************************************************************
        if(debug)   write(*, *) "L391 Ingesting the input source list" 
        write(*, *) "L391 Ingesting the input source list (calling MergeSourceLists)" 
        call MergeSourceLists( oldmasterCatalogue, srclistfile, newmasterCatalogue, &
             "SRCLIST",  nocolumns1, columndata1, compare1)
        if(debug)   write(*, *) "L360 Finished ingesting the input source list"
        write(*, *) "L396 Finished ingesting the input source list"

     end if



    !**********************************************************
    ! Do some basic checking
    !
    !*********************************************************
     if(debug)   write(*, *) "L406 implementing some basic checks on the catalogue"
     write(*, *) "L406 implementing some basic checks on the catalogue"
     !write(*,*) "L371 catalogue name = ",trim(newMasterCatalogue)
     call checkFlags( newMasterCatalogue)

     write(*,*) "L411 copying newMasterCatalogue to oldMasterCatalogue"
     call copydataSet( trim ( newMasterCatalogue), trim ( oldMasterCatalogue ) )
     
     if(debug)   write(*, *) "L414 Finishing the task ommastercatalogue" 
     write(*, *) "L414 Finishing the task ommastercatalogue" 
end subroutine  ommastercatalogue
!**********************************************************************************
! Creates a new master catalogue (the initial catalogue) from the 
! input source-list file
!
!
!*********************************************************************************
subroutine CreateCatalogue(mergedfile, newsrclistfile)
    character(*), intent(in) :: mergedfile, newsrclistfile

    type(datasetT) :: mergeddataSetH, newdataseth
   

    write(*, *) "L379 Creating catalogue from scratch, catalogue file=", trim(mergedFile)

    !******************************************************************************************
    ! Add the SRCLIST table
    !*****************************************************************************************
    if (setExists( trim(mergedFile))) then
       write(*,*) "L385 data set already exists: ", trim(mergedFile)
    endif

    mergedDataSetH = dataSet( trim(mergedFile), CREATE )
    newDataSetH = dataSet( trim(newsrclistFile), READ )

    call copyAllAttributes( attributable(newDataSetH), attributable(mergedDataSetH) )


    call release(newdataseth)

    call InsertTable( mergedDataSetH, "SRCLIST", nocolumns4, columndata4)

    write(*,*) "L437 inserting the summary table "
    !******************************************************************************************
    ! Add the SUMMARY table
    !******************************************************************************************
    call InsertTable( mergedDataSetH, "SUMMARY", nocolumns3, columndata3)


    call release( mergedDataSetH )

  
!    write(*, *) "Finished creating catalogue"

!    call fatal("stopped")

end subroutine CreateCatalogue
!********************************************************************
!
!
!
!********************************************************************
subroutine AddRowsToTable(name, nNewRows)

    character(*), intent(in) :: name
    integer(int32), intent(in) :: nNewRows
    integer(int32) :: nRows

    type(DataSetT) :: dataSetH
    type(TableT)   :: masterTableH

    type(ColumnT) :: columnH
    real(double), pointer :: r64Data(:) => null()
    integer :: i

    dataSetH = dataSet(trim(name), MODIFY)
    masterTableH = table(dataSetH, "SRCLIST", MODIFY)

    nRows = numberOfRows(masterTableH)

    call insertRows(masterTableH, nRows, nNewRows)


    columnH = Column(masterTableH, "RA", MODIFY)
    r64data => real64Data(columnh)


    do i = nRows, nRows + nNewRows
       r64Data(i - 1) = dble(i)     
    end do


    call release(columnH)

    call release(masterTableH)    
    call release(dataSetH)


end subroutine AddRowsToTable


!**********************************************************************************
! Merges the SRCLIST table in the omsrclistcomb SWS file with the
! SRCLIST table in the master catalogue file
!
!
!
!**********************************************************************************
subroutine MergeSourceLists(masterSourceList, newSourceList, mergedSourceList, srcTable, &
     nocolumns, columnData, compare)
  character(*), intent(in) :: masterSourceList, newSourceList, mergedSourceList, srcTable
  integer(int16), intent(in) :: nocolumns
  
  type(newcolumn), intent(in) :: columnData(:)
  type(dataSetT) :: masterdataSetH, newdataSetH, MergedDataSetH
  
  type(tableT) :: masterTableH, newtableH, mergedtableH
  integer(int32) :: i, j, k, iMaster, iNew, iMerged, nRowsNew, nRowsMaster, nRowsMerged
  real(double) :: mjdStart, mjdEnd
  character(100) :: columnName, comment   
  
  interface
     
     integer function Compare(newtableData, oldmastertableData, inew, imaster)  
       use ommasterdefs_mod
       type(tableData), intent(in) :: newtableData, oldMasterTableData
       integer, intent(in) :: inew, imaster
     end function Compare
     
  end interface
  
  
  !*****************************************************************************
  !
  !****************************************************************************
  
  type(tableData) :: oldmastertableData
  type(tableData) :: mergedmastertableData
  type(tableData) :: newtableData

  write(*,*) "L531 merging source lists "

  !newDataSetH = dataSet( trim(newsourceList),  READ )  ! The source-list to be added to the catalogue
  newDataSetH = dataSet( trim(newsourceList),  MODIFY )  ! The source-list to be added to the catalogue !! RDS - should be MODIFY as table is written to later

  if (.not. hasBlock(newDataSetH, trim(srcTable)) ) then
     call release(newDataSeth)

     !write(*, *) "L538 table name =", trim(srctable), " newsourcelist=", trim(newsourcelist)  
     call fatal("Table does not exist!")
     return
  else
     !write(*,*) "L542 Checked the table  ",trim(srctable) 
  end if

  !write(errstr,'(a, a, a, a)') "L519 Merging source list ", trim(newSourceList), &
  !     " with old master source list ", trim(masterSourceList)
  !call message(AppMsg, Verbose)

  write(*,*) "L549 Merging source list ", trim(newSourceList), &
       " with old master source list ", trim(masterSourceList)

  MasterDataSetH = dataSet(trim(masterSourceList), READ)     ! The old master catalogue to be updated 
  MergeddataSetH = dataSet(trim(mergedSourceList), MODIFY)   ! The new master catalogue

  !write(*,*) "L555"
  masterTableH = table(masterDataSetH, trim(srcTable), READ) 
  mergedTableH = table(mergedDataSetH, trim(srcTable), MODIFY)
  write(*,*) "L573 adding flag comments"
  call addFlagComments( mergedTableH )
  write(*,*) "L575 loading table columns from the master table nocolumns=", nocolumns
  call loadTableColumnData(masterDatasetH, masterTableH, oldMasterTableData, &
       nocolumns, columndata)
  write(*,*) "L578 nocolumns=", nocolumns

  !if(debug) write(*, *) "L535 after having loaded table columns for mastercatalogue - nocolumns=", noColumns

  newTableH = table(newdataSetH, trim(srcTable), MODIFY)

  !*************************************************************************************
  ! Add RA_HMS and DEC_DMS columns
  !*************************************************************************************

  if (debug) write(*, *) "L580 adding RA and DEC columns"
  
  call addraanddecstringcolumns1( newTableH ) 
  call release(newTableH)

  !!!!!!!!!!!!!!!!!!!!!
  ! here newDataSetH contains the number of filter-sources in the first line of column-17
  newTableH = table(newdataSetH, trim(srcTable), MODIFY)

  !write(*,*) "L584............. "
  !if(debug) write(*, *) "L585 loading from the source list to be ingested ... "

  write(*,*) "L600 calling loadTableColumnData nocolumns=", nocolumns 
  !call loadTableColumnData(newdataseth, newTableH, newTableData, nocolumns, columndata)
  call loadInputTableColumnData(newdataseth, newTableH, newTableData, nocolumns, columndata)
  write(*,*) "L603 nocolumns=", nocolumns 
  !if (debug) write(*, *) "L558"

  !****************************************************************************
  ! N_SUMMARY
  ! The following will record which observation number in the SUMMARY_FILE the
  ! update sources correspond to
  !****************************************************************************

  !if(debug) write(*, *) "L595"
  
  newtableData%attributeExists(1) = .true.
  newtableData%datatype(1) = 5
  newtableData%columnData(1)%int32value = obs_summary_no


  !if(debug) write(*, *) "L574"


  !****************************************************************
  ! SRC_NUM - the number of the unique source
  !***************************************************************

  newtableData%attributeExists(3) = .true.
  newtableData%datatype(3) = 5
  newtableData%columnData(3)%int32value = 666


  !if(debug) write(*, *) "L614 creating attributes for the column N_OBSID"

  !***********************************************************************************
  ! N_OBSID - the number of detections of other OBSIDs where this source is detected
  !***********************************************************************************

  newtableData%attributeExists(15) = .true.
  newtableData%datatype(15) = 5
  newtableData%columnData(15)%int32value = 0

  nRowsMaster = numberOfRows(masterTableH)
  nRowsNew = numberOfRows(newtableH)


  !if(debug) write(*, *) "L600"

  !*********************************************************************
  ! SRCDIST - the distance between a source and its nearest neighbour
  ! Need to compute these values and stick them in the SRCLIST table
  !*********************************************************************

  !if(debug) then 
     write(*, *) "************** Line 653 - setting up column SRCDIST **************"
     write(*, *) "nrowsmaster=", nrowsmaster, " nrowsnew=", nrowsnew
  !end if

  do i = 1, noFilters

     columnName = trim(omFilter(i)) // "_SRCDIST"
     
     !if (debug) write(*, *) i, ") line 684 ", trim(ColumnName)

     comment = "Dist. to nearest neighbour (" // trim(omFilter(i)) // ") in obs." 
     newtableData%datatype(3 + i) = 1

     newtableData%columnData(3 + i)%name = trim(columnName)

     if( newtableData%columnExists( 21 + i) ) then  ! Significance column exists

        newtableData%columnExists( 3 + i) = .true.
        
        !if(debug) &
        write(*, *) "L673 i=", i, ") Computing source distances using significance column ",  &
             trim(name(newTableData%columnData(21 + i)%columnh)) 

        !******************************************************************************
        !
        ! For each source compute the distance to its nearest neighbour
        !
        !******************************************************************************

        call ComputeNearestNeighbourDistances( trim(omfilter(i)), nRowsNew,             &
             newTableData%columnData(10)%real64ArrayData,     &
             newTableData%columnData(11)%real64ArrayData,     &
             newTableData%columnData(3 + i)%real32ArrayData,  &
             newTableData%columnData(21 + i)%real32ArrayData)

     else  ! No significance column

        write(*, *) "L690 i=", i, ") No significance column found "
        newtableData%columnExists( 3 + i) = .true.
        
        !            nullify( newTableData%columnData(3 + i)%real32ArrayData )

        allocate( newTableData%columnData(3 + i)%real32ArrayData(0 : nRowsNew - 1))
        newTableData%columnData(3 + i)%real32ArrayData = nullValue

     end if

  end do


  !if(debug) 
  write(*, *) "L704 nRowsMaster=", nRowsMaster, " nRowsNew=", nRowsNew

  nRowsMerged = nrowsMaster + nRowsNew ! The total number of rows in the updated master catalogue

  write(*,*) "L708 nRowsMerged=", nRowsMerged

  !if(debug) then
  !write(*, *) "Line 723 Inserting rows nrowsmerged=", nrowsmerged, " nrowsnew=", nrowsnew
  !end if
  !write(*,*) "L699 inserting rows into the merged table" 
  call insertrows( mergedTableH, nRowsMaster,  nRowsNew )


  !if(debug)  write(*, *) "L703 after inserting rows nRowsMaster=", &
  !     nRowsMaster, " nrowsnew=", nrowsnew, "..."
  write(*,*) "L718 calling loadTableColumnData nocolumns=", nocolumns 
  call loadTableColumnData( mergeddataseth, mergedTableH, mergedMasterTableData, &
       nocolumns, columndata )
  write(*,*) "L721 nocolumns=", nocolumns 

  !if(debug) then
  !  write(*, *) "Mergesourcelists line 627 the update table columns nocolumns=", noColumns
  !do i = 1, noColumns
  !    write(*, *) i, ") ",  newtableData%columnData(i)%name
  !end do

  !write(*, *) "********** The merged master table columns ***********"
  !write(*, *) "Mergesourcelists line 629 merged table data"

  !do i = 1, noColumns
  !    write(*, *) i, ") ",  mergedmastertableData%columnData(i)%name
  !end do
  !end if


  iMaster = 0  ! Start with row zero
  iNew = 0
  iMerged = 0


  !**********************************************************************************
  ! The actual merging code
  !**********************************************************************************

  nRowsNew = numberOfRows(newtableH)
  !if(debug) write(*, *) "L730 nRowsNew=", nRowsNew, " nRowsMaster=", nRowsMaster, &
  !     " imaster=", imaster

  do ! Keep looping until run out of rows

     if ( imaster .eq. nRowsMaster ) then
        !write(*,*) "L735 inew=", inew, " nRowsNew=", nRowsNew
        !********************************************************************************
        ! Copy any remaining records from the update file to the new master catalogue 
        !*******************************************************************************
        j = inew

        !if(debug) write(*, *) "L741 starting updating table columns"
        do i = j, nRowsNew - 1
           !write(*,*) "L743 j=", j
           call updateTableColumns(imerged, inew, mergedmastertableData,  newTableData)             
        end do
        exit
     end if

     !write(*,*) "L748 finished updating columns inew=", inew, " nRowsNew=", nRowsNew

     if (inew .eq. nRowsNew ) then

        !if(debug) write(*, *) "L754 imaster=", imaster

        ! Copy any remaining records from the old master catalogue to the new one 
        j = imaster

        do i = j, nRowsmaster - 1
           call updateTableColumns(imerged, imaster, mergedmastertableData, &
                oldmasterTableData)
        end do
        exit
     end if

     !if(debug) write(*, *) "L766"

     select case ( Compare( newtableData, oldmastertableData, inew, imaster ) )
     case (1)  ! New RA > old master RA
        !write(*,*) "L770 case 1 New RA > old master RA"
        call updateTableColumns(imerged, imaster,  mergedmastertableData, &
             oldmasterTableData )

     case (-1) ! New Ra < old master RA 
        !write(*,*) "L775 case -1 New Ra < old master RA"
        call updateTableColumns(imerged, inew, mergedmastertableData,  newTableData )
     case (0) ! Ras are equal - copy both to the new master catalogue
        !write(*,*) "L778 case 0 Ras are equal"
        call updateTableColumns(imerged, imaster,  mergedmastertableData, &
             oldmasterTableData )

        call updateTableColumns(imerged, inew,  mergedmastertableData, newTableData )

     case default
        !write(*,*) "L785 case default "
        !call fatal("bug at line 769")
     end select

  end do  ! End of updating the SRCLIST table


  !write(*,*) "L792 nrowsmerged=", nrowsmerged, " imerged=", imerged
  if (nrowsmerged .ne. imerged) then
     write(*, *) "L794 imerged=", imerged, " nrowsmerfged=", nrowsmerged
     call fatal("bug")
  end if

  !if(debug) write(*, *) "L798 After merging tables nrowsmerged=", nrowsmerged

  do i = 1, nofilters 
     if (associated(newTableData%columnData(i + 1)%real32ArrayData) ) then
        deallocate( newTableData%columnData(i + 1)%real32ArrayData)
        nullify( newTableData%columnData(i + 1)%real32ArrayData)
     end if
  end do

  call release(masterdataSetH)
  call release(newdataSetH)  
  call release(mergeddataSetH)


  !write(*,*) "L812 checking the merged table "
  ! Check whether the RA are sorted
  call checkMergedTable( mergedSourceList )

  !if(debug) then

  !write(*, *) "L818 imerged=", imerged, " nrowsmerged=", nrowsmerged
  !end if

  !call fatal("stopped at line 774")


end subroutine MergeSourceLists

!***********************************************************************
!
!
!
!***********************************************************************
subroutine LoadTableColumnData(dataseth, tableH, tableDataH, nocolumns, columndata)
    type(datasetT), intent(in) :: dataseth
    type(tableT), intent(inout) :: tableH
    type(TableData), intent(out) :: tableDataH
    integer(int16), intent(in) :: nocolumns
    type(newcolumn), intent(in) :: columndata(:)
    type(columnT) :: columnH
    integer :: i, j, k, nocolumns1
    character(30) :: nameI

    write(*,*) "L851 loading table column data LoadTableColumnData"
    tableDataH%tableH = tableh
    tableDataH%noRows = numberOfRows(tableH)    
    write(*,*) "L854 tableDataH-noRows=", tableDataH%noRows
    tableDataH%noColumns = 0    

    noColumns1 =  numberOfColumns(tableH) - 1
    write(*,*) "L858 noColumns1 (nColumns-1)=", noColumns1, " nocolumns=", nocolumns

!debug = .false.
!if(debug) then
!write(*, *) "Loadtablecolumndata line 909 nocolumns=", nocolumns, " nocolumns1=", nocolumns1
!end if
   
    iloop : do i = 1, nocolumns
        nameI = trim(columnData(i)%name)
          
       !if(debug .and. (i .eq. 17 .or. i .eq. 23)) then
       ! write(*, *) "L869 i=", i, "name=", trim(namei)
       !end if

     
!       if(debug) then 
!           write(*, *) i, ") loadtablecolumndata - column to check=", namei  
!       end if

       !************************************************************************
       ! Loop through all the columns in the update table
       !************************************************************************
       do j = 0, noColumns1 ! See if the column i exists in the update column 

          columnH = column(tableH, j, READ)

!          if(debug) then
!              write(*, *) j, ") line 941 name=", trim(name(columnh))
!          end if

          
          !**********************************************************************
          ! Column exists in the table
          !********************************************************************* 
          if ( trim(name(columnh)) .eq. nameI)  then
             !if(debug .and. i .eq. 17) then
             !   write(*, *) "L887 i=", i, " j=", j, "column nameI=", trim(namei), &
             !        " name(columnh)=", trim(name(columnh))  
             !endif

             tableDataH%columnExists(i) = .true. ! Record that the column exists
             tableDataH%noColumns = tableDataH%noColumns + 1
             tabledataH%columndata(i)%columnH = ColumnH
             tabledataH%columndata(i)%name = namei 

!if(debug) then
!    write(*, *) "j=", j, " Column ", trim(nameI), " exists", " data type=", columndatatype( columnh ) 
!end if             
             

             !*****************************************************************
             ! Set various parameters depending on column data type
             !*****************************************************************  

             select case ( columndatatype( tableDataH%columnData(i)%columnh) )

             case (real32) ! (1)
                tableDataH%columnData(i)%real32ArrayData => real32Data( columnH )
                tableDataH%datatype(i) = 1
     
!               if(debug) then
!                  write(*, *) "Line 811 column ", i, " exists-"
!                end if 

             case (real64)  ! (2)


                tableDataH%columnData(i)%real64ArrayData => real64Data(columnH)
                tableDataH%datatype(i) = 2

!                if(debug) then
!                  write(*, *) "Line 819 column ", i, " exists-"
!                end if 


             case (integer8)  ! (3)

!                if(debug) then
!                     write(*, *) "Line 915"
!                 end if


                tableDataH%columnData(i)%integer8ArrayData => int8Data(columnH)
                tableDataH%datatype(i) = 3

                call setNullValue( tabledataH%columndata(i)%columnH, int8NullValue )
        
        
!                if(debug) then
!                  write(*, *) "Line 834 column ", i, "name=",  name(tableDataH%columnData(i)%columnH), " exists- data type =int8"
!                end if 



             case (integer16) ! (4)

                 !if(debug .and. i .eq. 17) then
                 !    write(*, *) "Line 948"
                 !end if


                tableDataH%columnData(i)%integer16ArrayData => int16Data(columnH)
                tableDataH%datatype(i) = 4
       
                !if(debug .and. i .eq. 17) then
                !   write(*,*) "L959 -------------- tableDataH-columnData i=", i, ":"
                !   write(*,*) tableDataH%columnData(i)%integer16ArrayData
                !   write(*,*) "L961 --------------"
                !   write(*,*) "L963 setting the NULL value"
                !endif
                
                call setNullValue( tabledataH%columndata(i)%columnH, int16NullValue)
                
!               if(debug) then
!                  write(*, *) "Line 845 column ", i, " exists- dassigned data type=4 (int16), dal data type=", &
!                        datatype( columnh ), " name=", trim( name( columnh) )                 
!                end if 

                !if(debug .and. i .eq. 17) then
                !   write(*,*) "L973 -------------- tableDataH-columnData i=", i, ":"
                !   write(*,*) tableDataH%columnData(i)%integer16ArrayData
                !   write(*,*) "L975 --------------"
                !   write(*,*) "L976 setting to NULL "
                !endif

             case (integer32)  ! (5)


                tableDataH%columnData(i)%integer32ArrayData => int32Data(columnH)
                tableDataH%datatype(i) = 5

                call setNullValue( tabledataH%columndata(i)%columnH, int32NullValue )
        

!               if(debug) then
!                 write(*, *) "Line 856 column ", i, " exists- data type =int32"
!                end if 


             case ( string )   ! (6)

                tableDataH%datatype(i) = 6



!               if(debug) then
!                  write(*, *) "Line 867 column ", i, " exists-"
!                end if 


             end select

                      

             cycle iLoop ! The column exists so look for the next column


          end if
          call release(columnh)        

!if(debug) then
!    write(*, *) "Column ", nameI, " does not exist"
!end if             
   


       end do

       !***************************************************************
       !
       ! See if name exists as an attribute in main header
       !
       !***************************************************************
!       if(debug) then
!           write(*, *) "Line 983 checking for attribute ", trim(namei) 
!       end if

       if ( hasAttribute( dataSetH, trim(nameI) )) then  ! The name is a header attribute

 !         if(debug) then
 !             write(*, *) "Line 897 attribute", nameI, " exists in main  header, i=", i, " j=", j
 !         end if 
         tableDataH%attributeExists(i) = .true.


         select case ( columndata(i)%type )
         case (real32)           

            tableDataH%datatype(i) = 2
             

            tableDataH%columnData(i)%real32value = real32Attribute( dataSetH, trim(columndata(i)%name) )

         case (real64)           
            tableDataH%datatype(i) = 2

            tableDataH%columnData(i)%real64Value  = real64Attribute( dataSetH, trim(columndata(i)%name) )

   

         case (integer8)
            tableDataH%datatype(i) = 3

!if(debug) then
!    write(*, *) "Line 998 integer8 column"
!end if

            tableDataH%columnData(i)%int8value  = int32Attribute( dataSetH, trim(columndata(i)%name) )

         case (integer16)   
             tableDataH%datatype(i) = 4

             tableDataH%columnData(i)%int16Value  = int32Attribute( dataSetH, trim(columndata(i)%name) )

!if(debug) then
!    write(*, *) "Line 1009 integer16 column"
!end if




          case (integer32)
             tableDataH%datatype(i) = 5
             
             tableDataH%columnData(i)%int32value  = int32Attribute( dataSetH, trim(columndata(i)%name) )


          case (string)   


!if(debug) then
!    write(*, *) "Line 1039 string attribute"
!end if

             tableDataH%datatype(i) = 6

             tableDataH%columnData(i)%stringvalue  = stringAttribute( dataSetH, trim(columndata(i)%name) )

          case default
             call fatal("BUG AT LINE 1085")  ! Should never get here
          end select

          cycle iloop   ! Found it- go to the next column
        else

!            if(debug) then
!                write(*, *) "Line 957 attribute", nameI, " does not exist in main  header"
!            end if 
        end if



        if ( hasAttribute( tableH, nameI )) then ! The name is a table attribute

 !         if(debug) then
 !         write(*, *) "L:ine 966 attribute ", columndata(i)%name, " exists in table"
 !         end if

           tableDataH%attributeExists(i) = .true.


           select case ( columndata(i)%type )
           case (real32)           
              
              tableDataH%datatype(i) = 2
              tableDataH%columnData(i)%real32value = real32Attribute( tableH, trim(columndata(i)%name) )
              

           case (real64)           
              tableDataH%datatype(i) = 2
              tableDataH%columnData(i)%real64Value = real64Attribute( tableH, trim(columndata(i)%name) )
              
              
           case (integer8)
              tableDataH%datatype(i) = 3
              tableDataH%columnData(i)%int8value   = int32Attribute( tableH, trim(columndata(i)%name) )
              
           case (integer16)   
              tableDataH%datatype(i) = 4
              tableDataH%columnData(i)%int16Value  = int32Attribute( tableH, trim(columndata(i)%name) )
              
           case (integer32)
              tableDataH%datatype(i) = 5
              tableDataH%columnData(i)%int32value  = int32Attribute( tableH, trim(columndata(i)%name) )
              
           case (string)   
              
              
              tableDataH%datatype(i) = 6
              tableDataH%columnData(i)%stringvalue  = stringAttribute( tableH, trim(columndata(i)%name) )
              
              !write(*, *) "Line 643 value=",   tableDataH%columnData(i)%stringvalue
              
           case default
              call fatal("bug at line 1139")  ! Should never reach here   
              
           end select
           
           cycle iloop

        else


           !       if(debug) then
           !       write(*, *) "attribute ", columndata(i)%name, " does not exist in table"
           !       end if 
           
          call release(columnh)        
          
          
       end if
       
    end do iLoop
    


!if(debug) then
!  write(*, *) "Line 1222 leaving loadtablecolumndata no of columns=",  tableDataH%noColumns

!write(*, *) "Loadtablecolumndata line1101 - The table columns-"
!do i = 1, numberofColumns(tableDataH%tableH)
!    write(*, *) i, ") ", trim(name(column(tableDataH%tableH, i - 1))), " type", datatype(column(tableDataH%tableH, i - 1) ), &
!   " assigned type=", tableDataH%dataType(i)
!end do
!call fatal("stopped")
!end if

    !debug= .true.
  end subroutine loadTableColumnData



!***********************************************************************
!
! This routine should be different from loadTableColumnData
! becasue the number of input sources in the column N_filter_EXP
! should be set to 1 if the column filter_SIGNIFICANCE is not NULL  
!
!***********************************************************************
subroutine LoadInputTableColumnData(dataseth, tableH, tableDataH, nocolumns, columndata)
    type(datasetT), intent(in) :: dataseth
    type(tableT), intent(inout) :: tableH
    type(TableData), intent(out) :: tableDataH
    integer(int16), intent(in) :: nocolumns
    type(newcolumn), intent(in) :: columndata(:)
    type(columnT) :: columnH
    integer :: i, j, k, nocolumns1
    character(30) :: nameI
   
    integer :: iSignif, kSignif, nSignif
    type(columnT) :: columnSignifH
    character(30) :: nameIsignif
    logical :: columnSignifFound
    integer :: jj

    !write(*,*) "L1200 loading input table column data LoadTableColumnData"
    tableDataH%tableH = tableh
    tableDataH%noRows = numberOfRows(tableH)    
    !write(*,*) "L1203 tableDataH-noRows=", tableDataH%noRows
    tableDataH%noColumns = 0    

    noColumns1 =  numberOfColumns(tableH) - 1
    !write(*,*) "L1207 noColumns1=", noColumns1

!debug = .false.
!if(debug) then
!write(*, *) "Loadtablecolumndata line 909 nocolumns=", nocolumns, " nocolumns1=", nocolumns1
!end if
   
    iloop : do i = 1, nocolumns
       nameI = trim(columnData(i)%name)
          
       !if(debug .and. (i .eq. 17 .or. i .eq. 23)) then
       !   write(*, *) "L1220 i=", i, "name=", trim(namei)
       !end if



       !       if(debug) then 
!           write(*, *) i, ") loadtablecolumndata - column to check=", namei  
!       end if

       !************************************************************************
       ! Loop through all the columns in the update table
       !************************************************************************
       do j = 0, noColumns1 ! See if the column i exists in the update column 

          columnH = column(tableH, j, READ)

          !          if(debug) then
          !              write(*, *) j, ") line 941 name=", trim(name(columnh))
          !          end if

          !if (debug .and. i .eq. 17) then
          !   write(*,*) "L1244 j=", j, " name=", trim(name(columnh))  ! RDS REM
          !endif

          !**********************************************************************
          ! Column exists in the table
          !********************************************************************* 
          if ( trim(name(columnh)) .eq. nameI)  then
             !if(debug .and. (i .eq. 17 .or. i .eq. 23)) then
             !   write(*, *) "L1250 i=", i, " j=", j, "column nameI=", trim(namei), &
             !        " name(columnh)=", trim(name(columnh))  
             !endif

             tableDataH%columnExists(i) = .true. ! Record that the column exists
             tableDataH%noColumns = tableDataH%noColumns + 1
             tabledataH%columndata(i)%columnH = ColumnH
             tabledataH%columndata(i)%name = namei 

             !**********************************************************************
             ! in the case of the column number ranging from 16 to 21 (N_filter_EXP)
             ! we have to check the column filter_SIGNIFICANCE (number+6)
             !**********************************************************************
             !if (i .ge. 16 .and. i .le.21) then
             !   
             !   iSignif=i+6
             !   if(debug .and. i .eq. 17) then
             !      write(*,*) "L1265 i=", i, " iSignif=", iSignif, " noRows=", tableDataH%noRows
             !   endif
             !   columnSignifH = column(tableH, iSignif, READ)
             !   nSignif = 0  ! Count how many non-values there are
             !   if(debug .and. i .eq. 17) then
             !      write(*,*) "L1269 iSignif columndatatype=", columndatatype( columnSignifH) 
             !      write(*,*) "L1270 i       columndatatype=", columndatatype( tableDataH%columnData(i)%columnh) 
             !      !write(*,*) "L1271 columnH type=", columndatatype( columnH)
             !   endif
             !   tableDataH%columnData(iSignif)%real32ArrayData => real32Data( columnSignifH )
             !   do kSignif = 0, tableDataH%noRows  - 1
             !      if(debug .and. i .eq. 17 .and. kSignif .lt. 3) then
             !         write(*,*) "L1275 kSignif=", kSignif, " value=", &
             !              tableDataH%columnData(iSignif)%real32ArrayData(kSignif)
             !      endif
             !      if ( .not. isNullValue(  tableDataH%columnData(iSignif)%real32ArrayData(kSignif) )) then 
             !         nSignif = nSignif + 1         
             !      endif
             !   end do
             !   write(*,*) "L1282  nSignif=", nSignif
             !   call release(columnSignifH)    
             !endif

!if(debug) then
!    write(*, *) "j=", j, " Column ", trim(nameI), " exists", " data type=", columndatatype( columnh ) 
!end if             
             

             !*****************************************************************
             ! Set various parameters depending on column data type
             !*****************************************************************  

             !if(debug .and. (i .eq. 17 .or. i .eq. 23) ) then
             !   write(*,*) "L1266 i=", i, " columndatatype=", &
             !        columndatatype( tableDataH%columnData(i)%columnh)   
             !endif

             select case ( columndatatype( tableDataH%columnData(i)%columnh) )
   
             case (real32) ! (1)
                tableDataH%columnData(i)%real32ArrayData => real32Data( columnH )
                tableDataH%datatype(i) = 1
     
               !if(debug .and. (i .eq. 17 .or. i .eq. 23)) then
               !   write(*, *) "L1307 column ", i, " exists-"
               !   write(*,*) "L1308 -------------- tableDataH-columnData i=", i, ":"
               !   write(*,*) tableDataH%columnData(i)%real32ArrayData
               !   write(*,*) "L1310 --------------"
               !   write(*,*) "L1311 setting the NULL value"
               !end if 

             case (real64)  ! (2)


                tableDataH%columnData(i)%real64ArrayData => real64Data(columnH)
                tableDataH%datatype(i) = 2

!                if(debug) then
!                  write(*, *) "Line 819 column ", i, " exists-"
!                end if 


             case (integer8)  ! (3)

!                if(debug) then
!                     write(*, *) "Line 915"
!                 end if


                tableDataH%columnData(i)%integer8ArrayData => int8Data(columnH)
                tableDataH%datatype(i) = 3

                call setNullValue( tabledataH%columndata(i)%columnH, int8NullValue )
        
        
!                if(debug) then
!                  write(*, *) "Line 834 column ", i, "name=",  name(tableDataH%columnData(i)%columnH), " exists- data type =int8"
!                end if 



             case (integer16) ! (4)

                 !if(debug .and. i .eq. 17) then
                 !    write(*, *) "Line 1306"
                 !end if


                tableDataH%columnData(i)%integer16ArrayData => int16Data(columnH)
                tableDataH%datatype(i) = 4
       
                !if(debug .and. i .eq. 17) then
                !   write(*,*) "L1314 -------------- tableDataH-columnData i=", i, ":"
                !   write(*,*) tableDataH%columnData(i)%integer16ArrayData
                !   write(*,*) "L1316 --------------"
                !   write(*,*) "L1317 setting the NULL value"
                !endif
                
                !**********************************************************************
                ! in the case of the column number ranging from 16 to 21 (N_filter_EXP)
                ! we have to check the column filter_SIGNIFICANCE (number+6)
                !**********************************************************************

                if (i .ge. 16 .and. i .le.21) then
                
                   iSignif=i+6
                   nameIsignif=trim(columnData(iSignif)%name)
                   !if(debug .and. i .eq. 17) then
                   !   write(*,*) "L1373 i=", i, " iSignif=", iSignif, " noRows=", tableDataH%noRows, &
                   !        " nameIsignif=", nameIsignif
!
                   !endif
                   ! Search for the column filter_SIGNIFICANCE for this filter
                   columnSignifFound=.false.
                   do jj = 0, noColumns1
                      columnSignifH = column(tableH, jj, READ)
                      !if (debug .and. i .eq. 17) then
                      !   write(*,*) "L1389 jj=", jj, " name=", trim(name(columnSignifH))
                      !endif
                      if ( trim(name(columnSignifH)) .eq. nameIsignif)  then
                         columnSignifFound=.true.
                         exit
                      endif
                      call release(columnSignifH)                        
                   enddo
                   !if(debug .and. i .eq. 17) then
                   !   write(*,*) "L1398 columnSignifFound=", columnSignifFound
                   !endif
                   nSignif = 0  ! Count how many non-values there are
                   if (columnSignifFound) then
                      !if(debug .and. i .eq. 17) then
                      !   write(*,*) "L1405 column SIGNIF found: jj=", jj
                      !endif
                      ! if the SIGNIFICANCE column found then set the source nomber to 1
                      ! else set it to 0
                      tableDataH%columnData(iSignif)%real32ArrayData => real32Data( columnSignifH )
                      ! loop through all of the sources and set either 1 or 0
                      do kSignif = 0, tableDataH%noRows  - 1
                         !if(debug .and. i .eq. 17 .and. kSignif .lt. 20) then
                         !   write(*,*) "L1406 kSignif=", kSignif, " value=", &
                         !        tableDataH%columnData(iSignif)%real32ArrayData(kSignif)
                         !endif
                         if ( .not. isNullValue(  tableDataH%columnData(iSignif)%real32ArrayData(kSignif) )) then 
                            tableDataH%columnData(i)%integer16ArrayData(kSignif)=1
                            !if(debug .and. i .eq. 17 .and. kSignif .lt. 20) then
                            !   write(*,*) "             setting to 1"
                            !endif
                            nSignif = nSignif + 1         
                         else
                            !if(debug .and. i .eq. 17 .and. kSignif .lt. 20) then
                            !   write(*,*) "             setting to 0"
                            !endif
                            tableDataH%columnData(i)%integer16ArrayData(kSignif)=0
                         endif                        
                      end do

                      call release(columnSignifH)       
                   else
                      ! set all the source number for all sources to 0
                      !if(debug .and. i .eq. 17) then
                      !   write(*,*) "L1427 column SIGNIF not found found: jj=", jj
                      !endif                    
                      do kSignif = 0, tableDataH%noRows  - 1
                         tableDataH%columnData(i)%integer16ArrayData(kSignif)=0
                      enddo
                   endif

                   
                   !if(debug .and. i .eq. 17) then
                   !   write(*,*) "L1379 iSignif columndatatype=", columndatatype( columnSignifH) 
                   !   write(*,*) "L1380 i       columndatatype=", columndatatype( tableDataH%columnData(i)%columnh) 
                   !endif
                  
                   
                   !if(debug .and. i .eq. 17) then
                   !   write(*,*) "L1442  nSignif=", nSignif
                   !endif
                   
                endif

                call setNullValue( tabledataH%columndata(i)%columnH, int16NullValue)
                
!               if(debug) then
!                  write(*, *) "Line 845 column ", i, " exists- dassigned data type=4 (int16), dal data type=", &
!                        datatype( columnh ), " name=", trim( name( columnh) )                 
!                end if 

                !if(debug .and. i .eq. 17) then
                !   write(*,*) "L1410 -------------- tableDataH-columnData i=", i, ":"
                !   write(*,*) tableDataH%columnData(i)%integer16ArrayData
                !   write(*,*) "L1412 --------------"
                !   write(*,*) "L1413 setting to NULL "
                !endif

             case (integer32)  ! (5)


                tableDataH%columnData(i)%integer32ArrayData => int32Data(columnH)
                tableDataH%datatype(i) = 5

                call setNullValue( tabledataH%columndata(i)%columnH, int32NullValue )
        

!               if(debug) then
!                 write(*, *) "Line 856 column ", i, " exists- data type =int32"
!                end if 


             case ( string )   ! (6)

                tableDataH%datatype(i) = 6



!               if(debug) then
!                  write(*, *) "Line 867 column ", i, " exists-"
!                end if 


             end select

                      

             cycle iLoop ! The column exists so look for the next column


          end if
          call release(columnh)        

!if(debug) then
!    write(*, *) "Column ", nameI, " does not exist"
!end if             
   


       end do

       !***************************************************************
       !
       ! See if name exists as an attribute in main header
       !
       !***************************************************************
!       if(debug) then
         !  write(*, *) "Line 983 checking for attribute ", trim(namei) ! RDS REM
!       end if

       if ( hasAttribute( dataSetH, trim(nameI) )) then  ! The name is a header attribute

 !         if(debug) then
 !             write(*, *) "Line 897 attribute", nameI, " exists in main  header, i=", i, " j=", j
 !         end if 
         tableDataH%attributeExists(i) = .true.


          select case ( columndata(i)%type )
          case (real32)           

             tableDataH%datatype(i) = 2


             tableDataH%columnData(i)%real32value = real32Attribute( dataSetH, trim(columndata(i)%name) )

          case (real64)           
             tableDataH%datatype(i) = 2

             tableDataH%columnData(i)%real64Value  = real64Attribute( dataSetH, trim(columndata(i)%name) )

   

          case (integer8)
             tableDataH%datatype(i) = 3

!if(debug) then
!    write(*, *) "Line 998 integer8 column"
!end if

             tableDataH%columnData(i)%int8value  = int32Attribute( dataSetH, trim(columndata(i)%name) )

          case (integer16)   
             tableDataH%datatype(i) = 4

             tableDataH%columnData(i)%int16Value  = int32Attribute( dataSetH, trim(columndata(i)%name) )

!if(debug) then
!    write(*, *) "Line 1009 integer16 column"
!end if




          case (integer32)
             tableDataH%datatype(i) = 5

             tableDataH%columnData(i)%int32value  = int32Attribute( dataSetH, trim(columndata(i)%name) )


          case (string)   


!if(debug) then
!    write(*, *) "Line 1039 string attribute"
!end if

             tableDataH%datatype(i) = 6

             tableDataH%columnData(i)%stringvalue  = stringAttribute( dataSetH, trim(columndata(i)%name) )

          case default
             call fatal("BUG AT LINE 1448")  ! Should never get here
          end select

          cycle iloop   ! Found it- go to the next column
        else

!            if(debug) then
!                write(*, *) "Line 957 attribute", nameI, " does not exist in main  header"
!            end if 
        end if



        if ( hasAttribute( tableH, nameI )) then ! The name is a table attribute

 !         if(debug) then
 !         write(*, *) "L:ine 966 attribute ", columndata(i)%name, " exists in table"
 !         end if

            tableDataH%attributeExists(i) = .true.


            select case ( columndata(i)%type )
            case (real32)           

                tableDataH%datatype(i) = 2
                tableDataH%columnData(i)%real32value = real32Attribute( tableH, trim(columndata(i)%name) )


            case (real64)           
                tableDataH%datatype(i) = 2
                tableDataH%columnData(i)%real64Value = real64Attribute( tableH, trim(columndata(i)%name) )


            case (integer8)
                tableDataH%datatype(i) = 3
                tableDataH%columnData(i)%int8value   = int32Attribute( tableH, trim(columndata(i)%name) )

            case (integer16)   
                tableDataH%datatype(i) = 4
                tableDataH%columnData(i)%int16Value  = int32Attribute( tableH, trim(columndata(i)%name) )

            case (integer32)
                tableDataH%datatype(i) = 5
                tableDataH%columnData(i)%int32value  = int32Attribute( tableH, trim(columndata(i)%name) )

            case (string)   


                tableDataH%datatype(i) = 6
                tableDataH%columnData(i)%stringvalue  = stringAttribute( tableH, trim(columndata(i)%name) )

                !write(*, *) "Line 643 value=",   tableDataH%columnData(i)%stringvalue

            case default
                call fatal("bug at line 1503")  ! Should never reach here   

        end select

        cycle iloop

        else


!       if(debug) then
!       write(*, *) "attribute ", columndata(i)%name, " does not exist in table"
!       end if 

          call release(columnh)        


        end if

    end do iLoop



!if(debug) then
!  write(*, *) "Line 1222 leaving loadtablecolumndata no of columns=",  tableDataH%noColumns

!write(*, *) "Loadtablecolumndata line1101 - The table columns-"
!do i = 1, numberofColumns(tableDataH%tableH)
!    write(*, *) i, ") ", trim(name(column(tableDataH%tableH, i - 1))), " type", datatype(column(tableDataH%tableH, i - 1) ), &
!   " assigned type=", tableDataH%dataType(i)
!end do
!call fatal("stopped")
!end if

!debug= .true.
end subroutine loadInputTableColumnData

!*******************************************************************************
!
!
!
!******************************************************************************
subroutine SetUpColumnNames


    integer(int16) nocolumns
    integer(int8) :: i
    character(100) :: name, comment, newName

    !***************************************************************************************
    !
    ! Columns for INPUT SRCLIST
    !
    !***************************************************************************************

    noColumns1 = 0
 
 
    call createNewColumns( "N_SUMMARY", "", "Row number from HDU2 containing OBSID", integer32, noColumns1, columnData1 )
    call createNewColumns( "SRCID", "", "Number of the source in the OBSID source table", integer32, noColumns1, columnData1 )
    call createNewColumns( "SRCNUM", "", "Number of the source in the OM UV catalogue", integer32, noColumns1, columnData1 )

   
    !********************************************************************************************
    ! The following will store the distance between each source and its nearest neighbour
    !********************************************************************************************
!    call createNewColumns( "SRCDIST", "arcsec", "Nearest-neighbour distance in OBSID source table", real32, noColumns1, columnData1 )


    do i = 1, noFilters
        name = trim(omFilter(i)) // "_SRCDIST"
        comment = "Distance to nearest neighbour in obs. (" // trim(omFilter(i)) // ")" 
        call createNewColumns( trim(name), "arcsec", trim(comment), real32, noColumns1, columnData1 )
    end do


    call createNewColumns( "RA_CORR", "degrees", "Right Ascension of source (J2000)", real64, noColumns1, columnData1, "RA" )
    raCompare1Index = noColumns1  ! The index for RA comparison

    call createNewColumns( "DEC_CORR", "degrees", "Declination of source (J2000)", real64, noColumns1, columnData1, "DEC")
    call createNewColumns( "POSERR", "arcsec", "Positional error", real32, noColumns1, columnData1 )


    call createNewColumns( "LII", "degrees", "Galactic longitude of the source (J2000)", real64, noColumns1, columnData1 )
    call createNewColumns( "BII", "degrees", "Galactic latitude of the source (J2000)", real64, noColumns1, columnData1 )


    !****************************************************************************************************
    !
    !
    !****************************************************************************************************
    call createNewColumns( "N_OBSID", "", "Num. of other OBSIDs where source is detect.", &
                   integer32, noColumns1, columnData1 )
 


    !*******************************************************************************************************
    ! Add the number of exposures during the observation for this source
    !******************************************************************************************************


    !*********************************************************************************************
    ! No of exposures
    !********************************************************************************************

    do i = 1, noFilters
        name = "N_" // trim(omFilter(i)) // "_EXP"
        comment = "Number of " // trim(omFilter(i)) // " exposures during the observation" 
        call createNewColumns( trim(name), "", trim(comment), integer16, noColumns1, columnData1 )
    end do



    !*********************************************************************************************
    ! Significances
    !********************************************************************************************

    do i = 1, noFilters
        name = trim(omFilter(i)) // "_SIGNIFICANCE"
        newName = trim(omFilter(i)) // "_SIGNIF"

        comment = "Significance of detection (" // trim(omFilter(i)) // " filter)" 
        call createNewColumns( trim(name), "sigma", trim(comment), real32, noColumns1, columnData1, trim(newName) )
    end do


    !*********************************************************************************************
    ! Corrected count rates
    !********************************************************************************************

    do i = 1, noFilters
        name = trim(omFilter(i)) // "_CORR_RATE"
        newName = trim(omFilter(i)) // "_RATE"

        comment = "Corrected count rate ("// trim(omFilter(i)) // " filter)"
        call createNewColumns( trim(name), "count/sec", trim(comment), real32, noColumns1, columnData1, trim(newName) )
        name = trim(omFilter(i)) // "_CORR_RATE_ERR"
        newName = trim(omFilter(i)) // "_RATE_ERR"

        comment = "1-sigma count rate error (" // trim(omFilter(i)) // " filter)"
  
        call createNewColumns( trim(name), "count/sec", trim(comment), real32, noColumns1, columnData1, trim(newName) )
    end do


    !*********************************************************************************************
    ! AB fluxes
    !********************************************************************************************

    do i = 1, noFilters
        name = trim(omFilter(i)) // "_AB_FLUX"
        comment = "Source AB flux (" // trim(omfilter(i)) // " filter)" 
        call createNewColumns( trim(name), "erg/s/cm**2/A", trim(comment), real32, noColumns1, columnData1 )
        name = trim(omFilter(i)) // "_AB_FLUX_ERR"
        comment = "1 sigma  flux error (" // trim(omfilter(i)) // " filter)"    
        call createNewColumns( trim(name), "erg/s/cm**2/A", trim(comment), real32, noColumns1, columnData1 )
     
    end do

    !*********************************************************************************************
    ! AB magnitudes
    !********************************************************************************************
    do i = 1, noFilters
        name = trim(omFilter(i)) // "_AB_MAG"
        comment = "Source AB magnitude (" // trim(omfilter(i)) // " filter)"
        call createNewColumns( trim(name), "mag", trim(comment), real32, noColumns1, columnData1 )
        name = trim(omFilter(i)) // "_AB_MAG_ERR"

        comment = "1-sigma AB magnitude error (" // trim(omfilter(i)) // " filter)"
    
        call createNewColumns( trim(name), "mag", trim(comment), real32, noColumns1, columnData1 )
      
     
    end do


    !*********************************************************************************************
    ! Vega magnitudes (NB INSTRUMENTAL MAGNITUDES)
    !********************************************************************************************
    do i = 1, noFilters
        name = trim(omFilter(i)) // "_INS_MAG"
        newname = trim(omFilter(i)) // "_VEGA_MAG"
      
        comment = "Source Vega magnitude (" // trim(omfilter(i)) // " filter)"
        call createNewColumns( trim(name), "mag", trim(comment), real32, noColumns1, columnData1, trim(newName) )
        name = trim(omFilter(i)) // "_INS_MAG_ERR"
        newname = trim(omFilter(i)) // "_VEGA_MAG_ERR"
      
        comment = "1-sigma Vega magnitude error (" // trim(omfilter(i)) // " filter)"
    
        call createNewColumns( trim(name), "mag", trim(comment), real32, noColumns1, columnData1, trim(newName)  )
           
    end do

    !*********************************************************************************************
    ! The following are for variability statistics - will be null when catalogue is created
    !********************************************************************************************


!    do i = 1, noFilters
!        name = trim(omFilter(i)) // "_CHI"
!        comment = "Source variability chi**2 statistic for UVW2 filter " // trim(omfilter(i)) // " filter"  
!        call createNewColumns( trim(name),     "arcsec", trim(comment), real32, noColumns1, columnData1 )
!    end do



    !*********************************************************************************************
    ! Source semi-major axis
    !********************************************************************************************
    do i = 1, noFilters
        name = trim(omFilter(i)) // "_MAJOR_AXIS"
        comment = "Source major axis size (" // trim(omfilter(i)) // " filter)"  
        call createNewColumns( trim(name),     "arcsec", trim(comment), real32, noColumns1, columnData1 )
    end do



    !*********************************************************************************************
    ! Source semi-minor axis
    !********************************************************************************************


    do i = 1, noFilters
        name = trim(omFilter(i)) // "_MINOR_AXIS"
        comment = "Source minor axis size (" // trim(omfilter(i)) // " filter)"  
        call createNewColumns( trim(name),     "arcsec", trim(comment), real32, noColumns1, columnData1 )
    end do


    !*********************************************************************************************
    ! Source position angle
    !********************************************************************************************

    do i = 1, noFilters
        name = trim(omFilter(i)) // "_POSANG"
        comment = "Source position angle (" // trim(omfilter(i)) // " filter)"  
        call createNewColumns( trim(name),     "degree", trim(comment), real32, noColumns1, columnData1 )
    end do




    !*********************************************************************************************
    ! Quality flags
    !********************************************************************************************

    do i = 1, noFilters
        name = trim(omFilter(i)) // "_QUALITY_FLAG" 
        comment = "Source quality flag (" // trim(omfilter(i)) // " filter)"  
        call createNewColumns( trim(name),     "", trim(comment), integer16, noColumns1, columnData1 )
    end do



    !*********************************************************************************************
    ! Confusion flags
    !********************************************************************************************

 !   do i = 1, noFilters
 !       name = trim(omFilter(i)) // "_CONFUSION_FLAG" 
 !       comment = "Source confusion flag (" // trim(omfilter(i)) // " filter)"  
 !       call createNewColumns( trim(name),     "", trim(comment), integer8, noColumns1, columnData1 )
 !   end do



    !*********************************************************************************************
    ! Extension flags
    !********************************************************************************************

    do i = 1, noFilters
        name = trim(omFilter(i)) // "_EXTENDED_FLAG" 
        comment = "Source extension flag (" // trim(omfilter(i)) // " filter)"  
        call createNewColumns( trim(name),     "", trim(comment), integer8, noColumns1, columnData1 )
    end do




    !*********************************************************************************************
    ! Source flags
    !********************************************************************************************
!    call createNewColumns( "QFLAG_COMB", "", "", integer16, noColumns1, columnData1)



!    call createNewColumns( "EFLAG_MAX", "", "", integer8, noColumns1, columnData1)


    !***************************************************************************************
    !
    ! Columns for SRCLIST table in the catalogue
    !
    !
    !***************************************************************************************

    noColumns4 = 0
  

    call createNewColumns( "N_SUMMARY", "", "Row number from HDU2 containing OBSID", &
                               integer32, noColumns4, columnData4 )


    !***************************************************************************************
    ! The source no in the original observation file
    !***************************************************************************************
    call createNewColumns( "SRCID", "", "Number of the source in the OBSID source table",  &
                                 integer32, noColumns4, columnData4 )

    call createNewColumns( "SRCNUM", "", "Number of the source in the OM UV catalogue", &
                           integer32, noColumns4, columnData4 )
   
  
    !********************************************************************************************
    ! The following will store the distance between each source and its nearest neighbour
    !********************************************************************************************
!    call createNewColumns( "SRCDIST", "arcsec", "Nearest-neighbour distance in OBSID source table", &
!                     real32, noColumns4, columnData4 )



    do i = 1, noFilters
        name = trim(omFilter(i)) // "_SRCDIST"
        comment = "Distance to nearest neighbour " // trim(omFilter(i)) // " in observation" 
        call createNewColumns( trim(name), "arcsec", trim(comment), real32, noColumns4, columnData4 )
    end do




    call createNewColumns( "RA_CORR", "degrees", "",    real64,    noColumns4, columnData4, "RA")
    call createNewColumns( "DEC_CORR", "degrees", "",   real64,    noColumns4, columnData4, "DEC")
    call createNewColumns( "POSERR", "arcsec", "Positional error", real32, noColumns4, columnData4 )


    call createNewColumns( "LII", "degrees", "Galactic longitude (J2000)", real64, noColumns4, columnData4 )
    call createNewColumns( "BII", "degrees", "Galactic latitude (J2000)", real64, noColumns4, columnData4 )



    !****************************************************************************************************
    !
    !
    !****************************************************************************************************
    call createNewColumns( "N_OBSID", "", "Number of other OBSIDs where this source is detected", &
                  integer32, noColumns4, columnData4 )
 





    !*******************************************************************************************************
    ! Add the number of exposures during the observation for this source
    !******************************************************************************************************


    !*********************************************************************************************
    ! No of exposures
    !********************************************************************************************
    do i = 1, noFilters
        name = "N_" // trim(omFilter(i)) // "_EXP"
        comment = "Number of " // trim(omFilter(i)) // " exposures during the observation" 
        call createNewColumns( trim(name), "s", trim(comment), integer16, noColumns4, columnData4 )
    end do


    !*********************************************************************************************
    ! Significances
    !********************************************************************************************

    do i = 1, noFilters
        name = trim(omFilter(i)) // "_SIGNIFICANCE"
        call createNewColumns( trim(name), " ", "", real32, noColumns4, columnData4 )
    end do


    !*********************************************************************************************
    ! Corrected count rates
    !********************************************************************************************

    do i = 1, noFilters
        name = trim(omFilter(i)) // "_CORR_RATE"
        call createNewColumns( trim(name), "count/sec", "", real32, noColumns4, columnData4 )
        name = trim(omFilter(i)) // "_CORR_RATE_ERR"
        call createNewColumns( trim(name), "count/sec", "", real32, noColumns4, columnData4 )
    end do




    !*********************************************************************************************
    ! AB fluxes
    !********************************************************************************************

    do i = 1, noFilters
        name = trim(omFilter(i)) // "_AB_FLUX"
        call createNewColumns( trim(name), "erg/s/cm**2/A", "", real32, noColumns4, columnData4 )
        name = trim(omFilter(i)) // "_AB_FLUX_ERR"
        call createNewColumns( trim(name), "erg/s/cm**2/A", "", real32, noColumns4, columnData4 )
     
    end do



    !*********************************************************************************************
    ! AB magnitudes
    !********************************************************************************************
    do i = 1, noFilters
        name = trim(omFilter(i)) // "_AB_MAG"
        call createNewColumns( trim(name), "mag", "", real32, noColumns4, columnData4 )
        name = trim(omFilter(i)) // "_AB_MAG_ERR"
        call createNewColumns( trim(name), "mag", "", real32, noColumns4, columnData4 )
      
     
    end do







    !*********************************************************************************************
    ! Vega magnitudes
    !********************************************************************************************
    do i = 1, noFilters
        name = trim(omFilter(i)) // "_INS_MAG"
        newname = trim(omFilter(i)) // "_VEGA_MAG"


        comment = "Source Vega magnitude (" // trim(omfilter(i)) // " filter)"
        call createNewColumns( trim(name), "mag", trim(comment), real32, noColumns4, columnData4, trim(newName) )
        name = trim(omFilter(i)) // "_INS_MAG_ERR"
        newname = trim(omFilter(i)) // "_VEGA_MAG_ERR"

        comment = "1-sigma Vega magnitude error (" // trim(omfilter(i)) // " filter)"
    
        call createNewColumns( trim(name), "mag", trim(comment), real32, noColumns4, columnData4, trim(newName) )
      
     
    end do







    !*********************************************************************************************
    ! Source semi-major axis
    !********************************************************************************************
    do i = 1, noFilters
        name = trim(omFilter(i)) // "_MAJOR_AXIS"
        comment = "Source major axis size (" // trim(omfilter(i)) // " filter)"  
        call createNewColumns( trim(name), "arcsec", trim(comment), real32, &
                               noColumns4, columnData4 )
    end do



    !*********************************************************************************************
    ! Source semi-minor axis
    !********************************************************************************************


    do i = 1, noFilters
        name = trim(omFilter(i)) // "_MINOR_AXIS"
        comment = "Source minor axis size (" // trim(omfilter(i)) // " filter)"  
        call createNewColumns( trim(name),     "arcsec", trim(comment), real32, noColumns4, columnData4 )
    end do


    !*********************************************************************************************
    ! Source position angle
    !********************************************************************************************

    do i = 1, noFilters
        name = trim(omFilter(i)) // "_POSANG"
        comment = "Source position angle (" // trim(omfilter(i)) // " filter)"  
        call createNewColumns( trim(name),     "degrees", trim(comment), real32, noColumns4, columnData4 )
    end do


    !*********************************************************************************************
    ! Quality flags
    !********************************************************************************************

    do i = 1, noFilters
        name = trim(omFilter(i)) // "_QUALITY_FLAG" 
        comment = "Source quality flag (" // trim(omfilter(i)) // " filter)"  
        call createNewColumns( trim(name),     "", trim(comment), integer16, noColumns4, columnData4 )
    end do



    !*********************************************************************************************
    ! Confusion flags
    !********************************************************************************************

!    do i = 1, noFilters
!        name = trim(omFilter(i)) // "_CONFUSION_FLAG" 
!        comment = "Source confusion flag (" // trim(omfilter(i)) // " filter)"  
!        call createNewColumns( trim(name),     "", trim(comment), integer8, noColumns4, columnData4 )
!    end do



    !*********************************************************************************************
    ! Extension flags
    !********************************************************************************************

    do i = 1, noFilters
        name = trim(omFilter(i)) // "_EXTENDED_FLAG" 
        comment = "Source extension flag (" // trim(omfilter(i)) // " filter)"  
        call createNewColumns( trim(name),     "", trim(comment), integer8, noColumns4, columnData4 )
    end do









    !*********************************************************************************************
    ! Source flags
    !********************************************************************************************
!    call createNewColumns( "QFLAG_COMB", "", "", integer16, noColumns4, columnData4)

!    call createNewColumns( "CFLAG_MAX", "", "", integer8, noColumns4, columnData4)


!    call createNewColumns( "EFLAG_MAX", "", "", integer8, noColumns4, columnData4)





    !***************************************************************************************
    !
    ! Columns for the SUMMARY table
    !
    !***************************************************************************************

    noColumns3 = 0
    call createNewColumns( "N_SUMMARY",   "", "Row number within the current table", integer32, noColumns3, columnData3 )
    call createNewColumns( "REVOLUT",     "", "Orbital revolution number of spacecraft", integer32, noColumns3, columnData3 )
    call createNewColumns( "OBS_ID",      "", "Observation identification number", string, 10, noColumns3, columnData3 )
    call createNewColumns( "OBJECT",      "", "Science target during the observation", string, 20, noColumns3, columnData3 )
    call createNewColumns( "FILTERS",     "", "Filters used during the observation", string, 30, noColumns3, columnData3 )
    call createNewColumns( "MJD_START",   "days ", "Mod. Julian date of the start of the obs.", &
                                       real64, noColumns3, columnData3 )
    call createNewColumns( "MJD_END",     "days ", "Mod. Julian date of the end of the obs.", &
                                       real64, noColumns3, columnData3 )

    call createNewColumns( "RA_PNT",      "degrees",  "Right Ascension pointing direction (J2000)", & 
                                       real32, noColumns3, columnData3 )
    call createNewColumns( "DEC_PNT",     "degrees", "Declination pointing direction (J2000)", &
                                       real32, noColumns3, columnData3 )
    call createNewColumns( "RA_PNT_HMS",  " ", "Hr-Min-Sec Right Ascension pointing dir.",  &
                string, 13, noColumns3, columnData3 )
    call createNewColumns( "DEC_PNT_DMS", " ", "Dg-Min-Sec Declination pointing dir.", &
                string, 14, noColumns3, columnData3 )


    ! 2013-07-21 replacing the columns EXP_MIN and EXP_MAX with EXP_UVW2, EXP_UVM2, EXP_UVW1
    ! EXP_U, EXP_B, EXP_V
    !*********************************************************************************************
    ! The min exposure time
    !*********************************************************************************************
    !call createNewColumns( "EXP_MIN", "seconds","Minimum exposure time during the observation", integer32, &
    !     noColumns3, columnData3, "EXPOSURE_MIN" )
    call createNewColumns( "EXP_UVW2", "seconds","Exposure time for the filter UVW2", integer32, &
         noColumns3, columnData3, "EXPOSURE_UVW2" )

    !*********************************************************************************************
    ! The max exposure time
    !*********************************************************************************************   
    !call createNewColumns( "EXP_MAX", "seconds","Maximum exposure time during the observation", integer32, &
    !     noColumns3, columnData3, "EXPOSURE_MAX" )
    call createNewColumns( "EXP_UVM2", "seconds","Exposure time for the filter UVM2", integer32, &
         noColumns3, columnData3, "EXPOSURE_UVM2" )

    call createNewColumns( "EXP_UVW1", "seconds","Exposure time for the filter UVW1", integer32, &
         noColumns3, columnData3, "EXPOSURE_UVW1" )

    call createNewColumns( "EXP_U", "seconds","Exposure time for the filter U", integer32, &
         noColumns3, columnData3, "EXPOSURE_U" )

    call createNewColumns( "EXP_B", "seconds","Exposure time for the filter B", integer32, &
         noColumns3, columnData3, "EXPOSURE_B" )

    call createNewColumns( "EXP_V", "seconds","Exposure time for the filter V", integer32, &
         noColumns3, columnData3, "EXPOSURE_V" )

    !*****************************************
    !  The number of sources
    !****************************************
    call createNewColumns( "N_SOURCES", "", "Number of sources detected during the obs.", integer32, noColumns3, columnData3 )
 

    !********************************************************************************************
    !
    !*******************************************************************************************
    do i = 1, noFilters
        name = "N_SOURCES_" // trim(omFilter(i))
        comment = "Number of sources detected through " //  trim(omFilter(i)) // " filter"
        call createNewColumns( trim(name), " ", trim(comment), &
                 integer32, noColumns3, columnData3 )
      
    end do


    call createNewColumns( "RAOFFSET", "arcsec", "RA offset applied to correct spacecraft point.", &
                   real32, noColumns3, columnData3, "RAOFFSET" )

    call createNewColumns( "DEOFFSET", "arcsec", "Dec offset applied to correct spacecraft point.", &
                real32, noColumns3, columnData3, "DECOFFSET" )

    call createNewColumns( "RAOFFERR", "arcsec", "1-sigma error for RA pointing offset", &
           real32, noColumns3, columnData3, "RAOFFSET_ERR" )

    call createNewColumns( "DEOFFERR", "arcsec", "1-sigma error for Dec pointing offset", &
                real32, noColumns3, columnData3, "DECOFFSET_ERR" )

    call createNewColumns( "RMSRESID", "arcsec", "RMS residual of corr. source pos. v USNO_B", &
                 real32, noColumns3, columnData3, "OFFSET_RMS" )


    !***************************************************************************************
    !
    ! Columns for the AVERAGED_MULTIPLE_DETECTIONS  table in the catalogue
    !
    !***************************************************************************************

    noColumns5 = 0
   

    call createNewColumns( "SRC_NUM", "", "",     integer32, noColumns5, columnData5 )
    
  
  
    !****************************************************************************************************
    !
    !
    !****************************************************************************************************
    call createNewColumns( "N_OBSID", "", "Number of other OBSIDs where this source is detected", &
         integer32, noColumns5, columnData5 )
 




    call createNewColumns( "AV_RA", "degrees", "",    real64,    noColumns5, columnData5)
    call createNewColumns( "AV_DEC", "degrees", "",   real64,    noColumns5, columnData5)
    call createNewColumns( "AV_RA_DE_ERR", "arcsec", "Position error", real32, noColumns5, columnData5 )


    call createNewColumns( "AV_LII", "degrees", "Galactic longitude (J2000)", real64, noColumns5, columnData5 )
    call createNewColumns( "AV_BII", "degrees", "Galactic latitude (J2000)", real64, noColumns5, columnData5 )




   !*********************************************************************************************
   ! Significances 
   !********************************************************************************************

    do i = 1, noFilters
        name = "AVE_" // trim(omFilter(i)) // "_SIGNIF"
        call createNewColumns( trim(name), " ", "", real32, noColumns5, columnData5 )
    end do


    !*********************************************************************************************
    ! Corrected count rates
    !********************************************************************************************

    do i = 1, noFilters
        name = "AVE_" // trim(omFilter(i)) // "_CORR_RATE"
        call createNewColumns( trim(name), "count/sec", "", real32, noColumns5, columnData5 )
        name = "AVE_" // trim(omFilter(i)) // "_CORR_RATE_ERR"
        call createNewColumns( trim(name), "count/sec", "", real32, noColumns5, columnData5 )
    end do







    !*********************************************************************************************
    ! AB fluxes
    !********************************************************************************************

    do i = 1, noFilters
        name = "AVE_" // trim(omFilter(i)) // "_AB_FLUX"
        call createNewColumns( trim(name), "erg/s/cm**2/A", "", real32, noColumns5, columnData5 )
        name = "AVE_" // trim(omFilter(i)) // "_AB_FLUX_ERR"
        call createNewColumns( trim(name), "erg/s/cm**2/A", "", real32, noColumns5, columnData5 )
     
    end do



    !*********************************************************************************************
    ! AB magnitudes
    !********************************************************************************************
    do i = 1, noFilters
        name = "AVE_" // trim(omFilter(i)) // "_AB_MAG"
        call createNewColumns( trim(name), "mag", "", real32, noColumns5, columnData5 )
        name = "AVE_" // trim(omFilter(i)) // "_AB_MAG_ERR"
        call createNewColumns( trim(name), "mag", "", real32, noColumns5, columnData5 )
      
     
    end do



    !*********************************************************************************************
    ! Source shape parameters
    !********************************************************************************************

    call createNewColumns( "MEAN_MAJOR_AXIS",     "arcsec", "", real32, noColumns5, columnData5 )
    call createNewColumns( "MEAN_MINOR_AXIS",     "arcsec", "", real32, noColumns5, columnData5 )
    call createNewColumns( "MEAN_POSANGLE",    "degrees", "", real32, noColumns5, columnData5 )


    !*********************************************************************************************
    ! Source flags
    !********************************************************************************************
    call createNewColumns( "QUALITY_FLAG_COMB", "", "", integer16, noColumns5, columnData5)

    call createNewColumns( "CONFUSED_FLAG_MAX", "", "", integer8, noColumns5, columnData5)


    call createNewColumns( "EXTENDED_FLAG_MAX", "", "", integer8, noColumns5, columnData5)



end subroutine SetUpColumnNames

!****************************************************************
!
!*************************************************************** 
subroutine ObsSummary(mergedDataSetH, newDataSetH)
    type(dataSetT), intent(inout) :: mergeddataSetH
    type(dataSetT), intent(in) :: newdataSetH

    type(tableT) :: summaryTableH
    type(columnT) :: obsIdColumnH
    character(20) :: obSId


    summaryTableH = addTable(mergeddataSetH, "SUMMARY", 1)


    obsIdcolumnH =  addColumn(summaryTableH, "OBS_ID", string, units=" ", &
         comment = "OBSERVATION ID", dimensions = (/20/) )


    obsId = stringAttribute(newDataSetH, "OBS_ID")
    call setstringcell(obsidcolumnh, 0, obsId)
    call release(obsidColumnH)

end subroutine ObsSummary


!****************************************************************
! Updates the master catalogue file SUMMARY table
! with the data in the srclistfile
!
!
!*************************************************************** 
subroutine UpdateObsSummaryTable( mergedmasterCatalogue, srclistFile )

    character(*), intent(in) ::  mergedmasterCatalogue, srclistfile
    type(dataSetT) :: mergeddataSetH 
    type(dataSetT) :: newdataSetH

    type(tableT) :: mergedsummaryTableH 
    type(tableT) :: masterSummaryTableH
    type(tableT) :: updateTableH

    integer(int32) :: nMergedRows, nNewRows
    integer :: i, j, n
    character(10) :: newobsid, mergedObsId
    character(20) :: newobject
    integer(int32) :: revNo
    real(double) :: newObsidNo, mergedObsidno, mjdStart, mjdEnd, oldmjd
    real(double) :: raPoint, oldRAPoint
    real(single) :: value
    integer :: iMerged, iNew, nfilts
    type(tableData) :: mergedTableData, updatetabledata
    character(30) :: filters
    
    character(30) :: revString 
    real(double) :: raPnt, decPnt
    character(13) :: raString
    character(14) :: decString
    integer(int32) :: number

    real(single) :: minExposure=100000.0, maxExposure=-10000.0
    real(single) :: filtW2Exposure=0.0, filtM2Exposure=0.0, filtW1Exposure=0.0
    real(single) :: filtUUExposure=0.0, filtBBExposure=0.0, filtVVExposure=0.0

!if(debug) then
!write(*, *) "Line 1948 newobsid=", newObsId
!end if



    newDataSetH = dataSet( trim( srcListFile), READ )
    !write(*,*) "L1941 newDataSetH is opened for reading"

    nNewRows = numberOfRows ( Table( newDataSetH, "SRCLIST") )
    newObsId = stringAttribute( newdataSetH, "OBS_ID" )    ! Observation ID


!if(debug) then
!write(*, *) "Line 1964 newobsid=", newObsId
!end if


    !******************************************************************************
    ! Need to check that this obsid does not already exist in the summary table
    !******************************************************************************


    mergedDataSetH = dataSet( trim( mergedMasterCatalogue ), MODIFY )
    mergedSummaryTableH = table( mergedDataSetH, "SUMMARY", MODIFY )
    nMergedRows = numberOfRows( mergedSummaryTableH ) 

!if(debug) then
!write(*, *) "Line 1979 newobsid=", newObsId
!end if

    if( obsIdExists(mergedSummaryTableH, newObsId)) then

        call release( newDataSetH )
        call release( mergedDataSetH ) 
        write(errstr,'(a, a, a)') "OBSID ", trim(newObsId), &
            " already exits in the SUMMARY table- ignoring"
            call message(AppMsg, Verbose)
     
        stop
    end if




    newObject = stringAttribute( newdataSetH, "OBJECT" )   ! OBJECT
  
    ! 2013-07-21 replacing the min andmax exposures with those of the filters
    !minExposure = real32Attribute( newdataSetH, "EXP_MIN" )   ! Min exposure
    !maxExposure = real32Attribute( newdataSetH, "EXP_MAX" )   ! Max exposure

    filtW2Exposure = real32Attribute( newdataSetH, "EXP_UVW2" )   ! exposure UVW2
    filtM2Exposure = real32Attribute( newdataSetH, "EXP_UVM2" )   ! exposure UVM2
    filtW1Exposure = real32Attribute( newdataSetH, "EXP_UVW1" )   ! exposure UVW1
    filtUUExposure = real32Attribute( newdataSetH, "EXP_U" )   ! exposure U
    filtBBExposure = real32Attribute( newdataSetH, "EXP_B" )   ! exposure B
    filtVVExposure = real32Attribute( newdataSetH, "EXP_V" )   ! exposure U

    write(*,*) "L2505 exp w2=", filtW2Exposure, " m2=", filtM2Exposure, " w1=", filtW1Exposure, &
         " u=",filtUUExposure, " b=",filtBBExposure, " v=", filtVVExposure
  
    !*************************************************************
    ! Get the revolution number
    !*************************************************************

    if ( hasAttribute( newdatasetH, "REVOLUT" )) then

        revString =  stringAttribute( newdataSetH, "REVOLUT")
        read( revString, '(i06)' ) revNO
       
    else
        revNo = -999
    end if


!if(debug) write(*, *) "Line 2004"

    !*********************************************************************
    ! Get the date and convert to a MJD
    !*********************************************************************
    mjdStart = ModifiedJulianDate( stringAttribute(newDataSetH, "DATE-OBS"))    

    mjdEnd = ModifiedJulianDate( stringAttribute(newDataSetH, "DATE-END"))    


    raPoint = real64Attribute(newDataSetH, "RA_PNT")  ! RA pointing direction


    !***************************************************************************************
    ! Find out where the new observation should go by comparing the RA pointing directions
    !***************************************************************************************    

!debug = .true.
    !write(*,*) "L2022 ..."
    write(*,*) "L2548 calling loadTableColumnData  nocolumns3=", nocolumns3 
    call loadtableColumnData( mergeddataseth, mergedsummarytableH, mergedTabledata, nocolumns3, &
         columndata3) 

    write(*,*) "L2552 nocolumns3=", nocolumns3

!if (debug) write(*, *) "updateobssummarytable line 2024" 


    j = nmergedRows

!    do i = 0, nMergedRows - 1


!        oldRAPoint = mergedtableData%columnData(9)%real32arraydata(i) 


!        if (raPoint .lt. oldRAPoint) then
!            j = i  ! The row where the new observation details are stored
!            exit
     
!       end if

!    end do


    obs_summary_no = nMergedRows + 1  ! The current observation summary no

!write(*, *) "updateobssummarytable line 1505" 

    !*************************************************
    ! Insert a new row to store the new observation
    !************************************************
    call insertRows( mergedSummaryTableH, j,  1 )

    !if (debug) write(*, *) "L2058 updateobssummarytable ..." 
    write(*,*) "L2583 calling loadTableColumnData  nocolumns3=", nocolumns3 
    call loadtableColumnData( mergeddataseth, mergedsummarytableH, mergedTabledata, &
         nocolumns3, columndata3) 
    write(*,*) "L2586 nocolumns3=", nocolumns3 

    !if (debug) write(*, *) "updateobssummarytable line 2066" 
    
    ! Here we are taking the data set to be ingested 
    updateTableH = table( newDataSetH, "SRCLIST", read)
    !write(*,*) "L2068 ..."
    write(*,*) "L2593 calling loadTableColumnData  nocolumns3=", nocolumns3 
    call loadtableColumnData( newdataseth, updatetableH, updateTabledata, &
         nocolumns3, columndata3) 
    write(*,*) "L2596 nocolumns3=", nocolumns3 
    !if (debug) write(*, *) "updateobssummarytable line 2074" 


    i = 1

    !if(debug) write(*, *) "L2070 updatingtablecolumns, j=", j, " i=", i
    
    call updateTableColumns(j, i, mergedTableData, updateTableData)


    !if (debug) write(*, *) "L2075 j=", j 

  
    j = j - 1
    !if (debug) write(*, *) "L2079 j=", j
 
    !****************************************************************************
    ! Store the data which is stored as attributes in the omsrclistcomb file
    !****************************************************************************
 

!write(*, *) "Line 1590"


    mergedtableData%columnData(1)%integer32ArrayData(j) = nmergedRows + 1
    
    mergedtableData%columnData(2)%integer32ArrayData(j) = revNo
  
    mergedTableData%columnData(6)%real64ArrayData(j) = mjdStart

    mergedTableData%columnData(7)%real64ArrayData(j) = mjdEnd


!if (debug) write(*, *) "updateobssummarytable line 2106" 


    !**********************************************************************
    !
    ! Pointing directions
    !**********************************************************************

    raPnt = mergedtableData%columnData(8)%real32arraydata(j) 
    decPnt = mergedtableData%columnData(9)%real32arraydata(j) 

    !*********************************************************************
    ! Convert to strings
    !*********************************************************************
    call createRaDecStrings(raPnt, decPnt, raString, decString)

!write(*, *) "Line 2051 setting string cell"
!write(*, *) "raPnt=", raPnt, " rastring=", raString


    call setStringCell( mergedtableData%columnData(10)%columnh,  j, raString )

    call setStringCell( mergedtableData%columnData(11)%columnh,  j, decString )


    !****************************************************************************
    ! Store the minimum and maximum exposure times
    !****************************************************************************
    !mergedTableData%columnData(12)%integer32ArrayData(j) = int(minExposure + 0.00001)
    mergedTableData%columnData(12)%integer32ArrayData(j) = int(filtW2Exposure + 0.00001)


    !mergedTableData%columnData(13)%integer32ArrayData(j) = int(maxExposure + 0.00001)
    mergedTableData%columnData(13)%integer32ArrayData(j) = int(filtM2Exposure + 0.00001)
    mergedTableData%columnData(14)%integer32ArrayData(j) = int(filtW1Exposure + 0.00001)
    mergedTableData%columnData(15)%integer32ArrayData(j) = int(filtUUExposure + 0.00001)
    mergedTableData%columnData(16)%integer32ArrayData(j) = int(filtBBExposure + 0.00001)
    mergedTableData%columnData(17)%integer32ArrayData(j) = int(filtVVExposure + 0.00001)

    !mergedTableData%columnData(14)%integer32ArrayData(j) = nnewRows
    mergedTableData%columnData(18)%integer32ArrayData(j) = nnewRows

    !*****************************************************************************
    ! Find out the fraction of sources in each filter and produce a list
    !****************************************************************************
    write(*,*) "L2138 finding the fraction of sources in each filter"
    filters = "" ! The list of filters
    nfilts = 0

    do i = 1, size(omFilter)
      
       number = getFilterNumber1( updatetableH, trim(omFilter(i) ))
       !mergedTableData%columnData(14 + i)%integer32ArrayData(j) = number
       mergedTableData%columnData(18 + i)%integer32ArrayData(j) = number
       write(*,*) "L2675 i=", i, " number=", number
       if (number > 0) then         
          if (nfilts > 0) then
             filters = trim(filters) // ","              
          end if
          filters = trim(filters) // " " // trim(omFilter(i))
          nfilts = nfilts + 1        
       end if
       
    end do
    write(*,*) "L2685 nfilts=", nfilts
    
    call setStringCell( mergedtableData%columnData(5)%columnh,  j, filters )
    
    call release( updatetableH )
    call release( mergedsummaryTableH )
    call release( mergeddataSetH )
    call release( newdataSetH )

end subroutine UpdateObsSummaryTable


!****************************************************************
! Updates the master catalogue header
! 
!
!
!*************************************************************** 
subroutine testHeader( mergedmasterCatalogue )

    character(*), intent(in) ::  mergedmasterCatalogue
    type(dataSetT) :: mergeddataSetH

    type(AttributableT) thisAttributable
    type(AttributeT) att
    integer i

    !type(dataSetT) :: newdataSetH

    !type(tableT) :: mergedsummaryTableH
    !type(tableT) :: masterSummaryTableH
    !type(tableT) :: updateTableH

    !integer(int32) :: nMergedRows!, nNewRows
    !integer :: i, j, n
    !character(10) :: newobsid, mergedObsId
    !character(20) :: newobject
    !integer(int32) :: revNo
    !real(double) :: newObsidNo, mergedObsidno, mjdStart, mjdEnd, oldmjd
    !real(double) :: raPoint, oldRAPoint
    !real(single) :: value
    !integer :: iMerged, iNew, nfilts
    !type(tableData) :: mergedTableData, updatetabledata
    !character(30) :: filters
    
    !character(30) :: revString 
    !real(double) :: raPnt, decPnt
    !character(13) :: raString
    !character(14) :: decString
    !integer(int32) :: number

    !real(single) :: minExposure=100000.0, maxExposure=-10000.0
 

    !newDataSetH = dataSet( trim( srcListFile), READ )
  

    !nNewRows = numberOfRows ( Table( newDataSetH, "SRCLIST") )
    !newObsId = stringAttribute( newdataSetH, "OBS_ID" )    ! Observation ID




    !******************************************************************************
    ! Need to check that this obsid soes not already exist in the summary table
    !******************************************************************************


    mergedDataSetH = dataSet( trim( mergedMasterCatalogue ), MODIFY )

    !mergedSummaryTableH = table( mergedDataSetH, "SUMMARY", MODIFY )
    !nMergedRows = numberOfRows( mergedSummaryTableH ) 

    if (hasAttribute(mergedDataSetH, "BITPIX")) then
       write(*,*) "l2245 the data set ", trim(mergedMasterCatalogue), &
            " has the attribute BITPIX"
    else
       write(*,*) "l2248 the attribute BITPIX was not found  "

    endif

    thisAttributable=attributable(mergedDataSetH)
    do i = 0, numberOfAttributes( thisAttributable ) - 1
       att = attribute( thisAttributable, i )
       write(*,*) "L2261 i=", i, name( att ), stringAttribute( att ), units( att ), label( att )
    end do

    !call release(thisAttributable)
    !call release( mergedsummaryTableH )
    call release( mergeddataSetH )
    


end subroutine testHeader

!************************************************************************************
!
!
!
!
!***********************************************************************************
subroutine updateTableColumns(imerged, itable, mergedTableData, iTableData)

    integer, intent(inout) :: imerged, itable
    type(tableData), intent(inout) :: mergedTableData

    type(tableData), intent(in) :: iTabledata
    character(20) :: columnname    

    integer(int16) :: i, k
    integer(int8) :: j
  

    !if(debug) then
        !write(*, *) "L2271 updatetablecolumns, no merged columns=",  &
        !     mergedTabledata%noColumns
        !write(*, *) "L2273 updatetablecolumns, no update columns=",  &
        !     iTabledata%noColumns

        !write(*, *) "The columns in the merged table"
        !do i = 1, mergedTableData%noColumns
        !    write(*, *) "L2277 i=", i, " column name=",  &
        !         mergedTableData%columnData(i)%name, &
        !         " update column=", iTableData%columnData(i)%name  
        ! end do
      !end if

    !if (debug) write(*, *) "L2283 number of columns =", mergedTabledata%noColumns 
    do i = 1, mergedTabledata%noColumns

       
       !if(debug .and. i==17) then
       !   write(*,*) "L2292 ........................ i=", i, &
       !        " imerged=", imerged, " itable=", itable
       !   write(*, *) "L2286 i=", i, " column name=",  &
       !     mergedTableData%columnData(i)%name
!
!          !write(*,*) "L2306 -------------- mergedTableData-columnData i=", i, ":"
!          !write(*,*) mergedTableData%columnData(i)%integer16ArrayData
!          !write(*,*) "L2308 --------------"
!                        
!       endif
       if ( iTableData%columnExists(i) ) then  
          ! The column exists in the update file
!debug = .true.

          !if(debug .and. i==17 .and. itable < 3) then
          !    write(*, *) "L2293 column name=", itableData%columnData(i)%name, &
          !         " exists in the update table, itable=", itable 
          !    !write(*, *) "L2295 i=", i, "  column exists name= ", &
          !    !     itableData%columnData(i)%name 
          !    write(*, *) "L2297 updatetablecolumns, datatype= ", itableData%datatype(i)!
!
          !    write(*,*) "L2323 -------------- iTableData-columnData i=", i, ":"
          !    write(*,*) iTableData%columnData(i)%integer16ArrayData
          !    write(*,*) "L2325 --------------"
          ! end if

          select case( mergedTableData%datatype(i) )


          case (1)
             !if(debug .and. i==17) write(*, *) "L2304, imerged=", &
             !     imerged, " itable=", itable
             if ( abs( itabledata%columnData(i)%real32ArrayData(iTable) - nullValue) &
                  .lt. 1.0) then  
                call setToNull( mergedtableData%columnData(i)%columnh, imerged)
             else
                mergedtableData%columnData(i)%real32ArrayData(imerged) = &
                     itableData%columnData(i)%real32ArrayData(itable)
             end if
             !if(debug .and. i==17) write(*, *) "L2312"
          case (2)

             !if(debug .and. i==17) &
             !     write(*, *) "L2316 updatetablecolumns itable value=", &
             !     itableData%columnData(i)%real64ArrayData(itable)

             if ( abs( itabledata%columnData(i)%real64ArrayData(iTable) - nullValue) &
                  .lt. 1.0) then
                call setToNull( mergedtableData%columnData(i)%columnh, imerged)
             else
                mergedtableData%columnData(i)%real64ArrayData(imerged) = &
                     itableData%columnData(i)%real64ArrayData(itable)
             end if
            
             !if(debug .and. i==17) write(*, *) "L2327"

          case (3) ! Integer 8

             !if(debug .and. i==17)  then 
             !   write(*, *) "L2332 updatetablecolumns i=", i, " imerged=", &
             !        imerged, " itable=", itable
             !   write(*, *) "L2334 update value=", &
             !        itabledata%columnData(i)%integer8ArrayData(iTable)
             !   ! call fatal("stopped at line 2227")  
             !end if      
             if ( isNull( iTableData%columnData(i)%columnH, iTable) ) then
                !if(debug .and. i==17) then 
                !   write(*, *) "L2340 updatetablecolumns setting to null  i=", &
                !        i, " imerged=", imerged, " itable=", itable
                !   !call fatal("stopped at line 2211")
                !end if
                 call setToNull( mergedtableData%columnData(i)%columnh, imerged)
             else
                !write(*,*) "L2346"
                mergedtableData%columnData(i)%integer8ArrayData(imerged) = &
                     itableData%columnData(i)%integer8ArrayData(itable)
             end if
         

             !if(debug .and. i==17) write(*, *) "L2352 updatetablecolumns"

          case (4)  ! Integer 16
             !if(debug .and. i==17 .and. itable < 3) write(*, *) "L2355  integer 16- update value=", &
             !     itableData%columnData(i)%integer16ArrayData(itable)
             
             if ( isNull( iTableData%columnData(i)%columnH, itable )) then
                call setToNull( mergedtableData%columnData(i)%columnh, imerged)
                !if(debug .and. i==17) then 
                !   write(*, *) "L2360 setting to null"  
!
!                   write(*, *) "L2362 Column name=", name(iTableData%columnData(i)%columnh)
!                   write(*, *) "L2363 itabledata=",  &
!                        itabledata%columnData(i)%integer16ArrayData(iTable)
!                   write(*, *) "L2365 imerged=", imerged, " mergedtabledatavalue=", &
!                        mergedtableData%columnData(i)%integer16ArrayData(imerged) 
!                endif

                !call fatal("stopped at line 2234 setting int16 to null value")
             else
                !if(debug .and. i==17) write(*, *) "L2370 not  setting to null"  
                
                mergedtableData%columnData(i)%integer16ArrayData(imerged) = &
                     itableData%columnData(i)%integer16ArrayData(itable)
             end if
         
 




             !if(debug .and. i==17)   then 
             !   write(*, *) "L2382 updatetablecolumns"
             !end if

          case (5) ! Integer 32

             !if(debug .and. i==17) write(*, *) "L2387 updatetablecolumns"

             if ( itabledata%columnData(i)%integer32ArrayData(iTable) .eq. &
                  int32NullValue) then
                call setToNull( mergedtableData%columnData(i)%columnh, imerged)
             else                
                mergedtableData%columnData(i)%integer32ArrayData(imerged) = &
                     itableData%columnData(i)%integer32ArrayData(itable)
             end if
        
          case ( 6 )
             !if(debug .and. i==17) then
             !   write(*, *) "L2399 updatetablecolumns "

             !write(*, *) "columnname=", name( mergedtableData%columnData(i)%columnh )
             !write(*, *) "columnname=", name( itableData%columnData(i)%columnh )
             !end if
             call setStringCell( mergedtableData%columnData(i)%columnh,  imerged, &
                  stringCell( itableData%columnData(i)%columnH, itable) )


             !if(debug .and. i==17)  write(*, *) "L2407 updatetablecolumns "
             
          end select

          !if(debug .and. i==17) write(*, *) "L2411 column doesn't exist"

       else if ( iTableData%attributeExists(i) ) then  
          ! The attribute exists in the update file

          !if(debug .and. i==17) write(*, *) "L2416 i=", i, "  column name= ", &
          !     trim(name(  mergedtableData%columnData(i)%columnh ))
          select case( mergedTableData%datatype(i) )             
          case (1)
             !if(debug .and. i==17) write(*, *) "L2423 case 1"

             if (abs(itabledata%columnData(i)%real32value - nullvalue) .lt. 1.0) then
                call setToNull( mergedtableData%columnData(i)%columnh, imerged)
             else
                mergedtableData%columnData(i)%real32ArrayData(imerged) = &
                     itableData%columnData(i)%real32value
                
             end if
             !if(debug .and. i==17) write(*, *) "L2434"
          case (2)
             !if(debug .and. i==17) then 
             !   write(*, *) "L2434  case 2 itable value=", &
             !        itableData%columnData(i)%real64ArrayData(itable)
!
             !   write(*, *) "L2437"
             !endif
             mergedtableData%columnData(i)%real64ArrayData(imerged) = &
                  itableData%columnData(i)%real64value
             
          case (3)
          

             !if(debug .and. i==17) write(*, *) "L2445 updatetablecolumns case 3 "


              if ( itabledata%columnData(i)%int8value .eq. int8NullValue) then

                 !if(debug .and. i==17) then
                 !   write(*, *) "L2451 updatetablecolumns setting to null int8"
                 !   !call fatal("stopped at line 2381")
                 !end if
                 call setToNull( mergedtableData%columnData(i)%columnh, imerged)
             else
                mergedtableData%columnData(i)%integer8ArrayData(imerged) = &
                     itableData%columnData(i)%int8value
             end if
             
          case (4)

             !if(debug .and. i==17) write(*, *) "L2462 updatetablecolumns case 4"


            if ( itabledata%columnData(i)%int16value .eq. int16NullValue) then
          
               !if(debug .and. i==17) then
               !   write(*, *) "updatetablecolumns line 2468 setting to null"
               !   !   call fatal("stopped at line 2399")
               !end if
               call setToNull( mergedtableData%columnData(i)%columnh, imerged)
   
               !call fatal("stopped at line 2383 setting int16 to null value")
             else
                 mergedtableData%columnData(i)%integer16ArrayData(imerged) = &
                 itableData%columnData(i)%int16value
             end if

          case (5)

             if ( itabledata%columnData(i)%int32value .eq. int32NullValue) then

                !if(debug .and. i==17) then
                !   write(*, *) "L2484 updatetablecolumns line 2419 setting to null"
                !   !call fatal("stopped at line 2399")
                !end if
                call setToNull( mergedtableData%columnData(i)%columnh, imerged)
             else
                mergedtableData%columnData(i)%integer32ArrayData(imerged) = &
                     itableData%columnData(i)%int32value

             end if
             !if(debug .and. i==17) write(*, *) "L2493 Value=", &
             !     itableData%columnData(i)%int32value
          case ( 6 )
             if(debug .and. i==17)  write(*, *) "L2495 i=", i, " columnname= ", &
                  name(  mergedtableData%columnData(i)%columnh )

             if( .not. hasColumn(mergedTableData%tableh, &
                  name(  mergedtableData%columnData(i)%columnh) ) ) then
                 call fatal("bug at line 2500")
             end if
             !if(debug .and. i==17) then 
             !   write(*, *) "L2502 columnname=", &
             !        name( mergedtableData%columnData(i)%columnh )
             !   write(*, *) "L2504 i=", i
             !   
             !   write(*, *) "L2506 Value=", itableData%columnData(i)%stringValue
             !endif
              
             call setStringCell( mergedtableData%columnData(i)%columnh,  imerged, &
                  itableData%columnData(i)%stringvalue  )

             !if(debug .and. i==17) then 
             !   write(*, *) "L2515 updatetablecolumns column value ", &
             !        stringcell( mergedtableData%columnData(i)%columnh,  imerged)
             !endif
          end select

       else ! Column doesn't exist in the table - set rows to null

!debug = .true.

          !if(debug .and. i==17) then
          !   write(*, *) "L2524 i=", i, &
          !        " column doesnt exists setting rows to null name= ", &
          !        trim(name(  mergedtableData%columnData(i)%columnh ))
          !end if

          select case( mergedTableData%datatype(i) )

          case (1)

             !if(debug .and. i==17) then
             !   write(*, *) "L2534 case 1 i=", i, " column doesnt exists name= ", &
             !        name(  mergedtableData%columnData(i)%columnh )
             !end if
             call setToNull( mergedtableData%columnData(i)%columnh, imerged)
          case (2)

             !if(debug .and. i==17) then
             !   write(*, *)"L2541 case 2 i=",  i, " column doesnt exists name= ", &
             !        name(  mergedtableData%columnData(i)%columnh )
             !end if
             call setToNull( mergedtableData%columnData(i)%columnh, imerged)

          case (3)


!if(debug) then

!call fatal("stopped at line 2515")

!end if
              call setToNull( mergedtableData%columnData(i)%columnH, imerged)


          case (4)


!call fatal("stopped at line 2426")

!if(debug) then
!          write(*, *) i, ") Line 2331 column doesnt exists name= ", name(  mergedtableData%columnData(i)%columnh )
!end if

             call setToNull( mergedTableData%columnData(i)%columnH, imerged)

!             if ( .not. isNull( mergedTableData%columnData(i)%columnH, imerged) ) then

!                 call fatal("BUG AT LINE 2539")

!             end if   

          case (5)
             call setToNull( mergedtableData%columnData(i)%columnH, imerged)

          case ( 6 )

             call setStringCell( mergedtableData%columnData(i)%columnh,  imerged, " " )


          end select




       end if



       !if(debug) write(*, *) "L2591"

    end do

 
    imerged = imerged + 1
    itable = itable + 1
    !if(debug) then
    !   write(*, *) "L2596  leaving updatetablecolumns imerged=", imerged, &
    !        " itable=", itable
    !end if
   
end subroutine updateTableColumns




!************************************************************************************
!
!
!
!***********************************************************************************
subroutine setMergedTable(mergedTableData, iTableData, index)

   
    type(tableData), intent(inout) :: mergedTableData

    type(tableData), intent(in) :: iTabledata
    integer(int32), pointer :: index(:)

    integer(int16) :: i
    integer(int32) :: j, n
   
    type(ColumnT) :: mergedColumn, updateColumn
    type(datasetT) :: mergedDataSet, updateDataSet

 
    n = size(index) - 1
!debug = .true.
!if(debug) then
!write(*, *) "setMergedTable line 2561"
!end if

    !*******************************************************************
    ! Loop through all the columns
    !*******************************************************************
     
    do i = 1, mergedTabledata%noColumns

        if ( iTableData%columnExists(i) ) then  ! The column exists in the update file


            select case( mergedTableData%datatype(i) )

            case (1)  ! Real32
  

            
                do j = 0, n 
                    if (abs(itabledata%columnData(i)%real32ArrayData(j) - nullvalue) .lt. 1.0) then
                        call setToNull( mergedtableData%columnData(i)%columnh, index(j))
                    else         
                        mergedtableData%columnData(i)%real32ArrayData(index(j)) = &
                            itableData%columnData(i)%real32ArrayData(j)
                    end if

                end do
   
            case (2)  ! Real 64

    
                do j = 0, n 
                    if (abs(itabledata%columnData(i)%real64ArrayData(j) - nullvalue) .lt. 1.0) then
                        call setToNull( mergedtableData%columnData(i)%columnh, index(j))
                    else         
                        mergedtableData%columnData(i)%real64ArrayData(index(j)) = &
                            itableData%columnData(i)%real64ArrayData(j)
                    end if

                end do
   

               



            case (3)  ! Integer 8



!if(debug) then
!write(*, *) "setMergedTable line 2604"
!end if





                do j = 0, n

!if(debug) then
!write(*, *) j, ") update value=",  itableData%columnData(i)%integer16ArrayData(j)
!end if

                  


                    if(   isNull( itableData%columnData(i)%columnH, j ) ) then




!if(debug) then
!write(*, *) "Line 2621 setting to null"
!call fatal("stopped at line 2663")
!end if


                        call setToNull( mergedtableData%columnData(i)%columnh, index(j))
       

                    else
                        mergedtableData%columnData(i)%integer8ArrayData(index(j)) = &
                            itableData%columnData(i)%integer8ArrayData(j)
       
                    end if
                end do


           case(4) ! Integer 16                 

!if(debug) then
!write(*, *) "setMergedTable line 2615"
!end if



                do j = 0, n 

!if(debug) then
!write(*, *) j, ") update value=",  itableData%columnData(i)%integer16ArrayData(j)
!end if




                    if(  isNull( itableData%columnData(i)%columnH, j) ) then



! call fatal("stopped at line 2669 setting int16 to null value")


!if(debug) then
!write(*, *) "Line 2634 setting to null"
!end if

                        call setToNull( mergedtableData%columnData(i)%columnh, index(j))
       

                    else
                        mergedtableData%columnData(i)%integer16ArrayData(index(j)) = &
                            itableData%columnData(i)%integer16ArrayData(j)
       
                    end if
                end do

            case (5)

                do j = 0, n

                    if(   isNull( itableData%columnData(i)%columnH, j )) then

                        call setToNull( mergedtableData%columnData(i)%columnh, index(j))
       

                    else
                        mergedtableData%columnData(i)%integer32ArrayData(index(j)) = &
                            itableData%columnData(i)%integer32ArrayData(j)
                    end if
                end do
            case ( 6 )




!***************************************************************************************
! THERE IS A PROBLEM ACCESSING STRING CELLS - VERY SLOW
!****************************************************************************************

  

                do j = 0, n
!                    call setStringCell( mergedtableData%columnData(i)%columnh,  index(j), &
!                        stringCell( itableData%columnData(i)%columnH, j) )
 


         call setStringCell( mergedtableData%columnData(i)%columnh,  index(j), &
                        "TWIT" )
 




               end do


  

            end select


        else if ( iTableData%attributeExists(i) ) then  ! The column exists in the update file

      
            select case( mergedTableData%datatype(i) )

            case (1)  ! real32



                do j = 0, n
                    if (abs(itabledata%columnData(i)%real32value - nullvalue) .lt. 1.0) then
                        call setToNull( mergedtableData%columnData(i)%columnh, index(j))
               call fatal("stopped at line 2767")
                    else
                        mergedtableData%columnData(i)%real32ArrayData(index(j)) = &
                            itableData%columnData(i)%real32value
                   
                    end if





                end do

            case (2)  ! real64
                do j = 0, n
                    mergedtableData%columnData(i)%real64ArrayData(index(j)) = &
                        itableData%columnData(i)%real64value
                end do

            case (3)   ! Integer8
           
                do j = 0, n    

                
                
                    mergedtableData%columnData(i)%integer8ArrayData(index(j)) = &
                        itableData%columnData(i)%int8value
                end do

            case (4)   ! Integer16

                do j = 0, n            
                    mergedtableData%columnData(i)%integer16ArrayData(index(j)) = &
                        itableData%columnData(i)%int16value
                 end do



            case (5)   ! Integer 32

                do j = 0, n            
                    mergedtableData%columnData(i)%integer32ArrayData(index(j)) = &
                        itableData%columnData(i)%int32value
                end do
            case ( 6 )

                do j = 0, n            
                    call setStringCell( mergedtableData%columnData(i)%columnh,  index(j), &
                        itableData%columnData(i)%stringvalue  )
                end do

            end select

        else ! Column doesn't exist in the table - set rows to null


            select case( mergedTableData%datatype(i) )

            case (1)
                 

                do j = 0, n            
                    call setToNull( mergedtableData%columnData(i)%columnh, index(j))
                end do
            case (2)
                do j = 0, n                       
                    call setToNull( mergedtableData%columnData(i)%columnh, index(j))
                end do
            case (3)


!                if(debug) then
!                     write(*, *) "Line 2813 setting int8 values to null- n=", (n + 1)
!                     call fatal("stopped at line 2824")
!                end if
                do j = 0, n                       
                    call setToNull( mergedtableData%columnData(i)%columnh, index(j))
                end do
      


          


            case (4)

                do j = 0, n                       
                    call setToNull( mergedtableData%columnData(i)%columnH, index(j))
                end do

            case (5)
            
                do j = 0, n                       
                    call setToNull( mergedtableData%columnData(i)%columnH, index(j))
                end do


            case ( 6 )
                do j = 0, n                       
                    call setStringCell( mergedtableData%columnData(i)%columnh,  index(j), " " )
                end do

          end select

       end if
    end do


end subroutine setMergedTable

!******************************************************
!
!*****************************************************
subroutine addInt32DataColumn(tableh, columnName, value)
    type(tablet), intent(inout) :: tableH
    character(*), intent(in) :: columnName
    integer(int32), intent(in) :: value
    integer(int32), pointer :: int32Ptr(:) => null()
    type(ColumnT) :: columnH


    if ( hasColumn(tableH, trim(columnName) ) ) then
       call fatal("column exists at line 861")
    end if


    columnH = addColumn(tableh, trim(columnname), integer32, units=" ", &
         comment = "No of sources")

    int32Ptr => int32data(columnH) 
    int32Ptr(0) = value
    call release( columnH )    


end subroutine addInt32DataColumn

!***********************************************************************************
!
!
! RA comparison function for the SRCLIST RA column
!
!
!**********************************************************************************
integer function Compare1(newtableData, oldmastertableData, inew, imaster)  
    type(tableData), intent(in) :: newtableData, oldMasterTableData
    integer, intent(in) :: inew, imaster
    real(double) :: raNew, raMaster

    raNew = newtableData%columnData(raCompare1Index)%real64arrayData(inew)
    raMaster = oldMasterTableData%columnData(racompare1Index)%real64arrayData(imaster)


!    write(*, *) "compare1 inew=", inew, " imaster=", imaster, &
! " ramaster=", ramaster, " ranew=", ranew

!    write(*, *) "olad=", associated(  oldmastertableData%columnData(1)%real64arraydata)
!    write(*, *) "new=", associated(  newtableData%columnData(1)%real64arraydata)
!    write(*, *) "name=",  newtableData%columnData(1)%name



    if( raNew .gt. raMaster ) then

       compare1 = 1

    elseif( raNew .lt. raMaster ) then

       compare1 = -1
    else
       compare1 = 0

    end if
!    write(*, *) "leaving compare1 result=", compare1
end function compare1

!***********************************************************************************
!
!
!**********************************************************************************
integer function Compare2(newtableData, oldmastertableData, inew, imaster)  
    type(tableData), intent(in) :: newtableData, oldMasterTableData
    integer, intent(in) :: inew, imaster

 

    if( newtableData%columnData(8)%real32arrayData(inew) .gt. &
         oldmastertableData%columnData(8)%real32arraydata(imaster) ) then

       compare2 = 1

    elseif( newtableData%columnData(8)%real32arrayData(inew) .lt. &
         oldmastertableData%columnData(8)%real32arraydata(imaster) ) then

       compare2 = -1
    else
       compare2 = 0

    end if
end function compare2


!***********************************************************************************
!
! For updating the observation summary table
!
!**********************************************************************************
integer function Compare3(newtableData, oldmastertableData, inew, imaster)  
    type(tableData), intent(in) :: newtableData, oldMasterTableData
    integer, intent(in) :: inew, imaster
    real(double) :: newRA, oldRa
   



    oldRA = oldmastertableData%columnData(9)%real32arraydata(imaster) 

    newRA = newTableData%columnData(9)%real32arraydata(inew) 




    if( newRA .gt. oldRA ) then

       compare3 = 1

    elseif( newRA .lt. oldRA ) then

       compare3 = -1
    else
       compare3 = 0

    end if

end function compare3

!******************************************************************
!
!
!*****************************************************************
subroutine InsertTable( dataSetH, tableName, noColumns, columndata)
    type(dataSetT), intent(inout) :: dataSetH

    character(*), intent(in) :: tableName
    integer(int16), intent(in) :: nocolumns
    type(newColumn) :: columnData(:)

    type(TableT) :: tableH
    type(Columnt) :: columnH
    integer :: i


    tableH = addTable( dataSetH, trim(tableName), 0 )

!if(debug)  then
!write(*, *) "Insert table line 2806"
!write(*, *) "integer8=", integer8, " integer16=", integer16, " integer32=", integer32
!end if
    do i = 1, noColumns

!       if(debug) write(*, *) i, " inserting column ", columndata(i)%name, " type=", columnData(i)%type
       if ( columndata(i)%type .eq. STRING ) then

          columnH =  addColumn( tableH, trim( columndata(i)%name ) , columndata(i)%type, units = trim( columndata(i)%units), &
               comment = trim (columndata(i)%comment ), dimensions = (/columndata(i)%width/) )
       else


          columnH =  addColumn( tableH, trim( columndata(i)%name ) , columndata(i)%type, units = trim( columndata(i)%units), &
               comment = trim( columndata(i)%comment) )

          select case(columnData(i)%type)
              case(3) ! integer 8
                  call setNullValue( columnH, int8NullValue)
              case(4) ! Integer 16
                  call setNullValue( columnH, int16NullValue)
        
              case(5) ! Integer 32
                  call setNullValue( columnH, int32NullValue)
        
          end select



       end if
       call release( columnH )

!       if(debug) then
!           write(*, *) "AFTER ADDING COLUMN"
!           write(*, *) "Name=", trim(name(columnh)), " dal type=", datatype(columnh)
!        end if
    end do

    call release ( tableH )


end subroutine insertTable

!****************************************************************
! 
!
!***************************************************************
subroutine createNewColumns1( oldName, units, comment, type, noColumns, columnData, newName)
    character(*), intent(in) :: oldName, units, comment
    integer, intent(in) :: type
    integer(int16), intent(inout) :: noColumns
    type(newColumn), intent(inout) :: columnData(:)
    character(*), intent(in),  optional :: newName

    noColumns = noColumns + 1

    columnData(noColumns)%name = oldName

    if(present(newName)) then
        columnData(noColumns)%newName = newName
    else
        columnData(noColumns)%newName = oldName

    end if
    columnData(noColumns)%units = units
    columnData(noColumns)%comment = comment
    columnData(noColumns)%type = type

!if(debug) write(*, *) noColumns, ") createnewcolumns1 Creating column ", oldName, " type=", type

end subroutine createNewColumns1

!****************************************************************
! 
!
!***************************************************************
subroutine createNewColumns2( oldName, units, comment, type, width, noColumns, columnData, newName)
    character(*), intent(in) :: oldName, units, comment
    integer, intent(in) :: type
    integer, intent(in) :: width
    integer(int16), intent(inout) :: noColumns
    type(newColumn), intent(inout) :: columnData(:)
    character(*), intent(in), optional :: newName

    noColumns = noColumns + 1

    columnData(noColumns)%name = oldName

    if(present(newName)) then
        columnData(noColumns)%newName = newName
    else
        columnData(noColumns)%newName = oldName

    end if

    columnData(noColumns)%units = units
    columnData(noColumns)%comment = comment
    columnData(noColumns)%type = type

    columndata(noColumns)%width = width  


!if(debug) write(*, *) noColumns, ") createnewcolumns2 Creating column ", oldName, " type=", type



end subroutine createNewColumns2


!*****************************************************************************
!
! Find out the fraction of sources in each filter
!
!****************************************************************************
real(single) function getFilterPercentage( tableH, filter )
    type(tableT), intent(in) :: tableH
    character(*), intent(in) :: filter
    type(columnT) :: columnH    
    integer :: i, n

    columnh = column( tableH, "FILTER", read)



    n = 0
    do i = 0, numberOfRows( tableH ) - 1
       if (trim( stringCell( columnh, i)) .eq. filter) n = n + 1

    end do
    call release(columnH)

    getFilterPercentage = real(n * 100) / real(numberOfRows( tableh) )

!write(*, *) "getfilterpercentage, percentage=",  getFilterPercentage



end function getFilterPercentage


!*****************************************************************************
! Find out the fraction of sources in each filter
!
!****************************************************************************
integer(int32) function getFilterNumber( tableH, filter )
    type(tableT), intent(in) :: tableH
    character(*), intent(in) :: filter
    type(columnT) :: columnH    
    integer :: i, n

    columnh = column( tableH, "FILTER", read)



    n = 0
    do i = 0, numberOfRows( tableH ) - 1
       if (trim( stringCell( columnh, i)) .eq. filter) n = n + 1

    end do
    call release(columnH)

    getFilterNumber = n





end function getFilterNumber




!*****************************************************************************
!
! Find out the fraction of sources in each filter
!
!****************************************************************************
real(single) function getFilterPercentage1( tableH, filter )
    type(tableT), intent(in) :: tableH
    character(*), intent(in) :: filter
    type(columnT) :: columnH    
    integer :: i, n
    character(20) :: name
    real(single), pointer :: data(:) => null()
    name = trim(filter) // "_SIGNIFICANCE"

    if ( .not. hasColumn(tableH, trim(name))) then
        getFilterPercentage1 = 0.0
        return
    end if


    columnh = column( tableH, trim(name), read)
    data => real32Data( columnH )


    n = 0  ! Count how many non-values there are
    do i = 0, numberOfRows( tableH ) - 1

        if ( .not. isNullValue(data(i))) n = n + 1     

    end do
    call release(columnH)

    getFilterPercentage1 = real(n * 100) / real(numberOfRows( tableh) )

!write(*, *) "getfilterpercentage1, percentage=",  getFilterPercentage1


end function getFilterPercentage1


!*****************************************************************************
! Find out the fraction of sources in each filter
!
!****************************************************************************
integer function getFilterNumber1( tableH, filter )
    type(tableT), intent(in) :: tableH
    character(*), intent(in) :: filter
    type(columnT) :: columnH    
    integer :: i, n
    character(20) :: name
    real(single), pointer :: data(:) => null()
    name = trim(filter) // "_SIGNIFICANCE"

    if ( .not. hasColumn(tableH, trim(name))) then
        getFilterNumber1 = 0.0
        return
    end if
    
    columnh = column( tableH, trim(name), read)
    data => real32Data( columnH )

    n = 0  ! Count how many non-values there are
    do i = 0, numberOfRows( tableH ) - 1

        if ( .not. isNullValue(data(i))) n = n + 1     

    end do
    !write(*,*) "L3328 name=", trim(name), " n=", n
    call release(columnH)

    getFilterNumber1 = n


end function getFilterNumber1

!*****************************************************************************
!
! This is subroutine is called when the master-catalogue is first created 
!
!***************************************************************************
subroutine SetUniqueSrcNumber( fileName, tableName )
    character(*) :: fileName, tableName
    type(datasetT) :: dataSetH
    type(tableT) :: tableH
    type(columnT) :: columnH, raColumnH, decColumnh
    integer(int32), pointer :: integer32ArrayData(:) => null()
    
    integer :: i
    !write(*,*) "L3362 fileName=", trim(fileName)
    dataseth = dataSet( trim(fileName), MODIFY )
    tableh = table( dataseth, trim(tableName), MODIFY )

    columnH  = Column( tableH, "SRCID" )
    

    integer32ArrayData => int32Data(columnH)
    write(*,*) "L3368 setting numbers for SRCID,  number of rows =", numberOfRows( tableH)
    do i = 1, numberOfRows( tableH)
        
        integer32ArrayData(i - 1) = i

    end do

    call release(columnh)

    columnH  = Column( tableH, "N_OBSID" )
    

    integer32ArrayData => int32Data(columnH)
    write(*,*) "L3381 setting numbers for N_OBSID,  number of rows =", numberOfRows( tableH)
    do i = 1, numberOfRows( tableH)

            
        integer32ArrayData(i - 1) = 1


    end do

    call release(dataSetH)

end subroutine SetUniqueSrcNumber


!*****************************************************************************
! This subroutine checks each source to see if it has one or more 
! counterparts
!
!
!
!***************************************************************************
subroutine UpdateUniqueSrcNumber(prefix, fileName, tableName )

    integer :: prefix
    character(*) :: fileName, tableName
    type(datasetT) :: dataSetH
    type(tableT) :: tableH
    type(columnT) :: columnH, columnh1, columnh2, raColumnh, decColumnH, obsNoColumnH, newColumn
    integer(int32), pointer :: integer32ArrayData(:) => null()
    integer(int32), pointer :: integer32ArrayData1(:) => null()
    integer(int32), pointer :: obsNoData(:) => null() ! For the SUMMARY_NO column
    real(single),pointer :: realArrayData(:) 
    real(double),pointer :: doubleArrayData(:) 

    real(double), pointer :: raArrayData(:) => null()
    real(double), pointer :: decArrayData(:) => null()
    real(single), pointer :: poserrArrayData(:) => null()


    logical, pointer :: matched(:) => null()
    integer, pointer :: obsNo(:) => null()
    real(single) :: mean, error
    
    real(double) :: tolerance, ra1, dec1, ratol, raMean, decMean    
  
    integer(int32) :: i, j, k, l, m, n, istart 
    integer(int32) :: sourceNo, sourceNo1
    integer(int32) :: noUniqueSources, maxNoIdenticalSources, noMerged
    integer(int32), pointer, dimension(:) :: noCounterParts => null(), lowerIndex => null(), upperIndex => null()   
    integer(int32), pointer, dimension(:) :: index => null()   
    real(double), pointer, dimension(:) :: ra => null(), dec=> null()
    real(single), pointer :: poserr(:) => null()
    real(double) :: lII, BII, minSeparation, maxSeparation, distance

    type(TableT) :: newTableH
  
  
    real(single), pointer :: realData(:, :) => null()
   
    type(tableData) :: tableDataH, newTableDataH


    radiansToArcSeconds = radiansToDegrees * 3600.0_double


    !**********************************************************
    ! Open the catalogue file
    !**********************************************************
    datasetH = dataSet( trim(fileName), MODIFY )
    tableH = table( dataSetH, "SRCLIST", MODIFY )


    !***********************************************************************
    !
    !***********************************************************************
    write(*,*) "L3981 calling loadTableColumnData  nocolumns4=", nocolumns4 
    call LoadTableColumnData( datasetH, tableH, tableDataH, nocolumns4, columndata4)
    write(*,*) "L3983 nocolumns4=", nocolumns4    


!if(debug) write(*, *) "Line 3215"

    !****************************************************************************
    ! The following is for the observation number
    !
    !****************************************************************************
    obsNoData => tableDataH%columnData(1)%integer32ArrayData  



!if(debug) write(*, *) "Line 3225"



    !*************************************************************************
    ! The following will store the observation no of each identical source
    !*************************************************************************
    

    obsNo => allocateInt32(1, maxval(obsNoData), "obsNo")

    !****************************************************************************
    ! The following refers to the SRCNUM column
    !****************************************************************************
    integer32ArrayData => tableDataH%columnData(3)%integer32ArrayData  
          
    integer32ArrayData = 0  ! Reset to zero


!if( debug) write(*, *) "Line 3244"


    !****************************************************************************
    ! The following is the N_OBSID column
    !****************************************************************************
    integer32ArrayData1 => tableDataH%columnData(15)%integer32ArrayData  
  

!if( debug) write(*, *) "Line 3254"




    integer32ArrayData1 = 1 ! Every source has initially one unique source (itself)


    !****************************************************************************
    ! The following is for the RA
    !****************************************************************************
    raArrayData => tableDataH%columnData(10)%real64ArrayData  



!if (debug) write(*, *) "Line 3264"


    !****************************************************************************
    ! The following is for the Dec
    !****************************************************************************
    decArrayData => tableDataH%columnData(11)%real64ArrayData  


    !****************************************************************************
    ! The following is for the position error
    !****************************************************************************
    poserrArrayData => tableDataH%columnData(12)%real32ArrayData  


    n = numberOfRows( tableH ) - 1  ! NB indexing starts at 0



!if (debug) write(*, *) "Line 3283"




    !*****************************************************************************************
    ! Do the matching
    !****************************************************************************************


    allocate(matched(0 : n))
    matched = .false.  
 
   
    sourceNo = 0     ! The unique source number
    tolerance = 0.5  ! The tolerance (arcsecs) for source matching

 
    noCounterParts => allocateInt32(1, n + 1, "noCounterParts")
    noCounterParts = 1

    lowerIndex => allocateInt32(1, n + 1, "lowerIndex")
    lowerIndex = n
    upperIndex => allocateInt32(1, n + 1, "upperIndex")
    upperIndex = 0
     

!if(debug) write(*, *) "Line 3289"


    !**************************************************************************
    ! Loop through all the sources in the SRCLIST table
    !*************************************************************************
    do i = 0, n ! Number of sources - 1
    
        if( matched(i)) cycle              ! Source been matched        
        sourceNo = sourceNo + 1            ! The next source number 
        integer32ArrayData(i) = sourceNo   ! Record the unique source number of this sources        
            
        matched(i) = .true.                ! Record that we have checked this source
                                               ! Record the lower and upper indices of the
                                               ! identical sources for averaging later on 
        lowerIndex(sourceNo) = i           ! Lower index in list of this unique source
        upperIndex(sourceNo) = i           ! Upper index in list of this unique source
        obsNo = 0                          ! Set the list of observation nos to 0
        obsNo(1) = obsNoData(i)            ! Record the observation number- cannot merge sources from the same observation
  

        ra1 = raArrayData(i) * degreesToRadians 
        dec1 = decArrayData(i) * degreesToRadians

        raTol = (5.0 / cos(dec1) ) / 3600.0

        do j = i + 1, n ! See if any possible counterparts to source i
            if( matched(j)) cycle          ! Both have been assigned a unique source number



!****************************************************************
! NB BLANKED OUT FOR TEST
!***************************************************************


            if( any( Obsno .eq. obsNoData(j)) ) cycle        ! Don't check sources that originated from the same observation           

            
            if( abs(raArrayData(i) - raArrayData(j) ) > raTol) exit 


            !**************************************************************************************
            ! Compute distance between objects in arc secs
            !************************************************************************************** 
            if ( ArcLength( ra1, dec1,                          &
                            raArrayData(j) * degreesToRadians,  &
                            decArrayData(j) * degreesToRadians) &
                        .le. tolerance) then
     
            
                !*****************************************************************************
                ! Possibly the same unique source 
                !*****************************************************************************  
                if(matched(i)) then
                    l = integer32arraydata(i) ! The unique source number of source i
                    matched(j) = .true.
                elseif(matched(j)) then
                    l = integer32arraydata(j) ! The unique source number of source j
                    matched(i) = .true.
                endif                 
                             
                integer32ArrayData(j) = l ! The unique source number                   

  
                noCounterParts(l) = noCounterParts(l) + 1                   ! Record how many counterparts this unique source has
                  
                lowerIndex(l) = min( j, lowerIndex(l) )   ! Record the lower index of a source with unique number l
                upperIndex(l) = max( j, upperIndex(l) )   ! Record the upper index of a source with unique number l
                obsNo(noCounterParts(l)) = obsNoData(i)   ! Record the observation no of this possible counterpart
                        
            end if

        

        end do

 
    end do ! End of looping through all the sources





    !***********************************************************
    ! Set the number of counterparts column 
    !**********************************************************
    k = 0
    do i = 1, sourceNo

        do j = lowerIndex(i), upperIndex(i) 

            if( integer32ArrayData(j) .eq. i) then

                integer32ArrayData1(j) = noCounterParts(i)
            end if


        end do

    end do
 



    if(sourceNo .ne. maxval(integer32arraydata)) then
       call fatal("bug at line 2543")
    end if


    !if(debug) write(*, *) "Line 3399"




    !***************************************************************************
    ! The number of counter-parts should equal the number of input sources
    !***************************************************************************
    if(sum(noCounterParts(1 : sourceno) ) .ne. (n + 1)) then
       
       !write(*, *) "line 2551 no rows=", (n + 1), " sum=", sum(noCounterparts)
       
       call fatal("bug at line 2553")
       
    end if

    !write(*, *) "Line 2663"



    deallocate(matched) 
    nullify(matched)


    !if(debug) write(*, *) "Line 3423"



    noUniqueSources = sourceNo                     ! The total number of unique sources

    maxNoIdenticalSources = maxval(noCounterParts) ! The maximum no of identical sources for any particular unique source number

    !write(*, *) "Line 2753 maxnocounterparts=", maxNoIdenticalsources

    !*******************************************************************
    ! The number of unique sources with more than one "identical" source
    !******************************************************************* 

    noMerged = count(noCounterParts(1 : noUniqueSources) > 1)           ! The number of 

!    write(*, *) "No of unique sources with greater than one counterpart=", noMerged



    !********************************************************************************
    ! Create a new table to store the multiple detections of the same source
    !********************************************************************************
!    if( hasBlock(dataSetH,  "AV_MULTIPLE_DETECTIONS") ) then ! Shouldn't exist - may do if just testing
!        call deleteBlock( dataSetH,  "AV_MULTIPLE_DETECTIONS" )
!    end if


!if(debug)     write(*, *) "Line 3478"




!    newTableH = addTable( dataSetH, "AV_MULTIPLE_DETECTIONS", noMerged)
!    call copyAllAttributes( attributable(tableh), attributable( newtableH ) )


!    do j = 0, numberOfColumns( tableH ) - 1
!        newColumn = Column( tableH, j )
!        call copyColumn( newTableH, newColumn, name(newColumn))      
!    end do



    !***********************************************************************
    !
    !***********************************************************************

    !***********************************************************************
    ! Modify the unique source number if the prefix parameter was positive
    !************************************************************************
    if (prefix>=0) then
       write(*,*) "L4242 adding source number prefix"
       sourceNo1=prefix
       sourceNo1=sourceNo1*10000000

       do i = 0, n 
          integer32ArrayData(i) = integer32ArrayData(i) + sourceNo1
       enddo
    endif

    !if(debug) write(*, *) "L3732  !!!!!! "
    
    call release( dataSetH )
    
    ! NB: why return?
    !^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^
    return 
    !^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^

    write(*,*) "L4288 calling loadTableColumnData  nocolumns4=", nocolumns4 

    call LoadTableColumnData( datasetH, newTableH, newTableDataH, nocolumns4, columndata4)
    
    write(*,*) "L4292 nocolumns4=", nocolumns4 

!if (debug) write(*, *) "Line 3506"


    allocate(realData(1 : maxnoidenticalSources, 1 : 2))
  

    ra => allocateDouble(1, maxNoIdenticalSources, "RA")
    dec => allocateDouble(1, maxNoIdenticalSources, "DEC")
    poserr => allocateSingle(1, maxNoIdenticalSources, "poserr")

    index => allocateInt32(1, maxNoIdenticalSources, "index")


    !************************************************************************************
    ! Merge the identical sources
    !***********************************************************************************

    noMerged = 0
    minSeparation = 1e6
    maxSeparation = -minSeparation


    write(errstr,'(a, a, a, a)') "Merging multiply-detected sources"
    call message(AppMsg, Verbose)


!write(*, *) i, ") Line 2824"

    do i = 1, noUniqueSources ! Loop through each one and find the counterparts
                                  
        if(noCounterParts(i) == 1) cycle  ! Only one source with this unique number      
!        write(errstr, *) "Merging unique source ", i, " no of counterparts=", noCounterParts(i), &
!                    " lowerindex=", lowerindex(i), " upper=", upperindex(i)         
     
!        call message(AppMsg, Verbose)


        !********************************************************************
        ! Store the multiple sources for each unique source for averaging
        !********************************************************************

        l = 0


        do j = lowerIndex(i), upperIndex(i) ! Need to loop through all the sources
           
         
            if(integer32arraydata(j) == i) then ! One of the identical sources

             
                l = l + 1
              
                 
                index(l) = j ! Store the row number  
             
                           
            end if 
        end do




        !*********************************************************************
        ! Average the Ras and decs
        !
        !********************************************************************
        do m = 1, l

            ra(m) = raArrayData( index(m) )
            dec(m) = decArrayData( index(m) )
            poserr(m) = poserrArrayData( index(m))


if ( ArcLength( ra(1)*degreesToradians, dec(1) * degreesToRadians,     &
                    ra(m)*degreesToradians, dec(m) * degreesToRadians)     &                      
                        .gt. 20.0) then
     


write(*, *) "l=", l, " size(index)=",  size(index), " size(ra)=", size(ra)
do j = 1, l




    write(*, *) m, ") ra=", ra(j), " dec=", dec(j), " index=", index(j)

end do
write(*, *) "lower index=", lowerindex(i), " upper=", upperIndex(i)
    call fatal("BUG at line 2731")

end if



            if(m > 1) then
                distance = ArcLength( ra(m - 1)*degreesToradians,   &
                                    dec(m - 1) * degreesToRadians,  &
                                    ra(m)*degreesToradians, dec(m) * degreesToRadians)      
                minseparation = min(minSeparation, distance)

                maxseparation = max(maxSeparation, distance)
                if(distance .lt. 0.01) then
                    write(errstr, '(a, i7, a, i7, a, f6.2, a)') "Distance between sources ", index( m - 1), &
                              " and ", index(m), "=", distance, " arcsecs" 

                    call warning("Possible bug at line 2903", trim( errstr) ) 



                end if

                     
            end if
        end do      

!         write(*, *) i, ") Line 2899"


        !***********************************************************************
        ! NB nomerged is incremented in updatetablecolumns
        !***********************************************************************
        call updateTableColumns( nomerged, index(1), newTableDataH, tableDataH)
        nomerged = nomerged - 1


!write(*, *) i, ") Line 3601"


        !**************************************************************************
        ! Average the ras and decs
        !**************************************************************************
        call averageRADECs(l, ra, dec, posErr, raMean, decMean, error)
    


        newtableDataH%columnData(6)%real64ArrayData(nomerged) = raMean 

       
        newtableDataH%columnData(7)%real64ArrayData( nomerged ) = decMean  

        newtableDataH%columnData(8)%real32ArrayData( nomerged ) = error  

        !*******************************************************************
        ! Compute the galactic longitude and latitude
        !*******************************************************************

        call sla_EQGAL( raMean * degreesToRadians, decMean * degreesToRadians, LII, BII)
        newtableDataH%columnData(9)%real64ArrayData( nomerged ) = LII * radiansToDegrees
        newtableDataH%columnData(10)%real64ArrayData( nomerged ) = BII * radiansToDegrees
 


!write(*, *) i, ") Line 2959"


        !**********************************************************************
        ! Average the significances
        !*********************************************************************
        istart = 11        
        do k = 1, noFilters
        
            do m = 1, l

                realData(m, 1) = tableDataH%columnData(istart)%real32ArrayData( index(m) )  
             
            end do  

!write(*, *) i, ") Line 2974"

            call averageData(l, realData(:, 1), mean=mean, sigma=error)  
            newtableDataH%columnData(istart)%real32ArrayData( nomerged ) = mean  

!write(*, *) i, ") Line 2979"

            istart = istart + 1
        end do 



!write(*, *) i, ") Line 2986, istart=", istart, " size(realdata, 2)=", size(realdata, 2)


        !**********************************************************************
        ! Average the count rates
        !*********************************************************************
        
        call AverageTableData(l, nomerged, istart, index, realData, &
                              tableDataH, newtabledataH)







        if ( isNullValue(newtableDataH%columnData(istart - 2)%real32ArrayData( nomerged )  ) .and. &
            .not. isNullValue(newtableDataH%columnData(istart - 1)%real32ArrayData( nomerged ) ) ) then
    
            call fatal("BUG AT LINE 3001 X IS NAN BUT NOT DX!")
        end if





!write(*, *) i, ") Line 2998"



        !**********************************************************************
        ! Average the fluxes
        !*********************************************************************
!        call AverageTableData(l, nomerged, istart, index, realData, &
!                              tableDataH, newtabledataH)



       call ComputeAveragedABFluxes(l, nomerged, istart, tabledataH, newtableDataH)
    







!write(*, *) i, ") Line 3012"


        !**********************************************************************
        ! Average the AB mags
        !*********************************************************************
!        call AverageTableData(l, nomerged, istart, index, realData, &
!                              tableDataH, newtabledataH)


        call ComputeAveragedABMags(l, nomerged, istart, tabledataH, newtableDataH)
    




!write(*, *) i, ") Line 3022"


        !**************************************************************************
        ! Set row no nomerged in the new table
        !**************************************************************************
        nomerged = nomerged + 1  ! Move to next source               
            
    end do

    write(*, *) "Line 3032 MaxSepartion=", maxSeparation, " (arcsecs)"












    deallocate(ra, dec, poserr)
    deallocate( index, noCounterParts, lowerIndex, upperIndex, realData)

    !******************************************************************
    ! Store the repeat observations
    !*****************************************************************

    call release( dataSetH )




    write(errstr,'(a, f6.2, a, f6.2, a)') "Min separation between 2 multiply detected sources=", &
                        minSeparation, " max=", maxSeparation, " arcsecs"
    call message(AppMsg, Verbose)






end subroutine UpdateUniqueSrcNumber



!***********************************************
!
!***********************************************


logical function isnanvalue1( value )
    real(single), intent(in) :: value

   

    if ( IEEE_IS_NAN(value) ) then
!    if (((value > 0.0) .EQV. (value <= 0.0)) &
!         .or. value/=value ) then 
       isnanvalue1 = .true.
       return
    end if
    if ( .not. IEEE_IS_FINITE(value) ) then
        isnanvalue1 = .true.
        return
    end if
     




    isnanvalue1 =  .false.



end function isnanvalue1



!***********************************************
!
!
!
!***********************************************
logical function isnanvalue2( value )
    real(double), intent(in) :: value

   

    if ( IEEE_IS_NAN(value) ) then
    !if (((value > 0.0_double) .EQV. (value <= 0.0_double)) &
    !     .or. value/=value ) then 
       isnanvalue2 = .true.
       return
    end if
    if ( .not. IEEE_IS_FINITE(value) ) then
        isnanvalue2 = .true.
        return
    end if
     

    isnanvalue2 =  .false.



end function isnanvalue2



!******************************************************************************
!
!*****************************************************************************
subroutine averageRealData( n, data, error, mean, sigma)  
    integer(int32), intent(in) :: n
    real(single) :: data(:)
    real(single), optional :: error(:)
    real(single), intent(out) :: mean, sigma
    real(double) :: mean1, sigma1, error1(1 : n)


    if( present(error)) then

        error1(1 : n) = dble(error(1 : n))
        call averageDoubleData( n, dble(data), error=error1, mean=mean1, sigma=sigma1)  
    else

        call averageDoubleData( n, dble(data), mean=mean1, sigma=sigma1)  


    end if 




    mean = real(mean1)
    sigma = real(sigma1)

end subroutine averageRealData


!******************************************************************************
!
!*****************************************************************************
subroutine averageDoubleData( n, data, error, mean, sigma)  
    integer(int32), intent(in) :: n
    real(double) :: data(:)
    real(double), optional :: error(:)
    real(double), intent(out) :: mean, sigma
    integer :: i, k
    real(double) sum1, sum2, weight, minError



    k = 0
    do i = 1, n

        if( .not. isnanvalue( data(i)) ) then
            k = k + 1

        end if 

    end do

!write(*, *) "Averagedoubledata line 3210 k=", k

    if(k < 2) then  
        mean = data(1)
        if(present(error)) then
            sigma = error(1)
        end if 
!write(*, *) "line 3217 mean=", mean, " error=", sigma
        return
    end if
    
!write(*, *) "Averagedoubledata line 3220 k=", k

    if( .not. present(error)) then

        k = 0 
        sum1 = 0.0_double
        do i = 1, n
            if(.not. isnanvalue( data(i))) then        
                k = k + 1
                sum1 = sum1 + dble(data(i))
            end if
        end do
        mean = sum1 / dble(k)
        return

    end if


    !***********************************************************
    ! The weighted average
    !***********************************************************
    sum1 = 0.0_double
    sum2 = 0.0_double


    minError = 1e20
    do i = 1, n
        if(.not. isnanValue(error(i)) ) then        
 
  
            minError = min(minError, error(i) )
        end if
    end do 


!write(*, *) "Averagedoubledata line 3251 k=", k


    do i = 1, n
        if(.not. isnanvalue( data(i) ) .and. .not. isnanValue(error(i)) ) then        
        
            weight = 1.0_double / (error(i) / minError) ** 2
            sum1 = sum1 + weight * dble(data(i))
            sum2 = sum2 + weight
        end if
    end do

   
    mean = sum1 / sum2
    sigma = minError * dsqrt( 1.0_double /  sum1)

! write(*, *) "Averagedoubledata line 3267 mean=", mean, " sigma=", sigma


end subroutine averageDoubleData


!***************************************************************************************
!
!
!**************************************************************************************

subroutine addraanddecstringcolumns( masterCatalogue)
    character(*), intent(in) :: masterCatalogue
    
    type(TableT) :: srcTable
    type(DataSetT) :: dataSetH 
    real(double), pointer, dimension(:) :: ra=> null(), dec => null()
    
    integer :: i, nRows


    dataSetH = dataset(masterCatalogue, MODIFY)
    srcTable = Table(dataSetH, "SRCLIST")

    !***********************************************************************************
    ! Add the RA and DEC strings
    !**********************************************************************************
    nRows = numberOfRows( srcTable )

!    write(*, *) "Line 4004 adding ra and dec string columns - nrows=", nRows


    !**********************************************************
    ! 
    !
    !*********************************************************

    ra => allocateDouble(1, nRows, "ra")
    call ReadReal64Column(srcTable, "RA_CORR", nrows, 0, ra)



    dec => allocateDouble(1, nRows, "ra")    
    call ReadReal64Column(srcTable, "DEC_CORR", nrows, 0, dec)
 


    call addradecstringcolumns( srcTable, nrows, ra, dec, position=5)


    call release(dataSetH)

    deallocate(ra, dec)
    nullify(ra, dec)


end subroutine addraanddecstringcolumns


!***************************************************************************************
!
!
!**************************************************************************************

subroutine addraanddecstringcolumns1( srcTable) 
    
    type(TableT), intent(inout) :: srcTable
    real(double), pointer, dimension(:) :: ra=> null(), dec => null()
    integer :: index, nRows




    !***********************************************************************************
    ! Add the RA and DEC strings
    !**********************************************************************************
    nRows = numberOfRows( srcTable )

!if(debug)    write(*, *) "Line 4051 adding ra and dec string columns - nrows=", nRows

    !**********************************************************
    ! 
    !
    !*********************************************************

    ra => allocateDouble(1, nRows, "ra")

    dec => allocateDouble(1, nRows, "ra")    
 

    if( hasColumn(srcTable, "RA_CORR")) then

        call ReadReal64Column(srcTable, "RA_CORR", nrows, 0, ra)

        call ReadReal64Column(srcTable, "DEC_CORR", nrows, 0, dec)
      

    
        index = columnNumber( srcTable, "DEC_CORR")




    elseif( hasColumn(srcTable, "RA")) then


        call ReadReal64Column(srcTable, "RA", nrows, 0, ra)

        call ReadReal64Column(srcTable, "DEC", nrows, 0, dec)

        index = columnNumber( srcTable, "DEC")



    else

        call ReadReal64Column(srcTable, "AVE_RA", nrows, 0, ra)

        call ReadReal64Column(srcTable, "AVE_DEC", nrows, 0, dec)

        index = columnNumber( srcTable, "AVE_DEC")



    end if 

    index = index + 1
    call addradecstringcolumns( srcTable, nrows, ra, dec, position=index)

  
    deallocate(ra, dec)
    nullify(ra, dec)


end subroutine addraanddecstringcolumns1




!*********************************************************************
! Adds the Merged table to the file
!
!********************************************************************
subroutine finish( oldMasterCatalogue, newMasterCatalogue )
    character(*), intent(in) :: oldMasterCatalogue, newMasterCatalogue


    type(DataSetT) :: datasetH
    type(TableT) :: tableH

    integer, parameter :: noColumns = 200
    character(100), dimension(noColumns) ::  oldColumnName, newColumnName
   
    integer :: i, n

    integer :: prefix

    if(debug)     write(*, *) "L4392 finalising the catalogue "

    call copydataSet( trim ( oldMasterCatalogue), trim ( newmasterCatalogue ) )
 
    !goto 1

    !if(debug) write(*, *) "L4399 calling updateUniqueSrcNumber"

    prefix= intParameter("prefix")
    !write(*,*) "L4881 prefix=", prefix

    call UpdateUniqueSrcNumber( prefix, trim(newmastercatalogue), "SRCLIST" )

    if(debug)  write(*, *) "L4400 finished updating the unique source numbers "
    write(*,*) "L4401 newMasterCatalogue=", trim(newMasterCatalogue)
    dataSetH = dataSet( trim(newMastercatalogue), MODIFY)

    !*********************************************************
    ! Add epoch keyword
    !********************************************************/
    write(*,*) "L4407 adding the epoch keyword"

    call setAttribute( dataSetH, "EPOCH", "J2000", "Epoch of Equinox") 

    tableH = table( dataSetH, "SRCLIST", MODIFY)

    write(*,*) "L4413 adding the RA and DEC columns"
    call addraanddecstringcolumns1( tableH ) 

    write(*,*) "L4416 releasing the table"
    call release( tableh )


!    tableH = table( dataSetH, "AV_MULTIPLE_DETECTIONS" , MODIFY)
!    call addraanddecstringcolumns1( tableH ) 
!    call release( tableh )
    write(*,*) "L4423 releasing data set"

    call release(dataSetH)  


    !***********************************************************
    ! Rename some columns
    !**********************************************************
  
    write(*,*) "L4432 renaming some columns "
    !    oldColumnName(1) = "RA_CORR"
    !    newColumnName(1) = "RA"
    !    oldColumnName(2) = "DEC_CORR"
    !    newColumnName(2) = "DEC"
    
    

    !**********************************************************
    ! 
    !*********************************************************
   
    n = 2
    n = 1
    write(*,*) "L4446 noFilters=", noFilters
    do i = 1, noFilters
       !write(*,*) "L4440 i=", i, " omFilter(i)=", trim(omFilter(i))
 !      !n = n + 1
 !      oldColumnName(n)  = trim(omFilter(i)) // "_INS_MAG"
 !      !newColumnName(n)  = trim(omFilter(i)) // "_VEGA_MAG"
 !      newColumnName(n)  = trim(omFilter(i)) // "_INS_MAG"
 !      write(*,*) "L4454 n=", n, " newColumnName=", trim(newColumnName(n))
 !      n = n + 1
 !
!
!       oldColumnName(n)  = trim(omFilter(i)) // "_INS_MAG_ERR"
!       !newColumnName(n)  = trim(omFilter(i)) // "_VEGA_MAG_ERR"
!       newColumnName(n)  = trim(omFilter(i)) // "_INS_MAG_ERR"
!       write(*,*) "L4460 n=", n, " newColumnName=", trim(newColumnName(n))
!       n = n + 1
  
       oldColumnName(n)  = trim(omFilter(i)) // "_MAJ"
       newColumnName(n)  = trim(omFilter(i)) // "_MAJOR_AXIS"
       !write(*,*) "L4465 n=", n, " newColumnName=", trim(newColumnName(n))
       n = n + 1
       oldColumnName(n)  = trim(omFilter(i)) // "_MIN"
       newColumnName(n)  = trim(omFilter(i)) // "_MINOR_AXIS"
       !write(*,*) "L4469 n=", n, " newColumnName=", trim(newColumnName(n))
       n = n + 1
       oldColumnName(n)  = trim(omFilter(i)) // "_POS_ANGLE"
       newColumnName(n)  = trim(omFilter(i)) // "_POSANG"
       !write(*,*) "L4473 n=", n, " newColumnName=", trim(newColumnName(n))
       
       n = n + 1
       oldColumnName(n)  = "QFLAG_" // trim(omFilter(i))
       newColumnName(n)  = trim(omFilter(i)) // "_QUALITY_FLAG"
       !write(*,*) "L4478 n=", n, " newColumnName=", trim(newColumnName(n))

       n = n + 1
       oldColumnName(n)  = "CFLAG_" // trim(omFilter(i))
       newColumnName(n)  = trim(omFilter(i)) // "_CONFUSION_FLAG"
       !write(*,*) "L4483 n=", n, " newColumnName=", trim(newColumnName(n))
       n = n + 1

       oldColumnName(n)  = "EFLAG_" // trim(omFilter(i))
       newColumnName(n)  = trim(omFilter(i)) // "_EXTENDED_FLAG"
       !write(*,*) "L4488 n=", n, " newColumnName=", trim(newColumnName(n))  
    end do
   
    !write(*,*) "L4491 calling the routine renameColumns"
    call renameColumns(trim(newMasterCatalogue), "SRCLIST", n, oldColumnName, newColumnName)
    !write(*,*) "L4493 finished renaming columns"


    !call fatal("stopped at line 4427 after re-naming columns")


    oldColumnName(1) = "REVOLUT"
    newColumnName(1) = "REVOLUTION"

    oldColumnName(2) = "OBS_ID"
    newColumnName(2) = "OBSID"


    oldColumnName(3) = "OBJECT"
    newColumnName(3) = "TARGET"

    oldColumnName(4) = "DEOFFSET"
    newColumnName(4) = "DECOFFSET"

    oldColumnName(5) = "RAOFFERR"
    newColumnName(5) = "RAOFFSET_ERR"

    oldColumnName(6) = "DEOFFERR"
    newColumnName(6) = "DECOFFSET_ERR"

    oldColumnName(7) = "RMSRESID"
    newColumnName(7) = "OFFSET_RMS"


    write(*,*) "L4511 renaming the columns in the SUMMARY table"
    call renameColumns(trim(newMasterCatalogue), "SUMMARY", 7, oldColumnName, newColumnName)

    
!call fatal("stopped at line 4613 after re-naming columns")

    !*********************************************************************
    ! Change some columns in AVERAGED_MULTIPLE_DETECTIONS table
    !********************************************************************
!    call modifyAveragedTable( newMasterCatalogue )


    !*************************************************************************
    ! Delete the first table (empty)
    !************************************************************************
    write(*,*) "L4526 deleting the empty Primary block"
    dataSetH = dataSet( trim(newMasterCatalogue), modify)
    if (hasBlock(dataSetH, "Primary")) then
        call deleteBlock( dataSetH, "Primary");
       write(*, *) "Deleted PRIMARY BLOCK"
    end if

    write(*,*) "L4533 releasing the dataset"
    call release( dataSetH )

    dataSetH = dataSet( trim(newMasterCatalogue), modify)
    write(*,*) "-------------------------"
    write(*,*) "L4537 newMasterCatalogue structure:"
    write(*,*) "-------------------------"
    do i = 1, numberofblocks(dataSetH)
        write(*, *) "L4540 i=", i, ") block=", trim(name(block(dataSetH, i - 1)))
    end do

    call release( dataSetH )
    !write(*,*) "L4544 released newMasterCatalogue"

    !call fatal("stopped at line 4499 after re-naming columns")



    !*******************************************************
    ! Convert the Quality flags to a string column
    !*******************************************************/
    write(*,*) "L4553 converting the Quality Flag columns into strings"
    call convertFlagsToStrings()

    !call fatal("stopped at line 4510 after converting flags  columns")


    !1 continue

    !**********************************************************
    ! Add 0.7 arcsecs to position errors !! Don't add this now
    !*********************************************************


    !    call modifyPositionErrors("SRCLIST", "POSERR")

    !call fatal("stopped at line 4521 after modifying positional errors")
    !    call modifyPositionErrors("AV_MULTIPLE_DETECTIONS", "AV_RA_DEC_ERR")

    !write(*,*) "L4571 adding the OBSID column"
    call AddOBSIDColumn

    !call fatal("stopped at line 4531 after adding obsidcolumn")

    !*******************************************************************
    !
    !
    !******************************************************************
    !    call createFinalCatalogue


  end subroutine finish


!*********************************************************
!
!
!*********************************************************
subroutine modifyAveragedTable( newMasterCatalogue )

    character(*), intent(in) :: newMasterCatalogue

    integer :: i, n = 0
    character(100), dimension(200) ::  oldColumnName, newColumnName
    character(100) :: name
    type(DataSetT) :: dataSetH
    type(TableT)   :: tableH



    !*********************************************************
    ! Remove the following columns from the table
    !*********************************************************


    n = 3
    oldColumnName(1) = "SUMMARY_NO"
   
    oldColumnName(2) = "SRCID"
    oldColumnName(3) = "DIST_NN"



!    call removeColumns(trim(newMasterCatalogue), "AV_MULTIPLE_DETECTIONS", &
!              n, oldColumnName)




!    dataSetH = dataSet( trim(newMasterCatalogue), modify)
!    tableH = table( dataSetH, "AV_MULTIPLE_DETECTIONS" , MODIFY)
!    call addraanddecstringcolumns1( tableH ) 
!    call release( dataSetH )




    !*************************************************
    ! Rename some columns
    !*************************************************
    
    oldColumnName(1) = "RA_CORR"   
    newColumnName(1) = "AV_RA"
    oldColumnName(2) = "DEC_CORR"
    newColumnName(2) = "AV_DEC"
    oldColumnName(3) = "POSERR"
    newColumnName(3) = "AV_RA_DEC_ERR"
    oldColumnName(4) = "LII"
    newColumnName(4) = "AV_LII"
    oldColumnName(5) = "BII"
    newColumnName(5) = "AV_BII"


    oldColumnName(6) = "N_OBSID"
    newColumnName(6) = "AV_N_OBSID"


    oldColumnName(7) = "RA_HMS"
    newColumnName(7) = "AV_RA_HMS"


    oldColumnName(8) = "DEC_DMS"
    newColumnName(8) = "AV_DEC_DMS"







    !*********************************************************************************************
    ! Significances
    !********************************************************************************************
    n = 8
    do i = 1, noFilters
        n = n + 1
        oldColumnName(n) = trim(omFilter(i)) // "_SIGNIF"
        newColumnName(n) = "AV_" // trim(omFilter(i)) // "_SIGNIF"
    
    end do


    !*********************************************************************************************
    ! Corrected count rates
    !********************************************************************************************

    do i = 1, noFilters
        n = n + 1
        oldColumnName(n) = trim(omFilter(i)) // "_CORR_RATE"
        newColumnName(n) = "AV_" // trim(omFilter(i)) // "_CORR_RATE"
        n = n + 1
        oldColumnName(n) = trim(omFilter(i)) // "_CORR_RATE_ERR"
        newColumnName(n) = "AV_" // trim(omFilter(i)) // "_CORR_RATE_ERR"
  
    end do







    !*********************************************************************************************
    ! AB fluxes
    !********************************************************************************************

    do i = 1, noFilters
        n = n + 1

        oldColumnName(n) = trim(omFilter(i)) // "_AB_FLUX"
        newColumnName(n) = "AV_" // trim(omFilter(i)) // "_AB_FLUX"

        n = n + 1
     
        oldColumnName(n) = trim(omFilter(i)) // "_AB_FLUX_ERR"
        newColumnName(n) = "AV_" // trim(omFilter(i)) // "_AB_FLUX_ERR"
            
    end do



    !*********************************************************************************************
    ! AB magnitudes
    !********************************************************************************************
    do i = 1, noFilters
        n = n + 1
        oldColumnName(n) = trim(omFilter(i)) // "_AB_MAG"
        newColumnName(n) = "AV_" // trim(omFilter(i)) // "_AB_MAG"

        n = n + 1
        oldColumnName(n) = trim(omFilter(i)) // "_AB_MAG_ERR"
        newColumnName(n) = "AV_" // trim(omFilter(i)) // "_AB_MAG_ERR"
      
      
    end do

    n = n + 1
    oldColumnName(n) = "QFLAG_COMB"
    newColumnName(n) = "QFLAG"


    n = n + 1
    oldColumnName(n) = "CFLAG_MAX"
    newColumnName(n) = "CFLAG"

    n = n + 1
    oldColumnName(n) = "EFLAG_MAX"
    newColumnName(n) = "EFLAG"


!    call renameColumns(trim(newMasterCatalogue), "AV_MULTIPLE_DETECTIONS", &
!              n, oldColumnName, newColumnName)



end subroutine modifyAveragedTable

!*********************************************************
! Checks that the rows in the SRCLIST table are in the
! order of increasing RA
!
!*********************************************************
subroutine checkMergedTable( file )

    character(*), intent(in) :: file
    type(dataSetT) :: dataSetH    
    type(TableT)   :: tableH
    real(double), pointer, dimension(:) :: ra=> null()
    integer :: i, j, nRows

    dataSetH = dataSet( trim(file), READ)
    tableH = table( dataSetH, "SRCLIST", READ)

    nRows = numberOfRows( tableH )

    !if(debug)  write(*, *) "L4855 checkMergedTable  nrows=", nRows
    
    !**********************************************************
    ! 
    !
    !*********************************************************
    ra => allocateDouble(1, nRows, "ra")
    call ReadReal64Column(tableH, "RA_CORR", nrows, 0, ra)
   
    do i = 2, nRows
        if (ra(i) < ra(i - 1)) then
            do j = max(1, i - 10), min(nRows, i+ 10)
                write(*, *) j, " ) ", ra(j)
            end do
            write(*, *) "L4869 no of rows in table=", nRows
            call fatal("RAS not sorted!")
        end if
    end do
    deallocate( ra )
    call release( dataSetH )

end subroutine CheckMergedTable
!************************************************************************************
!
!************************************************************************************
subroutine setUpFilters

    character(100) :: filters

    filters = stringParameter( "filters" )
    !write(*,*) "L4896 setting up filters filters=", trim(filters)
    select case( filters )

    case ("uv/opt", "UV/OPT")
        noFilters = 6
        allocate(omFilter(1 : noFilters))
        omFilter(1) = "UVW2"
        omFilter(2) = "UVM2"
        omFilter(3) = "UVW1"
        omFilter(4) = "U"
        omFilter(5) = "B"
        omFilter(6) = "V"

        uvOpticalCat = .true. ! Record that this is an UV/optical catalogue
        ! (i.e., ingesting the optical sources only if at least one
        ! UV source is present)

    case ("opt/uv", "OPT/UV")
        noFilters = 6
        allocate(omFilter(1 : noFilters))
        omFilter(1) = "UVW2"
        omFilter(2) = "UVM2"
        omFilter(3) = "UVW1"
        omFilter(4) = "U"
        omFilter(5) = "B"
        omFilter(6) = "V"
        ! In this case ingesting all filters, even if the UV filters are not present

    case ("uv", "UV")
        noFilters = 3
        allocate(omFilter(1 : noFilters))   
        omFilter(1) = "UVW2"
        omFilter(2) = "UVM2"
        omFilter(3) = "UVW1"

    case ("opt", "OPT")

        noFilters = 3
        allocate(omFilter(1 : noFilters))   
        omFilter(1) = "U"
        omFilter(2) = "B"
        omFilter(3) = "V"

    end select


end subroutine setUpFilters

!*********************************************************************************
!
! subroutine refineSourceList
!
!*********************************************************************************
subroutine refinesourcelist( tableH )
   
    type(tableT), intent(inout) :: tableH
    character(20) :: columnname    

    integer(int32) :: i, j, nrows
!    integer(int16), pointer :: data(:) => null()

    real(single), pointer :: data(:) => null()

    character(100) :: name 

 

    nRows = numberOfRows( tableH )



!if(debug) then
!write(*, *) "refinesourcelist line 3993"
!write(*, *) "Before refining source list number of rows=", numberOfRows(tableH)  
!end if
   
 iLoop : do i = numberOfRows(tableH) - 1, 0, -1
     
        do j = 1, noFilters
           
 
            name = trim(omfilter(j)) // "_SIGNIFICANCE"



            if ( hascolumn( tableH, trim(name) )) then

                    data => real32Data( column( tableH, name))
 

                      
                    if( .not. isNullValue(data(i)) ) then
                

                        cycle iLoop
                    end if


            end if 
            

        end do

        !***************************************************************
        ! Get rid of this row
        !***************************************************************
        call deleteRows( tableH, i, 1)


    end do iLoop              
!   write(*, *) "After refining source list number of rows=", numberOfRows(tableH)  
       
end subroutine refinesourcelist

!*********************************************************************
!
!********************************************************************
logical function goodAstrometry( file )
    character(*), intent(in) :: file
    type(dataSetT) :: dataSetH
    type(tableT) :: tableH

    dataSetH = dataSet(trim(file), read)

    tableH = table(dataSetH, "SRCLIST")
    if(numberOfRows(tableH) == 0 .or. numberOfRows(tableH) > maxNumberOfRows) then
        goodAstrometry = .false.
    else

        goodAstrometry = booleanAttribute(tableH, "POSCOROK")
    end if
    call release(dataSetH)

end function goodAstrometry
!*************************************************************************
!
!
!*************************************************************************
subroutine sortFile( file )
    character(*), intent(in) :: file

    type(dataSetT) :: dataSetH
    type(tableT) :: tableH
    integer :: nRows, i, k
    real(double), pointer, dimension(:) :: ra=> null()
    integer, pointer :: raSortIndex(:) => null()

    dataSetH = dataSet(trim(file), modify)

    tableH = table(dataSetH, "SRCLIST")

    nRows = numberOfRows( tableH )

!if(debug) then
!write(*, *) "sortfile line 4298 no of rows=", numberOfRows(tableH)
!end if


    nRows = numberOfRows( tableH )



    if(nrows == 0) then
        call release(dataSetH)
         

        stop
    end if


!write(*, *) "sortfile line 4082"


    call refineSourceList( tableH )

!write(*, *) "sortfile line 4087"


    nRows = numberOfRows( tableH )


    if(nrows == 0) then
        nRows = numberOfRows( tableH )
        call release(dataSetH)
        stop
    end if



    raSortIndex => allocateInt32(1, nRows, "raSortindex")

    !**********************************************************
    ! 
    !
    !*********************************************************

    ra => allocateDouble(1, nRows, "ra")



    if(hasColumn(tableH, "RA_CORR") ) then

        call ReadReal64Column(tableH, "RA_CORR", nrows, 0, ra)
    else
        call ReadReal64Column(tableH, "RA", nrows, 0, ra)
    end if

    !*****************************************************
    ! Find the order of increasing  RA- store in RASortIndex
    ! for adding data to source list table in  
    ! increasing RA order (ra is not sorted)
    !****************************************************
    call IndexSort(ra, RASortIndex, nRows)


    call insertrows( tableH, nRows,  nRows )
    call copyRows( tableH, 0, nRows, nRows)

    !*************************************************************
    ! Reorder the table into increasing RA order
    !*************************************************************
    do i = 0, nRows - 1
        k = raSortIndex(i + 1) + nrows - 1       
      

        call copyRows( tableH, k, i, 1)
    end do

    call deleteRows( tableH, nrows, nrows)

    call release(dataSetH)
    deallocate(ra, rasortIndex)


!write(*, *) "Line 2766 checking file ", trim(file)
call checkMergedTable( file )

end subroutine sortFile


!**************************************************************************
!
!*************************************************************************
subroutine addFlagComments( tableSet )
    type(tableT), intent(inout) :: tableSet
    integer(int32) :: i
    

    ! Add the flags to the table 
 

    
    call addcomment(tableset, "QFLAG- bits set (value refers to integer value, colour refers to region colour)")
 

   
    call addcomment(tableSet, "QFLAG- bits set (colour refers to region colour)")

  
    call addcomment(tableSet, "if QFLAG=0 then region colour will be either")
    call addcomment(tableSet, "green (point-source) or cyan (extended source)")

!    do i = 1, getNoQflags()

!        call addcomment(tableSet, trim(getQFlagText(i)) )

!    end do





    call addcomment(tableSet, "CFLAG- bits set (colour refers to region colour")



!    do i = 1, getNoCflags()

!        call addcomment(tableSet, trim(getCFlagText(i)) )

!    end do



    call addcomment(tableset, "EG Bad pixel + read-out streak, bits 0 and 1 set, integer value =3 colour=red")


end subroutine addFlagcomments

!*************************************************************************
!
!*************************************************************************
subroutine renameColumns(fileName, tableName, noColumns, oldColumnName, newColumnName)
    character(*), intent(in) :: fileName, tableName
    integer, intent(in) :: noColumns
    character(*), dimension(:) ::  oldColumnName, newColumnName
    type(dataSetT) :: dataSetH
    type(tableT) :: tableH
    type(columnT) :: columnh
    integer :: i
    !write(*,*) "L5116 fileName=", trim(fileName)
    dataSetH = dataSet(trim(fileName), modify)
    !write(*,*) "L5118 tableName=", trim(tableName)
    tableH = table(dataSetH, trim(tableName) )
    !write(*,*) "L5120 noColumns=", noColumns
    do i = 1, noColumns
       !write(*,*) "L5122 i=", i, " oldColumnName=",trim(oldColumnName(i)) 
        if( hasColumn( tableH, trim(oldColumnName(i)) )) then

            columnH = column( tableH, trim(oldColumnName(i)), MODIFY)
            !write(*, *) "L5126 Renaming colum ", trim(name(columnH)), " to ", trim(newColumnName(i))
            call rename(columnH, trim(newColumnName(i)) )
        
        end if


    end do 

    call release(dataSetH)
end subroutine renameColumns



!*************************************************************************
!
!
!*************************************************************************
subroutine removeColumns(fileName, tableName, noColumns, oldColumnName)
    character(*), intent(in) :: fileName, tableName
    integer, intent(in) :: noColumns
    character(*), dimension(:) ::  oldColumnName
    type(dataSetT) :: dataSetH
    type(tableT) :: tableH
   
    integer :: i

    dataSetH = dataSet(trim(fileName), modify)

    tableH = table(dataSetH, trim(tableName) )

    do i = 1, noColumns
        if( hasColumn( tableH, trim(oldColumnName(i)) )) then

            
            call deleteColumn(tableH, trim(oldColumnName(i)) )
        

        end if


    end do 

    call release(dataSetH)
end subroutine removeColumns













!**************************************************************************
! Averages Right Ascensions and Declinations
!
!**************************************************************************
subroutine averageRADECs(ndata, ra, dec, error, raMean, decMean, sigma)
    integer, intent(in) :: ndata
    real(double), dimension(:) :: ra, dec
    real(single) :: error(:)

    real(double), intent(out) :: raMean, decMean
    real(single), intent(out) :: sigma
    real(double) :: raz, decz, xi, eta
    integer :: i, iflag
    real(double) :: weight, weightSum, raSum, decSum


    weightSum = 0.0_double
    raSum = 0.0_double
    decSum = 0.0_double

    raz = ra(1) * degreesToRadians
    decz = dec(1) * degreesToRadians



!    write(*, *) "Averaging RAs and Decs for ", ndata, " data"


    do i = 1,  nData
!        write(*, *) i, ") ra=", ra(i), " dec=", dec(i), " error=", error(i)
      
        weight = 1.0_double / ( error(i) * error(i) )
            
 
        weightSum = weightSum + weight
        !*************************************************************
        ! Convert RAs and decs to tangent-plane coordinates- average
        ! the sum of these and then convert the average to RA and dec
        !*************************************************************




!         write(*, *) "raz=", raz * radiansToDegrees, " decz=", decz * radiansToDegrees
!            write(*, *) i, ") ra=", ra(i), " dec=", dec(i), " error=", error(i)

        call sla_ds2tp(ra(i) * degreesToRadians, dec(i) * degreesToRadians, raz, decz, xi, eta, iflag)


        !*********************************
        ! Check to see okay
        !********************************
        call SlaErrorCheck( iflag )

        if(iflag .ne. 0) then
            write(*, *) "iflag=", iflag
            write(*, *) "raz=", raz, " decz=", decz
            write(*, *) i, ") ra=", ra(i), " dec=", dec(i), " error=", error(i)


        end if
    

        raSum = raSum + xi * weight     
        decSum = decSum + eta * weight
       
      
    end do
    


    
    ! Compute the average RA and declination
 
    raSum = raSum / weightSum
    decSum = decSum / weightSum
    call sla_dtp2s(raSum, decSum, raz, decz, raMean, decMean)

    ramean = raMean * radiansToDegrees

    decmean = decMean * radiansToDegrees

    sigma = dsqrt(1.0 / weightSum)



!    write(*, *) "Ramean=", ramean, " decmean=", decMean, " error=", sigma



if(isnanvalue(ramean) .or. isnanvalue(decmean) .or. isnanvalue(sigma)) then

   do i = 1,  nData
      write(*, *) i, ") ra=", ra(i), " dec=", dec(i), " error=", error(i)

    end do
end if

end subroutine averageRADECs

!*****************************************************
!
!
!****************************************************
subroutine checkForErrors


    logical :: val

    call IEEE_GET_FLAG(IEEE_Overflow, val)
    if(val) write(*, *) "An IEEE overflow\s occurred somewhere"

    call IEEE_GET_FLAG(IEEE_Underflow, val)
    if(val) write(*, *) "An IEEE underflow\s occurred somewhere"

    call IEEE_GET_FLAG(IEEE_Divide_By_zero, val)
    if(val) write(*, *) "An IEEE divide-by-zero\s occurred somewhere"

    call IEEE_GET_FLAG(IEEE_Inexact, val)
    if(val) write(*, *) "An IEEE inexact\s occurred somewhere"


    call checkSrcListTable




!    call checkAveragedTable





end subroutine checkForErrors



!**********************************************************************
! Average the count rates
!*********************************************************************
subroutine AverageRealTableData(l, nomerged, istart, index, data, tabledataH, newtableDataH)
    integer, intent(in) :: l, nomerged

    integer, intent(inout) :: iStart
    real(single), pointer :: data(:, :)
    integer :: index(:)
    type(tableData), intent(inout) :: tabledataH, newtabledataH

    real(single) :: mean, Error

    integer :: k, m


!write(*, *) "Averagetabeldata l=", l, " nomerged=", nomerged, " istart=", istart
        
    do k = 1, noFilters
!        write(*, *) "Filter ", k
        do m = 1, l

            data(m, 1) = tableDataH%columnData(istart)%real32ArrayData( index(m) )   

            data(m, 2) = tableDataH%columnData(istart + 1)%real32ArrayData( index(m) )  
             
!            write(*, *) m, ") data=", data(m, 1), " error=", data(m, 2)
        end do   
!write(*, *) "Line 4237, l=", l, " size(data, 1)=", size(data, 1)

        call averageData(l, data(:, 1), data(:, 2), mean=mean, sigma=error)  

!write(*, *) "Line 4241 mean=", mean, " error=", error

if(mean < 0.0 .or. error < 0.0  ) then



write(*, *) "Mean=", mean, " error=", error
call fatal("bug at line 3693")
end if


        newtableDataH%columnData(istart)%real32ArrayData( nomerged ) = mean  



        istart = istart + 1
        newtableDataH%columnData(istart)%real32ArrayData( nomerged ) = error  
        istart = istart + 1

    end do 


end subroutine AverageRealTableData










!**********************************************************************
! Average the count rates
!*********************************************************************
subroutine AverageDoubleTableData(l, nomerged, istart, index, data, tabledataH, newtableDataH)
    integer, intent(in) :: l, nomerged

    integer, intent(inout) :: iStart
    real(double), pointer :: data(:, :)
    integer :: index(:)
    type(tableData), intent(inout) :: tabledataH, newtabledataH

    real(double) :: mean, Error

    integer :: k, m


!write(*, *) "Averagetabeldata l=", l, " nomerged=", nomerged, " istart=", istart
        
    do k = 1, noFilters
!        write(*, *) "Filter ", k
        do m = 1, l

            data(m, 1) = dble( tableDataH%columnData(istart)%real64ArrayData( index(m) ))   

            data(m, 2) = dble( tableDataH%columnData(istart + 1)%real64ArrayData( index(m) ) ) 
             

        end do   


        call averageData(l, data(:, 1), data(:, 2), mean, error)  



if(mean < 0.0 .or. error < 0.0  ) then



write(*, *) "Mean=", mean, " error=", error
call fatal("bug at line 3693")
end if


        newtableDataH%columnData(istart)%real32ArrayData( nomerged ) = mean 



        istart = istart + 1
        newtableDataH%columnData(istart)%real32ArrayData( nomerged ) = error  
        istart = istart + 1

    end do 


end subroutine AverageDoubleTableData








!**********************************************************************
! Compute the averaged AB fluxes from the averaged count-rates
! 
!
!*********************************************************************
subroutine ComputeAveragedABFluxes(l, nomerged, istart, tabledataH, newtableDataH)
    integer, intent(in) :: l, nomerged

    integer, intent(inout) :: iStart
 
   
    type(tableData), intent(inout) :: tabledataH, newtabledataH
    real(double) :: counts, countsError, flux, fluxError
  
    type(OmFluxConvF) :: cf

    integer :: i, k

    k = iStart - 2 * noFilters
        
    do i = 1, noFilters

        !******************************************************************
        ! Get the flux conversion constants
        !******************************************************************

        cf = getFluxConstants( OMFilter(i) )

        if ( .not. isNullValue(  newtableDataH%columnData(k)%real32ArrayData( nomerged ) ) )then


            !****************************************************************************
            ! Get the averaged counts
            !****************************************************************************/

            counts = dble( newtableDataH%columnData(k)%real32ArrayData( nomerged ) )

            countsError = dble( newtableDataH%columnData(k + 1)%real32ArrayData( nomerged ) )

            flux = counts * cf%cfab

            fluxError = countsError * cf%cfab

        else
            flux =   newtableDataH%columnData(k)%real32ArrayData( nomerged )
            fluxError = flux                  

        end if


        newtableDataH%columnData(istart)%real32ArrayData( nomerged ) = real(flux)  



        istart = istart + 1
        newtableDataH%columnData(istart)%real32ArrayData( nomerged ) = real(fluxError)  
        istart = istart + 1
        k = k + 2

    end do 


end subroutine ComputeAveragedABFluxes



!**********************************************************************
! Compute the averaged AB mags from the averaged count-rates
! 
!
!*********************************************************************
subroutine ComputeAveragedABMags(l, nomerged, istart, tabledataH, newtableDataH)
    integer, intent(in) :: l, nomerged

    integer, intent(inout) :: iStart
 
   
    type(tableData), intent(inout) :: tabledataH, newtabledataH
    real(double) :: counts, countsError, mag, magError
  
    type(OmFluxConvF) :: cf

    integer :: i, k

    k = iStart - 4 * noFilters
        
    do i = 1, noFilters
  



        if ( .not. isNullValue( newtableDataH%columnData(k)%real32ArrayData( nomerged )  ) ) then






            !******************************************************************
            ! Get the magnitude conversion constants
            !******************************************************************


            cf = getFluxConstants( OMFilter(i) )

            !****************************************************************************
            ! Get the averaged counts
            !****************************************************************************/

            counts = dble( newtableDataH%columnData(k)%real32ArrayData( nomerged ) )

            countsError = dble( newtableDataH%columnData(k + 1)%real32ArrayData( nomerged ) )
    
  



             mag = cf%zpab - 2.5 * log10(counts)

             magError = max(0.01_double, 1.085735 * countsError / counts)

        else
            mag = newtableDataH%columnData(k)%real32ArrayData( nomerged )    

            magError = mag                  

         end if


        newtableDataH%columnData(istart)%real32ArrayData( nomerged ) = real(mag)  

        istart = istart + 1
        newtableDataH%columnData(istart)%real32ArrayData( nomerged ) = real(magError)  
        istart = istart + 1
        k = k + 2

    end do 


end subroutine ComputeAveragedABMags


!****************************************************************
!
! If creating a UV-OPT catalogue then we only want
! to keep sources that have at least one UV-measurment
!
!***************************************************************
subroutine refineSourceList2( srclistfile )
    character(*), intent(in) ::  srclistfile

    type(TableT) :: TableH, newTableH
    type(DataSetT) :: dataSetH, newDataSetH 
    integer :: i, j, noUVFilters, nRows, nNewRows  
    character(30) :: name
    real(single), pointer :: data(:, :) => null()

    dataSetH = dataset( trim(srcListFile), MODIFY)
    tableH = Table(dataSetH, "SRCLIST")

    !***********************************************************************************
    ! Add the RA and DEC strings
    !**********************************************************************************
    nRows = numberOfRows( tableH )


    !*********************************************************************
    ! Loop through each table row (starting at the end so the row can
    ! be deleted if necessary
    !*********************************************************************
     

    !*************************************************
    ! Check that there is at least one UV filters
    !*************************************************
    noUVFilters = 0

! write(*, *) "Refinesourcelist2 file=", trim(srclistfile), " nrows=", nrows
    do i = 1, 3
        name = trim(omFilter(i)) // "_SIGNIFICANCE"
        if( hasColumn(tableH, trim(name) )) then
            
            noUVFilters = noUVFilters + 1
        end if 
    end do

    if(noUVFilters .eq. 0) then

        call warning("There are no UV observations in this file", trim(srcListFile)) 
        call release( dataSetH )
        stop   ! Can't do any more- no UV data

    end if

    data => allocate2dSingle( 1, nrows, 1, noUVFilters, "data" )


    noUVFilters = 0

    do i = 1, 3
        name = trim(omFilter(i)) // "_SIGNIFICANCE"
        if( hasColumn(tableH, trim(name) )) then
            
            noUVFilters = noUVFilters + 1



            !****************************************************************************
            ! Load the significances for this filter
            !
            !****************************************************************************
            call LoadReal32Column( tableH, name, data(:, noUVFilters) )
    
        end if 
    end do


    nNewRows = nRows
       
iLoop : do i = nRows, 1, -1

        do j = 1, noUVFilters         
            if ( .not. isnanvalue1( data(i, j) ) .and. data(i, j) .ge. 0.0) cycle iLoop ! Keep this row

        end do


        !*******************************************************************
        ! Delete this row from the table
        !******************************************************************* 

     
 
        nNewRows = nNewRows - 1

        !***************************************************************
        ! Get rid of this row
        !***************************************************************
        call deleteRows( tableH, i - 1, 1)

    end do iLoop   

    deallocate(data)

    call release(DataSetH)




    if(nNewRows .eq. 0) then

        call warning("There are no useful UV observations in this file", trim(srcListFile)) 
       
        stop ! Can't do anything else - no UV data
    end if




end subroutine refineSourceList2




  
!******************************************************************************
!
!
! For each source compute the distance to its nearest neighbour
!
!
!
!******************************************************************************
subroutine ComputeNearestNeighbourDistances( filter, n, ra, dec, distance, significance)
    character(*), intent(in) :: filter 
    integer(int32), intent(in) :: n
    real(double), pointer, dimension(:) :: ra, dec

    real(single), pointer, dimension(:) :: distance, significance
  
    integer(int32) :: i, j, dn, minIndex, nident, nGood
    real(double) :: radius, minRadius
    real(double), pointer, dimension(:) :: ra1 => null(), dec1 => null()
    character(100) :: messageString

  
    distance => allocateSingle(0, n - 1, "") 
    write(*,*) "L6266 allocated n-distances, n=", n

    if( .not. associated(significance) ) then
        distance = nullValue
        return
    end if

    ra1 => allocateDouble(0, n - 1, "") 
    dec1 => allocateDouble(0, n - 1, "") 
    
    if(n .eq. 1) then
        distance(0) = 0.0
        return
    end if

    radiansToArcSeconds = radiansToDegrees * 3600.0_double

    ra1 => allocateDouble(0, n - 1, "") 
    dec1 => allocateDouble(0, n - 1, "") 
  
    !--------------------------------------------------------------------
    ! 2013-08-14 increasing the number of neighbour sources for checking
    ! the distances from 10 to 100 (otherwise, for some UV filters it was
    ! never arriving to the index of the detected source, while the 
    ! nondetected sources, which were detected in other filters, occupued
    ! the whole list with NONE significance values)
    !---------------------------------------------------------------------
    !dn = max(10, n / 10)
    dn = max(100, n / 10)

    nIdent = 0

    do i = 0, n - 1
    
        ra1(i)  = ra(i) * degreesToRadians
        dec1(i) = dec(i) * degreesToRadians

    end do
  
!if(debug) write(*, *) "Line 5909"
    
    do i = 0, n  - 1
       
        if( isNullValue(significance(i) ) ) then
         
            distance(i) = nullValue 
            cycle
        end if
        minRadius = 1e9
        nGood = 0  ! Count how many neighbours checked    

        !write(*,*) "L6309 i=", i, " ................"
        do j = i - 1, max(0, i - dn), -1
           !write(*,*) "        L6311 j=", j
            if( isNullValue(significance(j) ) ) then
               !write(*,*) "      L6313 cycling "
                cycle
            end if
            nGood = nGood + 1
            !write(*,*) "      L6317 nGood= ", nGood
            !*****************************************************************************
            ! Compute distance between objects in arc secs
            !***************************************************************************** 
            radius = ArcLength( ra1(i), dec1(i), ra1(j), dec1(j) )
            !write(*,*) "         L6321 j=", j, " radius=", radius, " minRadius=", minRadius 
            if(radius .lt. minRadius) then
                minRadius = radius
            end if           
                

        end do  
        
        do j = i + 1, min(n - 1, i + dn)
           !write(*,*) "        L6331 j=", j
            if( isNullValue(significance(j) ) ) then
               !write(*,*) "      L6333 cycling "
                cycle
            end if
            nGood = nGood + 1
            !write(*,*) "      L6337 nGood= ", nGood
            !*****************************************************************************
            ! Compute distance between objects in arc secs
            !***************************************************************************** 
            radius = ArcLength( ra1(i), dec1(i), ra1(j), dec1(j) )  
     
            !write(*,*) "         L6341 j=", j, " radius=", radius, " minRadius=", minRadius 

            if(radius .lt. minRadius) then
                minRadius = radius
                minIndex = j
            end if           
     
        end do
        !write(*,*) "            L6349 nGood=", nGood
        if(nGood .gt. 0) then
            distance(i) = minRadius  ! The minimim distance between source i and  a neighbour
            if(distance(i) .gt. 30000.0) then

                call fatal("Bug in program at line 5858 (distance too large) ")
            end if

            !write(*,*) "        L6357 i=", i, " distance=", distance(i)

            if(distance(i) .lt. 1.0) then
                nIdent = nIdent + 1
                !write(*, *) "L6361 Source ", i, " ra=", ra1(i) * radiansToDegrees, &
                !                      " dec=", dec1(i) * radiansToDegrees
                !write(*, *) "L6363 Source ", minindex, " ra=", ra(minindex), " dec=", dec(minindex)
                !write(*, *) "L6364 distance=", distance(i), " arcsecs"

        end if   

        else
            distance(i) = nullValue

        end if

    end do
 
    !if(debug) write(*, *) "L5900 nIdent=", nIdent

    if(associated(ra1)) then 
        deallocate(ra1, dec1)
        nullify(ra1, dec1)
    end if
    messageString="No of sources with separations < 1 arc secs="
    if(nIdent .gt. 0) then
       !write(*,*) "L5885 nIdent=", nIdent
       write(*,*) "L588 possibly identical sources: number of sources with separations <1 arcsec= ", nIdent
       !write(errstr, '(a, i6)')  messageString, nIdent
       !write(errstr, '(a)')  messageString
       !call warning("L5889 Possible bug!", errstr) 
    end if

    !write(*, *) "L5914 DISTANCES"
    !write(*,*) "----------------"
    !do i = 0, n - 1
    !   write(*, *) "L5917 i=", i, ") distance=", distance(i)
    !end do

end subroutine ComputeNearestNeighbourDistances


!*****************************************************
!
!****************************************************
subroutine Peirce1(nData, data, error, mean, std)

    integer, intent(in) :: nData
    real(single), dimension(1 : nData) :: data(:)
    real(single), optional, dimension(1 : nData) :: error(:)
    real(single), intent(out) :: mean, std
    real(double), dimension(1 : ndata) :: ddata, derror
    real(double) :: dmean, dstd

    ddata = dble(data(1 : ndata))
    if(present(error)) then

        derror = dble(error(1 : ndata))
        call Peirce(nData, ddata, derror, dmean, dstd)


    else
        call Peirce2(nData, ddata, mean=dmean, std=dstd)

    end if


    mean = real(dmean)

    std = real(dstd)



end subroutine Peirce1
!*****************************************************
!
!****************************************************
subroutine Peirce2(nDataIn, dataIn, errorIn, mean, std)
    integer, intent(in) :: nDataIn
    real(double), dimension(1 : nDataIn) :: dataIn(:)
    real(double), optional, dimension(1 : nDataIn) :: errorIn(:)
    real(double), intent(out) :: mean, std


    integer, parameter :: maxNoObs = 30, maxNoDoubtfulObs = 9
    real(single), dimension(maxNoObs, maxNoDoubtFulObs) :: table
   
    real(double), pointer, dimension(:) :: tempData => null(), temperror => null()
    real(double) :: r, dev, maxDev, xs
    integer :: noRejects, i, j, k, iter, n, n1, ndata 

    

    tempData => allocateDouble(1, nDataIn, "tempData")


  
    nData = 0
    do i = 1, ndataIn

        if(.not. isnanvalue( dataIn(i))) then        
            ndata = nData + 1
           
        end if
    end do
 
    !***************************************************************************
    ! NB 1 to maxnoobs is the total number of observations
    ! NB 1 to 9 is the number of doubtful observations
    !***************************************************************************

    if(nData .lt. 3) then
 
        return
    end if


    if(present(errorIn)) then
        tempError => allocateDouble(1, nData, "tempError")
    end if
    nData = 0
    do i = 1, ndataIn

        if(.not. isnanvalue( dataIn(i))) then        
            ndata = nData + 1
            tempdata(nData) = dataIn(i)
            if(present(errorIn)) then 
                tempError(nData) = errorIn(i)
            end if
        end if
    end do
 

    
    table(3, 1 : 9) = (/1.196, 0.0,   0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0/)
    table(4, 1 : 9) = (/1.383, 1.078,   0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0/)
    table(5, 1 : 9) = (/1.509, 1.200,   0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0/)
    table(6, 1 : 9) = (/1.610, 1.299,   1.099, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0/)
    table(7, 1 : 9) = (/1.693, 1.382,   1.187, 1.022, 0.0, 0.0, 0.0, 0.0, 0.0/)
    table(8, 1 : 9) = (/1.763, 1.453,   1.261, 1.109, 0.0, 0.0, 0.0, 0.0, 0.0/)
    table(9, 1 : 9) = (/1.824, 1.515,   1.324, 1.178, 0.045, 0.0, 0.0, 0.0, 0.0/)
    table(10, 1 : 9) = (/1.878, 1.570,   1.380, 1.237, 1.114, 0.0, 0.0, 0.0, 0.0/)
    table(11, 1 : 9) = (/1.925, 1.619,   1.430, 1.289, 1.172, 1.059, 0.0, 0.0, 0.0/)
    table(12, 1 : 9) = (/1.969, 1.663,   1.475, 1.336, 1.221, 1.118, 1.009, 0.0, 0.0/)
    table(13, 1 : 9) = (/2.007, 1.704,   1.516, 1.379, 1.266, 1.167, 1.070, 0.0, 0.0/)
    table(14, 1 : 9) = (/2.043, 1.741,   1.554, 1.417, 1.307, 1.210, 1.120, 1.026, 0.0/)
    table(15, 1 : 9) = (/2.076, 1.775,   1.589, 1.453, 1.344, 1.249, 1.164, 1.078, 0.0/)
    table(16, 1 : 9) = (/2.106, 1.807,   1.622, 1.486, 1.378, 1.285, 1.202, 1.122, 1.039/)
    table(17, 1 : 9) = (/2.134, 1.836,   1.652, 1.517, 1.409, 1.318, 1.237, 1.161, 1.084/)
    table(18, 1 : 9) = (/2.161, 1.864,   1.680, 1.546, 1.438, 1.348, 1.268, 1.195, 1.123/)
    table(19, 1 : 9) = (/2.185, 1.890,   1.707, 1.573, 1.466, 1.377, 1.298, 1.226, 1.158/)
    table(20, 1 : 9) = (/2.209, 1.914,   1.732, 1.599, 1.492, 1.404, 1.326, 1.255, 1.190/)
    table(21, 1 : 9) = (/2.230, 1.938,   1.756, 1.623, 1.517, 1.429, 1.352, 1.282, 1.218/)
    table(22, 1 : 9) = (/2.251, 1.960,   1.779, 1.646, 1.540, 1.452, 1.376, 1.308, 1.245/)
    table(23, 1 : 9) = (/2.271, 1.981,   1.800, 1.668, 1.563, 1.475, 1.399, 1.332, 1.270/)
    table(24, 1 : 9) = (/2.290, 2.000,   1.821, 1.689, 1.584, 1.497, 1.421, 1.354, 1.293/)
    table(25, 1 : 9) = (/2.307, 2.019,   1.840, 1.709, 1.604, 1.517, 1.442, 1.375, 1.315/)
    table(26, 1 : 9) = (/2.324, 2.037,   1.859, 1.728, 1.624, 1.537, 1.462, 1.396, 1.336/)
    table(27, 1 : 9) = (/2.341, 2.055,   1.877, 1.746, 1.642, 1.556, 1.481, 1.415, 1.356/)
    table(28, 1 : 9) = (/2.356, 2.071,   1.894, 1.764, 1.660, 1.574, 1.500, 1.434, 1.375/)
    table(29, 1 : 9) = (/2.371, 2.088,   1.911, 1.781, 1.677, 1.591, 1.517, 1.452, 1.393/)
    table(30, 1 : 9) = (/2.385, 2.103,   1.927, 1.797, 1.694, 1.608, 1.534, 1.469, 1.411/)


  
    noRejects = 0
    n = ndata

    if(present(errorIn)) then
        call ComputeAverage(tempdata, temperror, n, mean, std)
    else
        call ComputeAverage(tempdata, n, mean, std)
    end if 


  
    do iter = 1, nData - 3

        if(n .lt. 3) exit 
        if(noRejects .eq. 9) exit       
         
        r = table( nData, noRejects + 1)
        maxDev = r * std
       
        n1 = n
        do j = 1, n
            dev = abs(mean - tempData(j))
            if (dev .gt. maxDev) then
                do k = j, n - 1 
                    tempData(k) = tempData(k + 1)
                    if(present(errorIn)) then
                        tempError(k) = tempError(k + 1)
                    end if
                end do
                n1 = n - 1
                exit
                
            end if 
        end do
      
        if(n1 .eq. n) exit
        n = n1
        noRejects = noRejects + 1

    end do

return   
if(noRejects .gt. 0) then




    write(*, *) "Initial n=", n, " mean=", mean, " std=", std, " ndatain=", ndataIn, &
         " size(dataIn)=", size(dataIn)





    write(*, *) "a15 mean=", a15(ndataIn, dataIn, xs), " hi5 mean=", h15(ndataIn, dataIn, xs)


    write(*, *) "Number rejected =", noRejects


    if(present(errorIn)) then
        call ComputeAverage(tempdata, tempError, n, mean, std)


do i = 1, ndataIn
write(*, *) i, ") data=", dataIn(i), " error=", errorIn(i)
end do


    else
        call ComputeAverage(tempdata, n, mean, std)



do i = 1, ndataIn
write(*, *) i, ") data=", dataIn(i)
end do



    end if 

    write(*, *) "Final n=", n, " mean=", mean, " std=", std

end if


    deallocate(tempData)
    if(present(errorIn)) deallocate(temperror)

end subroutine Peirce2



!*******************************************************************
!
!******************************************************************



subroutine  testPeirce

    integer :: ndata = 10
    real(single), dimension(10) :: data= (/101.2, 90.0, 99.0, 102.0, 103.0, 100.2, &
             89.0, 98.1, 101.5, 102.0/) 
    real(single) :: mean, std 
call Peirce(nData, data, mean=mean, std=std)


stop
end subroutine  testPeirce
!************************************************************************
! 
!************************************************************************/
subroutine checkAveragedTable

    character(100) :: fileName
    type(dataSetT) :: dataSetH
    type(tableT) :: tableH 
 
    integer(int32), pointer, dimension(:) :: srcNum1=>null(), srcNum2 => null() 
    integer(int32), pointer, dimension(:) :: nDetections=>null(), nAveraged => null() 

    integer(int32) :: nRows1, nRows2, i, j

    fileName = trim( stringparameter("newmastercatalogue") )

    write(errstr,'(a, a, a, a)') "Checking AV_MULTIPLE_DETECTION table in ", fileName
    call message(AppMsg, Verbose)

    dataSetH = dataSet( fileName, read)




    !*******************************************************************
    ! Get the data from the SRCLIST table
    !*******************************************************************

    tableH = table( dataSetH, "SRCLIST" , READ)

    nRows1 = numberOfRows(tableH)

    srcNum1 => allocateInt32(1, nRows1, "Srcnum1")
    call LoadInt32Column( tableH, "SRC_NUM", srcNum1 )



    nDetections => allocateInt32(1, nRows1, "nDetections")

    call LoadInt32Column( tableH, "N_OBSID", nDetections )
  

    call release( tableH )


    !*******************************************************************
    ! Get the data from the AV_MULTIPLE_DETECTIONS table
    !*******************************************************************


    tableH = table( dataSetH, "AV_N_OBSID" , READ)

    nRows2 = numberOfRows(tableH)




    srcNum2 => allocateInt32(1, nRows2, "Srcnum1")
    call LoadInt32Column( tableH, "SRC_NUM", srcNum2 )
  


    nAveraged => allocateInt32(1, nRows2, "nAveraged")
    call LoadInt32Column( tableH, "N_AVERAGED", nAveraged )
  


    call release(dataSetH)


    do i = 1, maxval(srcNum1)
        if( nDetections(i) .lt. 2) cycle
        do j = 1, nRows2
            if(srcNum1(i) .eq. srcNum2(j)) then

                if(nDetections(i) .ne. nAveraged(j)) then
                    write(*, *) i, ") srcnum=", srcNum1(i), " ndetections", nDetections(i), &
                        " naveraged=", naveraged(j) 
                    call fatal("bug at line 4779")

                end if
             end if
         end do
    end do


 

end subroutine checkAveragedTable



!************************************************************************
! 
!************************************************************************/
subroutine checkSrcListTable

    character(100) :: fileName
    type(dataSetT) :: dataSetH
    type(tableT) :: tableH 
 
    real(double), pointer, dimension(:) :: ra, dec
    real(double) :: deltaRa
    integer(int32) :: nRows, i, j, count

    fileName = trim( stringparameter("newmastercatalogue") )

    write(errstr,'(a, a, a, a)') "Checking SRCLIST table in ", fileName
    call message(AppMsg, Verbose)

    dataSetH = dataSet( fileName, read)


    !*******************************************************************
    ! Get the data from the SRCLIST table
    !*******************************************************************

    tableH = table( dataSetH, "SRCLIST" , READ)

    nRows = numberOfRows(tableH)

    ra => allocateDouble(1, nRows, "ra")
    call Loadreal64Column( tableH, "RA", ra )

    dec => allocateDouble(1, nRows, "dec")
    call Loadreal64Column( tableH, "DEC", dec )


    count = 0
    do i = 1, nRows

        do j = i - 1, 1, -1
            deltaRa = abs(ra(i) - ra(j)) * 3600.0
            if(deltaRa .gt. 1.0) exit
             

            if( deltaRa .lt. 0.01 .and. &
                abs(dec(i) - dec(j)) .lt. 0.01) then
                count = count + 1   

            end if 


        end do

        do j = i + 1, nRows
            deltaRa = abs(ra(i) - ra(j)) * 3600.0
            if(deltaRa .gt. 1.0) exit
             

            if( deltaRa .lt. 0.01 .and. &
                abs(dec(i) - dec(j)) .lt. 0.01) then
                count = count + 1   

            end if 


        end do


    end do


    call release( dataSetH )



    write(errstr, *) "No of sources with identical coordinates=", count         
     
    call message(AppMsg, Verbose)






end subroutine checkSrcListTable


!*********************************************************************************
! Converts the flags into a string
!
!********************************************************************************
subroutine convertFlagsToStrings



    call convertFlagsToStrings1( "SRCLIST" )

!    call convertFlagsToStrings1( "AV_MULTIPLE_DETECTIONS" )


end subroutine convertFlagsToStrings

!*********************************************************************************
! Converts the flags into a string
!
!********************************************************************************
subroutine convertFlagsToStrings1(tableName)
    character(*), intent(in) :: tableName


    character(20), pointer :: flagText(:) => null()
    integer(int32) :: position, i, j, k, nRows
    character(100) :: fileName
    type(dataSetT) :: dataSetH
    type(tableT) :: tableH
    integer(int16), pointer :: qFlag(:) => null()
    integer(int16) :: noQFlags 
    character(100) :: name, text, comment

    fileName = trim( stringparameter("newmastercatalogue") )

    write(errstr,'(a, a, a, a)') "Checking SRCLIST table in ", fileName
    call message(AppMsg, Verbose)

    dataSetH = dataSet( fileName, MODIFY)


    !*******************************************************************
    ! Get the data from the SRCLIST table
    !*******************************************************************

    tableH = table( dataSetH, trim(tableName) , MODIFY)

    nRows = numberOfRows(tableH)

    qFlag => allocateInt16(1, nRows, "qflag")



    !***************************************************************
    !
    !***************************************************************


  
    noQFlags = getNoQflags()

    !************************************************************************
    ! Convert into a a text array
    !************************************************************************
    allocate(flagText(1 : nRows))



    !**********************************************************
    ! Loop through all the filters
    !*********************************************************
    position = columnNumber(tableH, "V_QUALITY_FLAG") + 1

    do k = 1, noFilters
        name = trim(omFilter(k)) // "_QUALITY_FLAG"
 
        call LoadInt16Column( tableH, trim(name), qFlag )

        flagText = "       NULL"
        do i = 1, nRows
            if ( isNullValue4( qflag(i) )) cycle
            do j = 1, noQFlags
                if ( btest( qFlag(i), j - 1)) then
                    flagText(i)(j : j) = 'T'
                else
                    flagText(i)(j : j) = 'F'
                end if
            end do
        end do
        name = trim(omFilter(k)) // "_QUALITY_FLAG_ST"
 
        comment = "Quality flag (FFFFFFFFFF=good) (" // trim(omFilter(k)) // " filter)"       
        !***************************************************
        ! 
        !***************************************************
        call AddCharColumn(tableH, trim(name), "", trim(comment), &
            nRows, flagtext, position=position)
        position = position + 1



    end do


!    call addcomment(tableH, "QFLAG_S F=flag not set, T=flag set")

    do i = 1, noQFlags
        text = getqFlagText( i )
        call addcomment( tableH, text(1 : index( text, "(" )  - 1) )
    end do




    call release(dataSetH)

    deallocate(flagText)

end subroutine convertFlagsToStrings1
!**********************************************************
!
!*********************************************************/
subroutine rectifyCoordinates



    integer(int32) :: position, i, nRows
    character(100) :: fileName
    type(dataSetT) :: dataSetH
    type(tableT) :: tableH
    real(double) :: longitude, latitude
    
    type(Columnt) :: columnH1, columnH2  
    real(double), pointer, dimension(:) :: ra, dec, data1, data2








    write(errstr,'(a)') "Correcting coordinates"
    call message(AppMsg, Verbose)




    fileName = trim( stringparameter("newmastercatalogue") )

 

    dataSetH = dataSet( fileName, MODIFY)


    !*******************************************************************
    ! Get the data from the SRCLIST table
    !*******************************************************************

    tableH = table( dataSetH, "SRCLIST" , MODIFY)

    nRows = numberOfRows(tableH)

 

    ra => allocateDouble(1, nRows, "RA")
    dec => allocateDouble(1, nRows, "DEC")

    if( .not. hasColumn(tableH, "RA_CORR")) then
        call release(dataseth)        
        call fatal("RA_CORR column does not exist")
    end if
   
    if( .not. hasColumn(tableH, "DEC_CORR")) then
        call release(dataseth)        
    
        call fatal("RA_CORR column does not exist")
    end if
   


    call Loadreal64Column( tableH, "RA_CORR", ra )
    call Loadreal64Column( tableH, "DEC_CORR", dec )


    columnH1 = Column( tableH, "LII", modify)
    columnH2 = Column( tableH, "BII", modify)

   

    data1 => real64Data( columnH1 )
    data2 => real64Data( columnH2 )




    do i = 1, nrows
        call sla_EQGAL( ra(i) * degreesToRadians, dec(i) * degreesToRadians, longitude, latitude )
        data1(i - 1) = longitude * radiansToDegrees
        data2(i - 1) = latitude * radiansToDegrees
    end do




    call release(columnH1)

    call release(columnH2)

    call release(dataSetH)

    deallocate(ra, dec)

end subroutine rectifyCoordinates




!*************************************************************************************
! For a given OM filter, this function returns a structure containing :
! 1) The magnitude zer0-point
! 2) The conversion factor from count-rate to flux
!
!
!*************************************************************************************
type(OmFluxConvF) function getFluxConstants( filter )
    character(*), intent(in) :: filter



    call CAL_setState(OM, filterId = filterIdToInt( trim(filter)) )


    getFluxConstants = cal_omgetfluxconvfactors()
    

end function getFluxConstants
!********************************************************************************
!
!*******************************************************************************

subroutine modifyPositionErrors(tableName, columnName)
    character(*), intent(in) :: tableName, columnName
    character(100) :: fileName
    integer :: i, nRows
    type(dataSetT) :: dataSetH
    type(TableT) :: tableH
    type(ColumnT) :: columnH
    real(single), pointer :: data(:) => null()
    real(single) :: error
    real(single), parameter :: distortionError = 0.7  ! Distortion error
   
    write(*, *) "Modifying position errors in column ", trim(columnName), &
        " in table ", trim(tableName)     

    fileName = trim( stringparameter("newmastercatalogue") )

 
    dataSetH = dataSet( fileName, MODIFY)

    tableH = table( dataSetH, trim(tableName) , MODIFY)

    nRows = numberOfRows(tableH)

 
    columnH = Column( tableH, trim(columnName), modify)
   
   
    data => real32Data( columnH )
 
    write(*, *) "Initial Minimum error=", minval(data), " max=", maxval(data)


    do i = 0, nrows - 1
        error = sqrt(data(i) * data(i) + distortionError * distortionError)        
        data(i) = error
    end do

    write(*, *) "Final Minimum error=", minval(data), " max=", maxval(data)


    call release(columnH)
    call release(dataSetH)


end subroutine modifyPositionErrors
!***************************************************************************
! 
!
!***************************************************************************
subroutine AddOBSIDColumn
   
    character(100) :: fileName
    
    type(dataSetT) :: dataSetH
    type(TableT) :: srcListTableH, summaryTableH
    
    character(10), pointer, dimension(:) :: summaryObsId => null(), obsId => null()
    integer, pointer, dimension(:) :: no => null(), summaryNo => null()
    integer :: i, size, size1, index, j
  
   

    fileName = trim( stringparameter("newmastercatalogue") )



    write(errstr,'(a, a)') "Adding OBSID Column to SRCLIST table in file ", trim(fileName)
    call message(AppMsg, Verbose)






    dataSetH = dataSet( fileName, MODIFY)


    summaryTableH = table( dataSetH, "SUMMARY" , read)

    size = numberOfRows( summaryTableH )
    summaryObsId => allocateChar(10, size, "summaryobsId" )


    !****************************************************************
    ! Load the OBS IDS from the summary table
    !****************************************************************

    call readCharColumn( summaryTableH, "OBSID", size, 0, summaryObsId)
    no => allocateInt32(1, size, "no")

    call LoadInt32Column( summaryTableH, "N_SUMMARY", no)
  


    !******************************************************************
    ! Add the OBS IDs to the SRCLIST table
    !*******************************************************************

 

    srcListtableH = table( dataSetH, "SRCLIST" , MODIFY)
    size1 = numberOfRows( srcListTableH )


    summaryNo => allocateInt32(1, size1, "no")


    call LoadInt32Column( srcListTableH, "N_SUMMARY", summaryNo)

    obsId => allocateChar(10, size1, "obsId" )

    do i = 1, size1
   
        index = -1
        do j = 1, size ! Loop through all the entries in the Summary Table
            if( no(j) .eq. summaryNo(i)) then
                index = j
                exit
            end if

        end do
        obsId( i ) = summaryObsId( index ) 
    end do
 

    call AddCharColumn( srcListTableH, "OBSID", "", "Observation ID", size1, obsId, 1)

    call release(dataSetH)


end subroutine AddOBSIDColumn

!********************************************************
! 
!********************************************************
subroutine renameColumns10

    character(100) :: fileName
    
    type(dataSetT) :: dataSetH
    type(TableT) :: tableH
    type(ColumnT) :: columnH
   

    fileName = trim( stringparameter("newmastercatalogue") )


    dataSetH = dataSet( fileName, MODIFY)
    tableH = table(dataSetH, "SRCLIST", MODIFY)

    columnH = column(tableH, "one", MODIFY)

 !   call setComment(columnH, "test")

    call release(dataSetH)


end subroutine renameColumns10




!*******************************************************************
! 
!
!
!
!
!******************************************************************
subroutine createFinalCatalogue(catalogueName, finalCatalogueName)
    character(*), intent(in) :: catalogueName, finalCatalogueName
    type(DataSetT) :: dataSet1, dataSet2
    type(BlockT) :: blockh
    integer :: i
    
    write(errstr,'(a, a)') "Creating final catalogue-", trim(finalCatalogueName)
    call message(AppMsg, Verbose)
  
    dataSet1 = dataSet( trim(catalogueName), read)

    !*********************************************************
    !
    ! The final catalogue - create from scratch
    !
    !*********************************************************    
    dataSet2 = dataSet( trim(finalCatalogueName), create)
    
    call copyTable( dataSet1, dataSet2, "SRCLIST")
    call copyTable( dataSet1, dataSet2, "SUMMARY")

 !   do i = 0, numberOfBlocks( dataSet1 ) - 1
 !       blockH = block( dataSet1, i, READ )
 !       write(errstr,'(i1, a, a, a)') i, ") Copying block ", name(blockH), " to final catalogue"
 !       call message(AppMsg, Verbose)

 !       call copyBlock( dataSet2, blockH )
  
 !   end do


    call release( dataSet1)

    !*************************************************************************
    ! Add the comments to the final catalogue
    !*************************************************************************
    call addComment(dataSet2, "This file contains the XMM-NEWTON Optical Monitor (OM) catalogue ")
    call addComment(dataSet2, "of sources. The catalogue was compiled by using the  XMM-Newton ")
    call addComment(dataSet2, "Science Analysis Software (SAS) pipeline products at the European")
    call addComment(dataSet2, "Space Astronomy Centre (ESAC) and Univerity College London's ")
    call addComment(dataSet2, "Mullard Space Science Laboratory (MSSL) for the XMM observations")
    call addComment(dataSet2, "made from 2000 to 2011. Feedback to V.Yershov (vny@mssl.ucl.ac.uk)")
 
 


    call addVegaMags( dataSet2 )

    call release( dataSet2)


    !call fatal("stopped at line at 6622 after creating final catalogue")

    call checkFlags( finalCatalogueName )

 

end subroutine createFinalCatalogue
!********************************************************************
! Subroutine checkFlags
!********************************************************************
subroutine checkFlags( file )
    character(*), intent(in) :: file
    type(dataSetT) :: dataSetH
    type(TableT) :: tableH
    integer(int8), pointer :: int8Data(:) => null()

    integer(int16), pointer :: int16Data(:) => null()

    integer :: i, j, k, nRows
    character(100) :: name

  return


    dataSetH = dataSet( file, read)


    write(errstr,'(a, a)') "Checking flags in file ", file
    call message(AppMsg, Verbose)



    !*******************************************************************
    ! Get the data from the SRCLIST table
    !*******************************************************************

    tableH = table( dataSetH, "SRCLIST" , READ)

    nRows = numberOfRows( tableH )
    int8Data => allocateInt8(1, nRows, "int8Data")
    int16Data => allocateInt16(1, nRows, "int16Data")

    do i = 1, noFilters



        name = "QFLAG_" // trim(omFilter(i))
        if( hasColumn(tableH, trim(name) )) then ! Check this column

           



            call LoadInt16Column( tableH, trim(name), int16Data )
            do j = 1, nRows
               if ( .not. isNullValue4( int16Data(j) )) then
                  if(int16Data(j) > 10000_int16) then
                     !write(errstr,'(a, a, a)') "Checking column ", trim(name), " -not okay"
                     !call message(AppMsg, Verbose)


                     write(*, *) j, ") ", int16Data(j)
                     write(*, *) "All the data for this column" 
                     do k = 1, nRows
                        write(*, *) k, ") ", int16Data(k)
                    

                     end do
                     call fatal("Bug with int16 values " // " column name-" // trim(name)) 
                  end if
                  
               end if
            end do
            write(errstr,'(a, a, a)') "Checking column ", trim(name), " -okay"
            call message(AppMsg, Verbose)
            
 
         end if





        name = "CFLAG_" // trim(omFilter(i))
        if( hasColumn(tableH, trim(name) )) then ! Check this column

           



            call LoadInt8Column( tableH, trim(name), int8Data )

            do j = 1, nRows
                if ( .not. isNullValue3( int8Data(j) )) then
                    if(int8Data(j) > 100_int8) then
                    write(errstr,'(a, a, a)') "Checking column ", trim(name), " -not okay"
                    call message(AppMsg, Verbose)


                        write(*, *) j, ") ", int8Data(j)
                        write(*, *) "All the data for this column" 
                        do k = 1, nRows
                            write(*, *) k, ") ", int8Data(k)
                    

                        end do 
                 

                        call fatal("Bug with int8 values " // " column name-" // trim(name)) 
             
                    end if

                end if
            end do
            write(errstr,'(a, a, a)') "Checking column ", trim(name), " -okay"
            call message(AppMsg, Verbose)

 

        end if

        name = "EFLAG_" // trim(omFilter(i))
        if( hasColumn(tableH, trim(name) )) then ! Check this column

           call LoadInt8Column( tableH, trim(name), int8Data )


            do j = 1, nRows
                if ( .not. isNullValue3( int8Data(j) )) then
                    if(int8Data(j) > 100_int8) then
                    write(errstr,'(a, a, a)') "Checking column ", trim(name), " -not okay"
                    call message(AppMsg, Verbose)

 


                        write(*, *) j, ") ", int8Data(j)
 
                       write(*, *) "All the data for this column" 
                        do k = 1, nRows
                            write(*, *) k, ") ", int8Data(k)
                    

                        end do 
                 

                        call fatal("Bug with int8 values " // " column name-" // trim(name)) 
                    end if

                end if
            end do

            write(errstr,'(a, a, a)') "Checking column ", trim(name), " -okay"
            call message(AppMsg, Verbose)

 




        end if

    end do
    call release( dataSetH)

    deallocate(int8Data, int16Data)

end subroutine checkFlags

!********************************************************************
!
!*******************************************************************
logical Function isNullValue3(x)

    integer(int8), intent(in) :: x
    character(100) :: text

    write(text, '(i6)') x
    
    if(index(text, "NULL") .ne. 0 .or. &
       index(text, "Null") .ne. 0 .or. &
       index(text, "null") .ne. 0 .or. &
       index(text, "NAN") .ne. 0 .or. &
       index(text, "NaN") .ne. 0 .or. &
       index(text, "nan") .ne. 0) then

       isNullValue3 = .true.
    else
       isNullValue3 = .false.
    
    end if

!    write(*, *) "isnullvalue3 x=", x, " text=", trim(text)
 


end function IsNullValue3

!********************************************************************
!
!*******************************************************************
logical Function isNullValue4(x)
    integer(int16), intent(in) :: x
  


   character(100) :: text

    write(text, '(i6)') x

!write(*, *) "x=", x, " text=", trim(text)   
    if(x == int16NullValue .or.        &
       index(text, "NULL") .ne. 0 .or. &
       index(text, "Null") .ne. 0 .or. &
       index(text, "null") .ne. 0 .or. &
       index(text, "NAN") .ne. 0 .or. &
       index(text, "NaN") .ne. 0 .or. &
       index(text, "nan") .ne. 0) then

       isNullValue4 = .true.
    else
       isNullValue4 = .false.
    
    end if
!    write(*, *) "isnullvalue4 x=", x, " text=", trim(text)
 





end function IsNullValue4







!******************************************************************
!
!
!*****************************************************************


subroutine copyTable(sourceDataSet, destDataSet, tableName)

    type(DataSetT) :: sourceDataSet, destDataSet
    type(TableT) :: sourceTable, destTable
    type(columnT) :: sourceCol, destCol   

    integer :: nrows, ncols, i, j
    character(len=20) :: colName
    
    character(len=*) :: tableName
   
  
    integer(int8),  pointer  :: temp_i8(:) => null()
    integer(int16), pointer  :: temp_i16(:) => null()
    integer(int32), pointer  :: temp_i32(:) => null()
    real(single),  pointer  :: temp_real32(:) => null()
    real(double),  pointer  :: temp_real64(:) => null()

    

    integer :: colType

 
    sourceTable = table(sourceDataSet,tableName)
    
    nrows = numberOfRows(sourceTable)
    ncols = numberOfColumns(sourceTable)
    
    destTable = addTable(destDataSet, tableName, nrows)

   
!if(debug) write(*, *) "CopyTable line 7095"
    
    do i = 0, ncols-1
       
       sourceCol = column(sourceTable, i, read)
       colType = columnDataType(sourceCol)
       colName = name(sourceCol)


      

       !write(errstr,'(a, a)') "Adding column ", colName
       !call message(AppMsg, Verbose)




       call copyColumn( destTable, sourceCol, colName )

  
       call release(sourceCol)   

       colType = columnDataType( sourceCol )


    
 
       !******************************************************
       ! Need to reset null values due to a dal problem
       !**********************************************************

       select case( colType )     



       case( Integer8 ) 
         
         
             
          destCol =  column( destTable, colName, modify)
   
          call setNullValue( destCol, int8NullValue)
         
          temp_i8 => int8Data(destCol)
       
          do j = 0, nrows - 1
          
              if( isNull(destCol, j) ) then
                  call setToNull( destCol, j)

              end if
          end do         
      


          call release(destCol)
         
    
    
       case( Integer16 ) 
             
          destCol =  column( destTable, colName, modify)
   

          call setNullValue( destCol, int16NullValue)
      
          temp_i16 => int16Data(destCol)
          do j = 0, nrows - 1
          
              if( isNull(destCol, j) ) then
                  call setToNull( destCol, j)

              end if
          end do         
          call release(destCol)
       
    
       case( Integer32 ) 
             
          destCol =  column( destTable, colName, modify)
   
          call setNullValue( destCol, int32NullValue)
      

          temp_i32 => int32Data(destCol)
      
          do j = 0, nrows - 1
          
              if( isNull(destCol, j) ) then
                  call setToNull( destCol, j)

              end if
          end do         
      
 
          call release(destCol)
       
    
 
      case( Real32 ) 
             
          destCol =  column( destTable, colName, modify)
   
       

      

          temp_real32 => real32Data(destCol)
      
          do j = 0, nrows - 1
          
              if( isNull(destCol, j) .or. abs(9999.0 + temp_real32(j)) < 1.0 ) then
                  call setToNull( destCol, j)

              end if
          end do         
      
 
          call release(destCol)
       
    




      case( Real64 ) 
             
          destCol =  column( destTable, colName, modify)
   
       
      

          temp_real64 => real64Data(destCol)
      
          do j = 0, nrows - 1
          
              
              if( isNull(destCol, j) .or. abs(9999.0 + temp_real64(j)) < 1.0 ) then
        
                  call setToNull( destCol, j)

              end if
          end do         
      
 
          call release(destCol)
       
    



    


       case default
 
          
    
          
       end select

    end do

    call copyAllAttributes(attributable(sourceTable), attributable(destTable))
    call release(destTable)
    call release(sourceTable)
    
end subroutine copyTable
!*******************************************************************
!
!******************************************************************  


logical function obsIdExists(tableH, obsId)
    type(TableT), intent(In) :: tableH
    character(*), intent(in) :: obsId
    type(ColumnT) :: columnH
    integer :: i




    columnH = column(tableH, "OBS_ID", read)
   
  
    do i = 0 , numberOfRows( tableH) - 1
      
       if( index( trim( stringcell( columnH, i)), trim(obsId) ) > 0) then
            obsIdExists = .true.
            call release( columnH )

            return
       end if

    enddo
    call release( columnH )


    obsIdExists =  .false.


end function obsIdExists
!***********************************************************************
! addVegaMags

!***********************************************************************
subroutine addVegaMags( dataSetH )
    type(dataSetT), intent(in) :: dataSetH
    type(tableT) :: tableH

    tableH = table(dataSetH, "SRCLIST", modify)





    call release(tableH)





end subroutine addVegaMags
!***************************************************************
! Replace the comments from omsrclistcomb with the new ones
!
!
!***************************************************************
subroutine modifyColumns(file)
    character(*), intent(in) :: file
    type(DataSetT) :: dataSetH
   

    write(errstr,'(a, a, a, a)') "Modifying columns in ", trim(file)
    call message(AppMsg, Verbose)



    !if(debug) write(*, *) "L7374 (modifyColumns): setting up the filters"

    !********************************************************
    !
    !
    !*******************************************************
    call setUpFilters()

    !********************************************************************
    !
    !*********************************************************************

    call setUpColumnNames


    dataSetH = dataSet(trim(file), modify)


    !************************************************************************
    ! Modify the columns in the SRCLIST table
    !************************************************************************
    call modifytheColumns(dataSetH, "SRCLIST", noColumns1, columnData1)

    call release(dataSetH)


    dataSetH = dataSet(trim(file), modify)

    !************************************************************************
    !  Modify the columns in the SUMMARY table
    !************************************************************************
    call modifytheColumns(dataSetH, "SUMMARY", noColumns3, columnData3)




    call release( dataSetH )

end subroutine modifyColumns
!***************************************************************
!
!
!***************************************************************
subroutine modifyTheColumns( dataSetH, tableName, noColumns, columnData )

    type(dataSetT), intent(inout) :: dataSetH
    character(*), intent(in) :: tableName
   
    integer(int16), intent(in) :: noColumns
    type(newColumn), intent(in) :: columnData(:)

    type(tableT) :: oldTableH,  newTableH
    type(ColumnT) :: columnH, newColumnH
    integer :: i, position, type
    character(100) :: colName, newColName, comment
    

    oldTableH = table(dataSetH, trim(tableName), MODIFY)

    !write(*, *) "Line 7396 modifythe columns"   

    do i = 1, noColumns
        colName = trim(columnData(i)%name)
        !write(*, *) "Checking column ", trim(colName)

        if ( hasColumn( oldTableH, colName) ) then

            !**************************************************************
            ! Ignore string column names because of a bug
            !**************************************************************
        
            if ( dataType( Column(oldTableH, colName) ) == string) cycle
            newColName = trim(columnData(i)%newname)

            !write(errstr,'(a, a, a, a)') "Changing column ", trim(colName), " to ", trim(newColName)
            !call message(AppMsg, Verbose)

  
            position = columnNumber( oldTableH, colName)
            columnH = column(oldTableH, colName)

        
            call copyColumn( oldTableH, columnH, "TEMPCOLUMN")
            columnH = column(oldTableH, "TEMPCOLUMN")

             
            call deleteColumn( oldTableH, colName)
    !        write(*, *) i, ") old column name=", trim(colName), " new=", trim(newColName)

            newColumnH =  addColumn( oldtableH, trim( newcolName ) , &
                       columndata(i)%type, units = trim( columndata(i)%units), &
                       comment = trim( columndata(i)%comment), position=position )

            call loadData(columnh, newColumnH)

            call release(columnh)
            call release(newColumnH)
            call deleteColumn( oldTableH, "TEMPCOLUMN")

               


       
!            write(*, *) i, " ", trim(colname), " units=",  trim(columnData(i)%units), &
!                 " comment=", trim(columnData(i)%comment)
        end if

    end do

    call release(oldTableH)    
  
    write(errstr,'(a, a, a, a)') "Finished modifying the columns"
    call message(AppMsg, Verbose)


 


end subroutine modifyTheColumns 
!*****************************************************************************
!
!
!****************************************************************************
subroutine loaddata( columnH, newColumnH )
    type(columnT), intent(in) :: columnH
    type(columnT), intent(inout) :: newColumnH
    real(single), pointer :: real32DataOld(:),  real32DataNew(:) 
    real(double), pointer :: real64DataOld(:),  real64DataNew(:) 

    integer(int8), pointer :: integer8DataOld(:),  integer8DataNew(:) 
    integer(int16), pointer :: integer16DataOld(:),  integer16DataNew(:) 
    integer(int32), pointer :: integer32DataOld(:),  integer32DataNew(:) 
    integer :: i

    select case ( dataType(columnH) )

        case (real32) ! (1)

            real32DataOld => real32Data( columnH )
      
            real32DataNew => real32Data( newcolumnH )
            real32DataNew = real32DataOld 
            
          

        case (real64)  ! (2)

            real64DataOld => real64Data( columnH )
      
            real64DataNew => real64Data( newcolumnH )
            real64DataNew = real64DataOld 
            
       


      

        case (integer8)  ! (3)
              

            integer8DataOld => int8Data( columnH )
      
            integer8DataNew => int8Data( newcolumnH )
            integer8DataNew = integer8DataOld 
            
        
        
        
 
        case (integer16) ! (4)
            
            integer16DataOld => int16Data( columnH )
      
            integer16DataNew => int16Data( newcolumnH )
            integer16DataNew = integer16DataOld 
       

                

        case (integer32)  ! (5)

            integer32DataOld => int32Data( columnH )
      
            integer32DataNew => int32Data( newcolumnH )
            integer32DataNew = integer32DataOld 
       


        case ( string )   ! (6)

           do i = 0, numberOfRows(columnH) - 1
               call setStringCell( newColumnh,  i, stringCell( columnh, i) )
               if(i .lt. 10) then
                   write(*, *) i, ") old string=", trim(stringcell(columnH, i)), &
                  " copied string=", trim(stringcell(newcolumnH, i))
                                    
             
               end if


           end do 

               
    end select

                      
    
end subroutine loaddata
!***********************************************************
!
!
!************************************************************
subroutine addIAUCoordinates( catFile )

    character(*), intent(in) :: catFile

    type(dataSetT) :: dataSetH
    type(TableT)   :: tableH  
    real(double), pointer, dimension(:) :: ra => null(), dec => null()
    integer :: nRows, i, position
    character(30), pointer :: string(:) => null()

    write(*, *) "L7950 Adding IAU column to ", trim(catFile)

    dataSetH = dataSet(trim(catfile), modify)


    tableH = table( dataSetH, "SRCLIST" , MODIFY)

    nRows = numberOfRows(tableH)

    allocate(string(1 : nRows))

    !**********************************************************
    ! 
    !
    !*********************************************************

    ra => allocateDouble(1, nRows, "ra")
    call ReadReal64Column( tableH, "RA", nrows, 0, ra)



    dec => allocateDouble(1, nRows, "ra")    
    call ReadReal64Column( tableH, "DEC", nrows, 0, dec)
 
    write(*, *) "Creating column entries"

    do i = 1, nrows

        call CreateIAUString(ra(i), dec(i), string(i))


    end do


    position = columnNumber( tableH, "RA")



    position = 0

    write(*, *) "Adding column IAUNAME at position ", position
    call AddCharColumn(tableH, "IAUNAME", " ", "IAU designation" , nRows, &
                string, position)
 
   


    call release( dataSetH )

    deallocate(ra, dec, string)
    nullify(ra, dec, string)


end subroutine addIAUCoordinates



!***************************************************************************************
!
!
!**************************************************************************************
subroutine CreateIAUString(ra, dec, string)

    real(double), intent(in) :: ra, dec
    character(*), intent(out) :: string
    integer :: r1, r2, d1, d2
    real(single) :: r3, d3
    character(10) :: s1, s2, s3
   
    call ConvertRA(ra, r1, r2, r3)

    !******************************************
    ! Create the RA string
    !
    !******************************************
    if (r1 < 10) then    ! Hours
        write(s1, '(a1, i1)')  "0", r1
    else
        write(s1, '(i2)') r1
    end if 

    if (r2 < 10) then    ! Minutes
        write(s2, '(a1, i1)')  "0", r2
    else
        write(s2, '(i2)') r2
    end if 

    if (r3 < 9.95) then    ! Seconds
        write(s3, '(a1, f3.1)')  "0", r3
    else
        write(s3, '(f4.1)') r3
    end if 
                       

    string = "XMMOM J" // trim(s1) // trim(s2) // trim(s3) 

    !******************************************
    !
    ! Create the dec string
    !
    !******************************************
    call ConvertDec(dec, d1, d2, d3)
    if (d1 < 0 .or. d2 < 0 .or. d3 < 0.0) then
        if (d1 > -10) then   ! Degrees
            write(s1, '(a2, i1)') "-0", abs(d1)
        else 
                write(s1, '(a1, i2)') "-", abs(d1)
        end if
    else
        if (d1 < 10) then
            write(s1, '(a2, i1)') "+0", d1
        else 
            write(s1, '(a1, i2)') "+", d1
        end if
    end if

    d2 = abs(d2)   ! Minutes
    if (d2 < 10) then
        write(s2, '(a1, i1)') "0", d2
    else 
        write(s2, '(i2)') d2
    end if
          
    d3 = abs(d3)     ! Seconds          
    if ( d3 < 10.0) then
        write(s3, '(a1, i1)') "0", int( d3 )  !
    else 
        write(s3, '(i2)') nint(d3)
    end if
           
             
    string  = trim(string) // trim(s1) // trim(s2) // trim(s3)          
          
end subroutine CreateIAUString



end module ommastercatalogue_mod
